@@ -3,7 +3,8 @@
 # testing from a 'correct' interaction between exchange,
 # wallet and merchant.
 #
-# Creates $BASEDB.sql, $BASEDB.fees and $BASEDB.mpub.
+# Creates $BASEDB.sql, $BASEDB.fees, $BASEDB.mpub and
+# $BASEDB.age.
 # Default $BASEDB is "auditor-basedb", override via $1.
 #
 # Currently must be run online as it interacts with
@@ -14,6 +15,8 @@
 #
 set -eu
 
+trap "kill `jobs -p` &> /dev/null || true" ERR
+
 # Exit, with status code "skip" (no 'real' failure)
 function exit_skip() {
     echo $1
@@ -28,10 +31,6 @@ BASEDB=${1:-"auditor-basedb"}
 # elsewhere
 TARGET_DB=taler-auditor-basedb
 
-# FIXME: try to generate DB from scratch, fall back
-# to pre-generated DB if generate-auditor-basedb.sh
-# fails with status code 77!
-
 # Configuation file will be edited, so we create one
 # from the template.
 CONF=generate-auditor-basedb-prod.conf
@@ -69,8 +68,8 @@ BANK_URL=http://localhost:${BANK_PORT}/
 AUDITOR_URL=http://localhost:8083/
 
 # patch configuration
-taler-config -c $CONF -s EXCHANGE -o MASTER_PUBLIC_KEY -V $MASTER_PUB
-taler-config -c $CONF -s EXCHANGE-DEFAULT -o MASTER_KEY -V $MASTER_PUB
+taler-config -c $CONF -s exchange -o MASTER_PUBLIC_KEY -V $MASTER_PUB
+taler-config -c $CONF -s merchant-exchange-default -o MASTER_KEY -V $MASTER_PUB
 taler-config -c $CONF -s exchangedb-postgres -o CONFIG -V postgres:///$TARGET_DB
 taler-config -c $CONF -s auditordb-postgres -o CONFIG -V postgres:///$TARGET_DB
 taler-config -c $CONF -s merchantdb-postgres -o CONFIG -V postgres:///$TARGET_DB
@@ -78,13 +77,13 @@ taler-config -c $CONF -s bank -o database -V postgres:///$TARGET_DB
 
 # setup exchange
 echo "Setting up exchange"
-taler-exchange-dbinit -r -c $CONF
+taler-exchange-dbinit -c $CONF
 taler-exchange-wire -c $CONF 2> taler-exchange-wire.log
 taler-exchange-keyup -L INFO -c $CONF -o e2a.dat 2> taler-exchange-keyup.log
 
 # setup auditor
 echo "Setting up auditor"
-taler-auditor-dbinit -r -c $CONF
+taler-auditor-dbinit -c $CONF
 taler-auditor-exchange -c $CONF -m $MASTER_PUB -u $EXCHANGE_URL
 taler-auditor-sign -c $CONF -u $AUDITOR_URL -r e2a.dat -o a2e.dat -m $MASTER_PUB
 rm -f e2a.dat
@@ -103,7 +102,7 @@ taler-exchange-wirewatch -c $CONF 2> taler-exchange-wirewatch.log &
 taler-auditor-httpd -c $CONF 2> taler-auditor-httpd.log &
 
 # Wait for all services to be available
-for n in `seq 1 20`
+for n in `seq 1 50`
 do
     echo -n "."
     sleep 0.1
@@ -144,6 +143,7 @@ echo $MASTER_PUB > ${BASEDB}.mpub
 
 WIRE_FEE_DIR=`taler-config -c $CONF -f -s exchangedb -o WIREFEE_BASE_DIR`
 cp $WIRE_FEE_DIR/x-taler-bank.fee ${BASEDB}.fees
+date +%s > ${BASEDB}.age
 
 # clean up
 echo "Final clean up"
