@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014, 2015, 2018 GNUnet e.V.
+  Copyright (C) 2014, 2015, 2018 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -26,15 +26,6 @@
 #include <microhttpd.h>
 #include "taler_auditordb_plugin.h"
 
-/**
- * Should we return "Connection: close" in each response?
- */
-extern int TAH_auditor_connection_close;
-
-/**
- * The exchange's configuration.
- */
-extern struct GNUNET_CONFIGURATION_Handle *cfg;
 
 /**
  * Our DB plugin.
