@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2015, 2016, 2017 Taler Systems SA
+  Copyright (C) 2015--2020 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -27,7 +27,7 @@
 
 
 /**
- * @brief An admin/add/incoming Handle
+ * @brief An admin/add-incoming Handle
  */
 struct TALER_BANK_AdminAddIncomingHandle
 {
@@ -40,7 +40,7 @@ struct TALER_BANK_AdminAddIncomingHandle
   /**
    * POST context.
    */
-  struct TEAH_PostContext post_ctx;
+  struct TALER_CURL_PostContext post_ctx;
 
   /**
    * Handle for the request.
@@ -50,7 +50,7 @@ struct TALER_BANK_AdminAddIncomingHandle
   /**
    * Function to call with the result.
    */
-  TALER_BANK_AdminAddIncomingResultCallback cb;
+  TALER_BANK_AdminAddIncomingCallback cb;
 
   /**
    * Closure for @a cb.
@@ -62,7 +62,7 @@ struct TALER_BANK_AdminAddIncomingHandle
 
 /**
  * Function called when we're done processing the
- * HTTP /admin/add/incoming request.
+ * HTTP /admin/add-incoming request.
  *
  * @param cls the `struct TALER_BANK_AdminAddIncomingHandle`
  * @param response_code HTTP response code, 0 on error
@@ -112,32 +112,27 @@ handle_admin_add_incoming_finished (void *cls,
   case MHD_HTTP_BAD_REQUEST:
     /* This should never happen, either us or the bank is buggy
        (or API version conflict); just pass JSON reply to the application */
-    ec = TALER_BANK_parse_ec_ (j);
+    GNUNET_break_op (0);
+    ec = TALER_JSON_get_error_code (j);
     break;
   case MHD_HTTP_FORBIDDEN:
     /* Access denied */
-    ec = TALER_BANK_parse_ec_ (j);
+    ec = TALER_JSON_get_error_code (j);
     break;
   case MHD_HTTP_UNAUTHORIZED:
-    /* Nothing really to verify, bank says one of the signatures is
-       invalid; as we checked them, this should never happen, we
-       should pass the JSON reply to the application */
-    ec = TALER_BANK_parse_ec_ (j);
+    /* Nothing really to verify, bank says the password is invalid; we should
+       pass the JSON reply to the application */
+    ec = TALER_JSON_get_error_code (j);
     break;
   case MHD_HTTP_NOT_FOUND:
-    /* Nothing really to verify, this should never
-       happen, we should pass the JSON reply to the application */
-    ec = TALER_BANK_parse_ec_ (j);
-    break;
-  case MHD_HTTP_NOT_ACCEPTABLE:
-    /* Nothing really to verify, this should never
-       happen, we should pass the JSON reply to the application */
-    ec = TALER_BANK_parse_ec_ (j);
+    /* Nothing really to verify, maybe account really does not exist.
+       We should pass the JSON reply to the application */
+    ec = TALER_JSON_get_error_code (j);
     break;
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
     /* Server had an internal issue; we should retry, but this API
        leaves this to the application */
-    ec = TALER_BANK_parse_ec_ (j);
+    ec = TALER_JSON_get_error_code (j);
     break;
   default:
     /* unexpected response code */
@@ -145,7 +140,7 @@ handle_admin_add_incoming_finished (void *cls,
                 "Unexpected response code %u\n",
                 (unsigned int) response_code);
     GNUNET_break (0);
-    ec = TALER_BANK_parse_ec_ (j);
+    ec = TALER_JSON_get_error_code (j);
     response_code = 0;
     break;
   }
@@ -160,53 +155,43 @@ handle_admin_add_incoming_finished (void *cls,
 
 
 /**
- * Notify the bank that we have received an incoming transaction
- * which fills a reserve.  Note that this API is an administrative
- * API and thus not accessible to typical bank clients, but only
- * to the operators of the bank.
+ * Perform a wire transfer from some account to the exchange to fill a
+ * reserve.  Note that this API is usually only used for testing (with
+ * fakebank and our Python bank) and thus may not be accessible in a
+ * production setting.
  *
  * @param ctx curl context for the event loop
- * @param bank_base_url URL of the bank (used to execute this request)
  * @param auth authentication data to send to the bank
- * @param exchange_base_url base URL of the exchange (for tracking)
- * @param subject wire transfer subject for the transfer
- * @param amount amount that was deposited
- * @param debit_account_no account number to withdraw from (53 bits at most)
- * @param credit_account_no account number to deposit into (53 bits at most)
+ * @param reserve_pub wire transfer subject for the transfer
+ * @param amount amount that was is to be deposited
+ * @param debit_account account to deposit from (payto URI, but used as 'payfrom')
  * @param res_cb the callback to call when the final result for this request is available
  * @param res_cb_cls closure for the above callback
  * @return NULL
- *         if the inputs are invalid (i.e. invalid amount).
+ *         if the inputs are invalid (i.e. invalid amount) or internal errors.
  *         In this case, the callback is not called.
  */
 struct TALER_BANK_AdminAddIncomingHandle *
-TALER_BANK_admin_add_incoming (struct GNUNET_CURL_Context *ctx,
-                               const char *bank_base_url,
-                               const struct TALER_BANK_AuthenticationData *auth,
-                               const char *exchange_base_url,
-                               const char *subject,
-                               const struct TALER_Amount *amount,
-                               uint64_t debit_account_no,
-                               uint64_t credit_account_no,
-                               TALER_BANK_AdminAddIncomingResultCallback res_cb,
-                               void *res_cb_cls)
+TALER_BANK_admin_add_incoming (
+  struct GNUNET_CURL_Context *ctx,
+  const struct TALER_BANK_AuthenticationData *auth,
+  const struct TALER_ReservePublicKeyP *reserve_pub,
+  const struct TALER_Amount *amount,
+  const char *debit_account,
+  TALER_BANK_AdminAddIncomingCallback res_cb,
+  void *res_cb_cls)
 {
   struct TALER_BANK_AdminAddIncomingHandle *aai;
   json_t *admin_obj;
   CURL *eh;
 
-  if (NULL == exchange_base_url)
-  {
-    GNUNET_break (0);
-    return NULL;
-  }
-  admin_obj = json_pack ("{s:{s:s}, s:s, s:s, s:o, s:I, s:I}",
-                         "auth", "type", "basic",
-                         "exchange_url", exchange_base_url,
-                         "subject", subject,
-                         "amount", TALER_JSON_from_amount (amount),
-                         "debit_account", (json_int_t) debit_account_no,
-                         "credit_account", (json_int_t) credit_account_no);
+  admin_obj = json_pack ("{s:o, s:o, s:s}",
+                         "reserve_pub",
+                         GNUNET_JSON_from_data_auto (reserve_pub),
+                         "amount",
+                         TALER_JSON_from_amount (amount),
+                         "debit_account",
+                         debit_account);
   if (NULL == admin_obj)
   {
     GNUNET_break (0);
@@ -215,27 +200,47 @@ TALER_BANK_admin_add_incoming (struct GNUNET_CURL_Context *ctx,
   aai = GNUNET_new (struct TALER_BANK_AdminAddIncomingHandle);
   aai->cb = res_cb;
   aai->cb_cls = res_cb_cls;
-  aai->request_url = TALER_BANK_path_to_url_ (bank_base_url,
-                                              "/admin/add/incoming");
-  aai->post_ctx.headers = TALER_BANK_make_auth_header_ (auth);
-
-  GNUNET_assert
-    (NULL != (aai->post_ctx.headers = curl_slist_append
-                                        (aai->post_ctx.headers,
-                                        "Content-Type: application/json")));
+  aai->request_url = TALER_url_join (auth->wire_gateway_url,
+                                     "admin/add-incoming",
+                                     NULL);
+  if (NULL == aai->request_url)
+  {
+    GNUNET_free (aai);
+    json_decref (admin_obj);
+    GNUNET_break (0);
+    return NULL;
+  }
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Requesting administrative transaction at `%s' for reserve %s\n",
+              aai->request_url,
+              TALER_B2S (reserve_pub));
+  aai->post_ctx.headers = curl_slist_append
+                            (aai->post_ctx.headers,
+                            "Content-Type: application/json");
 
   eh = curl_easy_init ();
-
-  GNUNET_assert (GNUNET_OK ==
-                 TALER_curl_easy_post (&aai->post_ctx, eh, admin_obj));
-
+  if ( (NULL == eh) ||
+       (GNUNET_OK !=
+        TALER_BANK_setup_auth_ (eh,
+                                auth)) ||
+       (CURLE_OK !=
+        curl_easy_setopt (eh,
+                          CURLOPT_URL,
+                          aai->request_url)) ||
+       (GNUNET_OK !=
+        TALER_curl_easy_post (&aai->post_ctx,
+                              eh,
+                              admin_obj)) )
+  {
+    GNUNET_break (0);
+    TALER_BANK_admin_add_incoming_cancel (aai);
+    if (NULL != eh)
+      curl_easy_cleanup (eh);
+    json_decref (admin_obj);
+    return NULL;
+  }
   json_decref (admin_obj);
 
-  GNUNET_assert (CURLE_OK ==
-                 curl_easy_setopt (eh,
-                                   CURLOPT_URL,
-                                   aai->request_url));
-
   aai->job = GNUNET_CURL_job_add2 (ctx,
                                    eh,
                                    aai->post_ctx.headers,
@@ -252,8 +257,8 @@ TALER_BANK_admin_add_incoming (struct GNUNET_CURL_Context *ctx,
  * @param aai the admin add incoming request handle
  */
 void
-TALER_BANK_admin_add_incoming_cancel (struct
-                                      TALER_BANK_AdminAddIncomingHandle *aai)
+TALER_BANK_admin_add_incoming_cancel (
+  struct TALER_BANK_AdminAddIncomingHandle *aai)
 {
   if (NULL != aai->job)
   {
