@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2014-2019 Taler Systems SA
+  (C) 2014-2020 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it
   under the terms of the GNU Affero General Public License as
@@ -16,15 +16,13 @@
   along with TALER; see the file COPYING.  If not,
   see <http://www.gnu.org/licenses/>
 */
-
 /**
- * @file merchant/backend/taler-merchant-httpd.c
+ * @file benchmark/taler-exchange-benchmark.c
  * @brief HTTP serving layer intended to perform crypto-work and
  * communication with the exchange
  * @author Marcello Stanisci
  * @author Christian Grothoff
  */
-
 #include "platform.h"
 #include <gnunet/gnunet_util_lib.h>
 #include <microhttpd.h>
@@ -33,11 +31,9 @@
 #include "taler_signatures.h"
 #include "taler_exchange_service.h"
 #include "taler_json_lib.h"
-#include "taler_wire_lib.h"
 #include "taler_bank_service.h"
 #include "taler_fakebank_lib.h"
 #include "taler_testing_lib.h"
-#include "taler_testing_bank_lib.h"
 #include "taler_error_codes.h"
 
 /* Error codes.  */
@@ -58,27 +54,8 @@ enum BenchmarkError
  */
 #define UNITY_SIZE 6
 
-/**
- * Account number of the merchant.  Fakebank likes any number,
- * the only requirement is that this number then matches the
- * number given when building payto URLs at deposit time.
- */
-#define TALER_TESTING_USER_ACCOUNT_NUMBER 3
-
 #define FIRST_INSTRUCTION -1
 
-#define CMD_TRANSFER_TO_EXCHANGE(label, amount) \
-  TALER_TESTING_cmd_fakebank_transfer_retry \
-    (TALER_TESTING_cmd_fakebank_transfer (label, amount, \
-                                          exchange_bank_account.details. \
-                                          x_taler_bank.bank_base_url, \
-                                          TALER_TESTING_USER_ACCOUNT_NUMBER, \
-                                          exchange_bank_account.details. \
-                                          x_taler_bank.no, \
-                                          "dummy_user", \
-                                          "dummy_password", \
-                                          "http://example.com/"))
-
 
 /**
  * What mode should the benchmark run in?
@@ -105,7 +82,17 @@ enum BenchmarkMode
 /**
  * Hold information regarding which bank has the exchange account.
  */
-static struct TALER_Account exchange_bank_account;
+static struct TALER_BANK_AuthenticationData exchange_bank_account;
+
+/**
+ * Configuration of our exchange.
+ */
+static struct TALER_TESTING_ExchangeConfiguration ec;
+
+/**
+ * Hold information about a user at the bank.
+ */
+static char *user_payto_uri;
 
 /**
  * Time snapshot taken right before executing the CMDs.
@@ -118,11 +105,26 @@ static struct GNUNET_TIME_Absolute start_time;
  */
 static struct GNUNET_TIME_Relative duration;
 
+/**
+ * Array of all the commands the benchmark is running.
+ */
+static struct TALER_TESTING_Command *all_commands;
+
+/**
+ * Name of our configuration file.
+ */
+static char *cfg_filename;
+
 /**
  * Exit code.
  */
 static int result;
 
+/**
+ * Use the fakebank instead of the Python bank.
+ */
+static int use_fakebank;
+
 /**
  * How many coins we want to create per client and reserve.
  */
@@ -143,6 +145,11 @@ static unsigned int refresh_rate = 10;
  */
 static unsigned int howmany_clients = 1;
 
+/**
+ * Bank configuration to use.
+ */
+static struct TALER_TESTING_BankConfiguration bc;
+
 /**
  * Log level used during the run.
  */
@@ -164,9 +171,14 @@ static char *mode_str;
 static enum BenchmarkMode mode;
 
 /**
- * Config filename.
+ * Configuration.
  */
-static char *cfg_filename;
+static struct GNUNET_CONFIGURATION_Handle *cfg;
+
+/**
+ * Should we create all of the reserves at the beginning?
+ */
+static int reserves_first;
 
 /**
  * Currency used.
@@ -183,12 +195,70 @@ static char *remote_host;
  */
 static char *remote_dir;
 
+/**
+ * Array of command labels.
+ */
+static char **labels;
+
+/**
+ * Length of #labels.
+ */
+static unsigned int label_len;
+
+/**
+ * Offset in #labels.
+ */
+static unsigned int label_off;
+
 /**
  * Don't kill exchange/fakebank/wirewatch until
  * requested by the user explicitly.
  */
 static int linger;
 
+/**
+ * Performance counters.
+ */
+static struct TALER_TESTING_Timer timings[] = {
+  { .prefix = "createreserve" },
+  { .prefix = "withdraw" },
+  { .prefix = "deposit" },
+  { .prefix = "melt" },
+  { .prefix = "reveal" },
+  { .prefix = "link" },
+  { .prefix = NULL }
+};
+
+
+/**
+ * Add label to the #labels table and return it.
+ *
+ * @param label string to add to the table
+ * @return same string, now stored in the table
+ */
+const char *
+add_label (char *label)
+{
+  if (label_off == label_len)
+    GNUNET_array_grow (labels,
+                       label_len,
+                       label_len * 2 + 4);
+  labels[label_off++] = label;
+  return label;
+}
+
+
+static struct TALER_TESTING_Command
+cmd_transfer_to_exchange (const char *label,
+                          const char *amount)
+{
+  return TALER_TESTING_cmd_admin_add_incoming_retry
+           (TALER_TESTING_cmd_admin_add_incoming (label,
+                                                  amount,
+                                                  &exchange_bank_account,
+                                                  user_payto_uri));
+}
+
 
 /**
  * Decide which exchange account is going to be
@@ -202,9 +272,9 @@ static void
 pick_exchange_account_cb (void *cls,
                           const char *section)
 {
-  if (0 == strncasecmp ("account-",
+  if (0 == strncasecmp ("exchange-account-",
                         section,
-                        strlen ("account-")))
+                        strlen ("exchange-account-")))
   {
     const char **s = cls;
 
@@ -233,7 +303,10 @@ eval_probability (float probability)
 
 
 /**
- * Actual commands collection.
+ * Actual commands construction and execution.
+ *
+ * @param cls unused
+ * @param is interpreter to run commands with
  */
 static void
 run (void *cls,
@@ -242,23 +315,22 @@ run (void *cls,
   struct TALER_Amount total_reserve_amount;
   struct TALER_Amount withdraw_fee;
   char *withdraw_fee_str;
-  struct TALER_TESTING_Command all_commands
-  [howmany_reserves * (1     /* Withdraw block */
-                       + howmany_coins)   /* All units */
-   + 1 /* End CMD */];
-  char *AMOUNT_5;
-  char *AMOUNT_4;
-  char *AMOUNT_1;
+  char *amount_5;
+  char *amount_4;
+  char *amount_1;
 
   (void) cls;
-  GNUNET_asprintf (&AMOUNT_5, "%s:5", currency);
-  GNUNET_asprintf (&AMOUNT_4, "%s:4", currency);
-  GNUNET_asprintf (&AMOUNT_1, "%s:1", currency);
-
+  all_commands = GNUNET_new_array (howmany_reserves * (1     /* Withdraw block */
+                                                       + howmany_coins)   /* All units */
+                                   + 1 /* stat CMD */
+                                   + 1 /* End CMD */,
+                                   struct TALER_TESTING_Command);
+  GNUNET_asprintf (&amount_5, "%s:5", currency);
+  GNUNET_asprintf (&amount_4, "%s:4", currency);
+  GNUNET_asprintf (&amount_1, "%s:1", currency);
   GNUNET_assert (GNUNET_OK == TALER_amount_get_zero (currency,
                                                      &total_reserve_amount));
   total_reserve_amount.value = 5 * howmany_coins;
-
   GNUNET_asprintf (&withdraw_fee_str,
                    "%s:0.1",
                    currency);
@@ -275,13 +347,12 @@ run (void *cls,
     char *create_reserve_label;
 
     GNUNET_asprintf (&create_reserve_label,
-                     "create-reserve-%u",
+                     "createreserve-%u",
                      j);
     {
       struct TALER_TESTING_Command make_reserve[] = {
-        CMD_TRANSFER_TO_EXCHANGE
-          (create_reserve_label,
-          TALER_amount_to_string (&total_reserve_amount)),
+        cmd_transfer_to_exchange (add_label (create_reserve_label),
+                                  TALER_amount2s (&total_reserve_amount)),
         TALER_TESTING_cmd_end ()
       };
       char *batch_label;
@@ -289,8 +360,10 @@ run (void *cls,
       GNUNET_asprintf (&batch_label,
                        "batch-start-%u",
                        j);
-      all_commands[j * (howmany_coins + 1)]
-        = TALER_TESTING_cmd_batch (batch_label,
+      all_commands[reserves_first
+                   ? j
+                   : j * (howmany_coins + 1)]
+        = TALER_TESTING_cmd_batch (add_label (batch_label),
                                    make_reserve);
     }
     for (unsigned int i = 0; i < howmany_coins; i++)
@@ -299,67 +372,65 @@ run (void *cls,
       char *order_enc;
       struct TALER_TESTING_Command unit[UNITY_SIZE];
       char *unit_label;
+      const char *wl;
 
       GNUNET_asprintf (&withdraw_label,
                        "withdraw-%u-%u",
                        i,
                        j);
+      wl = add_label (withdraw_label);
       GNUNET_asprintf (&order_enc,
                        "{\"nonce\": %llu}",
                        i + (howmany_coins * j));
       unit[0] =
         TALER_TESTING_cmd_withdraw_with_retry
-          (TALER_TESTING_cmd_withdraw_amount
-            (withdraw_label,
-            create_reserve_label,
-            AMOUNT_5,
-            MHD_HTTP_OK));
+          (TALER_TESTING_cmd_withdraw_amount (wl,
+                                              create_reserve_label,
+                                              amount_5,
+                                              MHD_HTTP_OK));
       unit[1] =
         TALER_TESTING_cmd_deposit_with_retry
-          (TALER_TESTING_cmd_deposit
-            ("deposit",
-            withdraw_label,
-            0, /* Index of the one withdrawn coin in the traits.  */
-            TALER_TESTING_make_wire_details
-              (TALER_TESTING_USER_ACCOUNT_NUMBER,
-              exchange_bank_account.details.x_taler_bank.hostname),
-            order_enc,
-            GNUNET_TIME_UNIT_ZERO,
-            AMOUNT_1,
-            MHD_HTTP_OK));
-
+          (TALER_TESTING_cmd_deposit ("deposit",
+                                      wl,
+                                      0, /* Index of the one withdrawn coin in the traits.  */
+                                      user_payto_uri,
+                                      add_label (order_enc),
+                                      GNUNET_TIME_UNIT_ZERO,
+                                      amount_1,
+                                      MHD_HTTP_OK));
       if (eval_probability (refresh_rate / 100.0))
       {
         char *melt_label;
         char *reveal_label;
+        const char *ml;
+        const char *rl;
 
         GNUNET_asprintf (&melt_label,
-                         "refresh-melt-%u-%u",
+                         "melt-%u-%u",
                          i,
                          j);
+        ml = add_label (melt_label);
         GNUNET_asprintf (&reveal_label,
-                         "refresh-reveal-%u-%u",
+                         "reveal-%u-%u",
                          i,
                          j);
+        rl = add_label (reveal_label);
         unit[2] =
-          TALER_TESTING_cmd_refresh_melt_with_retry
-            (TALER_TESTING_cmd_refresh_melt
-              (melt_label,
-              withdraw_label,
-              MHD_HTTP_OK,
-              NULL));
+          TALER_TESTING_cmd_melt_with_retry
+            (TALER_TESTING_cmd_melt (ml,
+                                     wl,
+                                     MHD_HTTP_OK,
+                                     NULL));
         unit[3] =
           TALER_TESTING_cmd_refresh_reveal_with_retry
-            (TALER_TESTING_cmd_refresh_reveal
-              (reveal_label,
-              melt_label,
-              MHD_HTTP_OK));
+            (TALER_TESTING_cmd_refresh_reveal (rl,
+                                               ml,
+                                               MHD_HTTP_OK));
         unit[4] =
           TALER_TESTING_cmd_refresh_link_with_retry
-            (TALER_TESTING_cmd_refresh_link
-              ("refresh-link",
-              reveal_label,
-              MHD_HTTP_OK));
+            (TALER_TESTING_cmd_refresh_link ("link",
+                                             rl,
+                                             MHD_HTTP_OK));
         unit[5] = TALER_TESTING_cmd_end ();
       }
       else
@@ -369,20 +440,59 @@ run (void *cls,
                        "unit-%u-%u",
                        i,
                        j);
-      all_commands[j * (howmany_coins + 1) + (1 + i)]
-        = TALER_TESTING_cmd_batch (unit_label,
+      all_commands[reserves_first
+                   ? howmany_reserves + j * howmany_coins + i
+                   : j * (howmany_coins + 1) + (1 + i)]
+        = TALER_TESTING_cmd_batch (add_label (unit_label),
                                    unit);
     }
   }
   all_commands[howmany_reserves * (1 + howmany_coins)]
+    = TALER_TESTING_cmd_stat (timings);
+  all_commands[howmany_reserves * (1 + howmany_coins) + 1]
     = TALER_TESTING_cmd_end ();
   TALER_TESTING_run2 (is,
                       all_commands,
                       GNUNET_TIME_UNIT_FOREVER_REL); /* no timeout */
+  GNUNET_free (amount_1);
+  GNUNET_free (amount_4);
+  GNUNET_free (amount_5);
+  GNUNET_free (withdraw_fee_str);
   result = 1;
 }
 
 
+/**
+ * Print performance statistics for this process.
+ */
+static void
+print_stats (void)
+{
+  for (unsigned int i = 0; NULL != timings[i].prefix; i++)
+  {
+    char *total;
+    char *latency;
+
+    total = GNUNET_strdup (
+      GNUNET_STRINGS_relative_time_to_string (timings[i].total_duration,
+                                              GNUNET_YES));
+    latency = GNUNET_strdup (
+      GNUNET_STRINGS_relative_time_to_string (timings[i].success_latency,
+                                              GNUNET_YES));
+    fprintf (stderr,
+             "%s-%d took %s in total with %s for latency for %u executions (%u repeats)\n",
+             timings[i].prefix,
+             (int) getpid (),
+             total,
+             latency,
+             timings[i].num_commands,
+             timings[i].num_retries);
+    GNUNET_free (total);
+    GNUNET_free (latency);
+  }
+}
+
+
 /**
  * Stop the fakebank.
  *
@@ -400,27 +510,17 @@ stop_fakebank (void *cls)
 /**
  * Start the fakebank.
  *
- * @param cls the URL of the fakebank
+ * @param cls NULL
  */
 static void
 launch_fakebank (void *cls)
 {
-  const char *bank_base_url = cls;
-  const char *port;
-  long pnum;
   struct TALER_FAKEBANK_Handle *fakebank;
 
-  port = strrchr (bank_base_url,
-                  (unsigned char) ':');
-  if (NULL == port)
-    pnum = 80;
-  else
-    pnum = strtol (port + 1, NULL, 10);
-  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              "Starting Fakebank on port %u (%s)\n",
-              (unsigned int) pnum,
-              bank_base_url);
-  fakebank = TALER_FAKEBANK_start ((uint16_t) pnum);
+  (void) cls;
+  fakebank
+    = TALER_TESTING_run_fakebank (exchange_bank_account.wire_gateway_url,
+                                  currency);
   if (NULL == fakebank)
   {
     GNUNET_break (0);
@@ -438,19 +538,19 @@ launch_fakebank (void *cls)
  * @param main_cb main function to run per process
  * @param main_cb_cls closure for @a main_cb
  * @param config_file configuration file to use
- * @param exchange_url exchange URL to use
  * @return #GNUNET_OK on success
  */
 static int
 parallel_benchmark (TALER_TESTING_Main main_cb,
                     void *main_cb_cls,
-                    const char *config_file,
-                    const char *exchange_url)
+                    const char *config_file)
 {
   int result = GNUNET_OK;
   pid_t cpids[howmany_clients];
   pid_t fakebank = -1;
   int wstatus;
+  struct GNUNET_OS_Process *bankd = NULL;
+  struct GNUNET_OS_Process *auditord = NULL;
   struct GNUNET_OS_Process *exchanged = NULL;
   struct GNUNET_OS_Process *wirewatch = NULL;
   struct GNUNET_OS_Process *exchange_slave = NULL;
@@ -458,23 +558,41 @@ parallel_benchmark (TALER_TESTING_Main main_cb,
 
   if ( (MODE_CLIENT == mode) || (MODE_BOTH == mode) )
   {
-    /* start fakebank */
-    fakebank = fork ();
-    if (0 == fakebank)
+    if (use_fakebank)
     {
-      GNUNET_log_setup ("benchmark-fakebank",
-                        NULL == loglev ? "INFO" : loglev,
-                        logfile);
-      GNUNET_SCHEDULER_run (&launch_fakebank,
-                            exchange_bank_account.details.x_taler_bank.
-                            bank_base_url);
-      exit (0);
+      /* start fakebank */
+      fakebank = fork ();
+      if (0 == fakebank)
+      {
+        GNUNET_log_setup ("benchmark-fakebank",
+                          NULL == loglev ? "INFO" : loglev,
+                          logfile);
+        GNUNET_SCHEDULER_run (&launch_fakebank,
+                              NULL);
+        exit (0);
+      }
+      if (-1 == fakebank)
+      {
+        GNUNET_log_strerror (GNUNET_ERROR_TYPE_ERROR,
+                             "fork");
+        return GNUNET_SYSERR;
+      }
     }
-    if (-1 == fakebank)
+    else
     {
-      GNUNET_log_strerror (GNUNET_ERROR_TYPE_ERROR,
-                           "fork");
-      return GNUNET_SYSERR;
+      /* start bank */
+      if (GNUNET_OK !=
+          TALER_TESTING_prepare_bank (cfg_filename,
+                                      GNUNET_NO,
+                                      "exchange-account-2",
+                                      &bc))
+      {
+        return 1;
+      }
+      bankd = TALER_TESTING_run_bank (cfg_filename,
+                                      "http://localhost:8082/");
+      if (NULL == bankd)
+        return 77;
     }
   }
 
@@ -487,17 +605,56 @@ parallel_benchmark (TALER_TESTING_Main main_cb,
                                          "taler-exchange-httpd",
                                          "taler-exchange-httpd",
                                          "-c", config_file,
-                                         "-i",
                                          "-C",
                                          NULL);
     if ( (NULL == exchanged) && (MODE_BOTH == mode) )
     {
-      GNUNET_assert (-1 != fakebank);
-      kill (fakebank,
-            SIGTERM);
-      waitpid (fakebank,
-               &wstatus,
-               0);
+      if (-1 != fakebank)
+      {
+        kill (fakebank,
+              SIGTERM);
+        waitpid (fakebank,
+                 &wstatus,
+                 0);
+      }
+      if (NULL != bankd)
+      {
+        GNUNET_OS_process_kill (bankd,
+                                SIGTERM);
+        GNUNET_OS_process_destroy (bankd);
+      }
+      return 77;
+    }
+    /* start auditor */
+    auditord = GNUNET_OS_start_process (GNUNET_NO,
+                                        GNUNET_OS_INHERIT_STD_ALL,
+                                        NULL, NULL, NULL,
+                                        "taler-auditor-httpd",
+                                        "taler-auditor-httpd",
+                                        "-c", config_file,
+                                        NULL);
+    if (NULL == auditord)
+    {
+      GNUNET_OS_process_kill (exchanged,
+                              SIGTERM);
+      if (MODE_BOTH == mode)
+      {
+        if (-1 != fakebank)
+        {
+          kill (fakebank,
+                SIGTERM);
+          waitpid (fakebank,
+                   &wstatus,
+                   0);
+        }
+        if (NULL != bankd)
+        {
+          GNUNET_OS_process_kill (bankd,
+                                  SIGTERM);
+          GNUNET_OS_process_destroy (bankd);
+        }
+      }
+      GNUNET_OS_process_destroy (exchanged);
       return 77;
     }
     /* start exchange wirewatch */
@@ -510,16 +667,26 @@ parallel_benchmark (TALER_TESTING_Main main_cb,
                                          NULL);
     if (NULL == wirewatch)
     {
+      GNUNET_OS_process_kill (auditord,
+                              SIGTERM);
       GNUNET_OS_process_kill (exchanged,
                               SIGTERM);
       if (MODE_BOTH == mode)
       {
-        GNUNET_assert (-1 != fakebank);
-        kill (fakebank,
-              SIGTERM);
-        waitpid (fakebank,
-                 &wstatus,
-                 0);
+        if (-1 != fakebank)
+        {
+          kill (fakebank,
+                SIGTERM);
+          waitpid (fakebank,
+                   &wstatus,
+                   0);
+        }
+        if (NULL != bankd)
+        {
+          GNUNET_OS_process_kill (bankd,
+                                  SIGTERM);
+          GNUNET_OS_process_destroy (bankd);
+        }
       }
       GNUNET_OS_process_destroy (exchanged);
       return 77;
@@ -560,21 +727,29 @@ parallel_benchmark (TALER_TESTING_Main main_cb,
 
   /* We always wait for the exchange, no matter if it's running locally or
      remotely */
-  if (0 != TALER_TESTING_wait_exchange_ready (exchange_url))
+  if (0 != TALER_TESTING_wait_exchange_ready (ec.exchange_url))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Failed to detect running exchange at `%s'\n",
-                exchange_url);
+                ec.exchange_url);
     GNUNET_OS_process_kill (exchanged,
                             SIGTERM);
     if ( (MODE_BOTH == mode) || (MODE_CLIENT == mode))
     {
-      GNUNET_assert (-1 != fakebank);
-      kill (fakebank,
-            SIGTERM);
-      waitpid (fakebank,
-               &wstatus,
-               0);
+      if (-1 != fakebank)
+      {
+        kill (fakebank,
+              SIGTERM);
+        waitpid (fakebank,
+                 &wstatus,
+                 0);
+      }
+      if (NULL != bankd)
+      {
+        GNUNET_OS_process_kill (bankd,
+                                SIGTERM);
+        GNUNET_OS_process_destroy (bankd);
+      }
     }
     GNUNET_OS_process_wait (exchanged);
     GNUNET_OS_process_destroy (exchanged);
@@ -585,58 +760,79 @@ parallel_benchmark (TALER_TESTING_Main main_cb,
       GNUNET_OS_process_wait (wirewatch);
       GNUNET_OS_process_destroy (wirewatch);
     }
+    if (NULL != auditord)
+    {
+      GNUNET_OS_process_kill (auditord,
+                              SIGTERM);
+      GNUNET_OS_process_wait (auditord);
+      GNUNET_OS_process_destroy (auditord);
+    }
     return 77;
   }
   if ( (MODE_CLIENT == mode) || (MODE_BOTH == mode) )
   {
-    sleep (1); /* make sure fakebank process is ready before continuing */
+    if (-1 != fakebank)
+      sleep (1); /* make sure fakebank process is ready before continuing */
 
     start_time = GNUNET_TIME_absolute_get ();
     result = GNUNET_OK;
-    for (unsigned int i = 0; i<howmany_clients; i++)
-    {
-      if (0 == (cpids[i] = fork ()))
-      {
-        /* I am the child, do the work! */
-        GNUNET_log_setup ("benchmark-worker",
-                          NULL == loglev ? "INFO" : loglev,
-                          logfile);
 
-        result = TALER_TESTING_setup
-                   (main_cb,
-                   main_cb_cls,
-                   cfg_filename,
-                   exchanged,
-                   GNUNET_YES);
-        if (GNUNET_OK != result)
-          GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                      "Failure in child process test suite!\n");
-        if (GNUNET_OK == result)
-          exit (0);
-        else
-          exit (1);
-      }
-      if (-1 == cpids[i])
-      {
-        GNUNET_log_strerror (GNUNET_ERROR_TYPE_ERROR,
-                             "fork");
-        howmany_clients = i;
-        result = GNUNET_SYSERR;
-        break;
-      }
-      /* fork() success, continue starting more processes! */
-    }
-    /* collect all children */
-    for (unsigned int i = 0; i<howmany_clients; i++)
+    if (1 == howmany_clients)
     {
-      waitpid (cpids[i],
-               &wstatus,
-               0);
-      if ( (! WIFEXITED (wstatus)) ||
-           (0 != WEXITSTATUS (wstatus)) )
+      result = TALER_TESTING_setup (main_cb,
+                                    main_cb_cls,
+                                    cfg,
+                                    exchanged,
+                                    GNUNET_YES);
+      print_stats ();
+    }
+    else
+    {
+      for (unsigned int i = 0; i<howmany_clients; i++)
       {
-        GNUNET_break (0);
-        result = GNUNET_SYSERR;
+        if (0 == (cpids[i] = fork ()))
+        {
+          /* I am the child, do the work! */
+          GNUNET_log_setup ("benchmark-worker",
+                            NULL == loglev ? "INFO" : loglev,
+                            logfile);
+
+          result = TALER_TESTING_setup (main_cb,
+                                        main_cb_cls,
+                                        cfg,
+                                        exchanged,
+                                        GNUNET_YES);
+          print_stats ();
+          if (GNUNET_OK != result)
+            GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                        "Failure in child process test suite!\n");
+          if (GNUNET_OK == result)
+            exit (0);
+          else
+            exit (1);
+        }
+        if (-1 == cpids[i])
+        {
+          GNUNET_log_strerror (GNUNET_ERROR_TYPE_ERROR,
+                               "fork");
+          howmany_clients = i;
+          result = GNUNET_SYSERR;
+          break;
+        }
+        /* fork() success, continue starting more processes! */
+      }
+      /* collect all children */
+      for (unsigned int i = 0; i<howmany_clients; i++)
+      {
+        waitpid (cpids[i],
+                 &wstatus,
+                 0);
+        if ( (! WIFEXITED (wstatus)) ||
+             (0 != WEXITSTATUS (wstatus)) )
+        {
+          GNUNET_break (0);
+          result = GNUNET_SYSERR;
+        }
       }
     }
   }
@@ -645,8 +841,9 @@ parallel_benchmark (TALER_TESTING_Main main_cb,
   if (MODE_EXCHANGE == mode)
     (void) getchar ();
 
-  if ( (GNUNET_YES == linger) && ( ((mode == MODE_BOTH) || (mode ==
-                                                            MODE_CLIENT) ) ) )
+  if ( (GNUNET_YES == linger) &&
+       ( ((mode == MODE_BOTH) ||
+          (mode == MODE_CLIENT) ) ) )
   {
     printf ("press ENTER to stop\n");
     (void) getchar ();
@@ -674,6 +871,7 @@ parallel_benchmark (TALER_TESTING_Main main_cb,
   {
     GNUNET_assert (NULL != wirewatch);
     GNUNET_assert (NULL != exchanged);
+    GNUNET_assert (NULL != auditord);
     /* stop wirewatch */
     GNUNET_break (0 ==
                   GNUNET_OS_process_kill (wirewatch,
@@ -681,6 +879,13 @@ parallel_benchmark (TALER_TESTING_Main main_cb,
     GNUNET_break (GNUNET_OK ==
                   GNUNET_OS_process_wait (wirewatch));
     GNUNET_OS_process_destroy (wirewatch);
+    /* stop auditor */
+    GNUNET_break (0 ==
+                  GNUNET_OS_process_kill (auditord,
+                                          SIGTERM));
+    GNUNET_break (GNUNET_OK ==
+                  GNUNET_OS_process_wait (auditord));
+    GNUNET_OS_process_destroy (auditord);
     /* stop exchange */
     GNUNET_break (0 ==
                   GNUNET_OS_process_kill (exchanged,
@@ -693,19 +898,25 @@ parallel_benchmark (TALER_TESTING_Main main_cb,
   if ( (MODE_CLIENT == mode) || (MODE_BOTH == mode) )
   {
     /* stop fakebank */
-    GNUNET_assert (-1 != fakebank);
-    if (0 != kill (fakebank,
-                   SIGTERM))
-      GNUNET_log_strerror (GNUNET_ERROR_TYPE_WARNING,
-                           "kill");
-    waitpid (fakebank,
-             &wstatus,
-             0);
-    if ( (! WIFEXITED (wstatus)) ||
-         (0 != WEXITSTATUS (wstatus)) )
+    if (-1 != fakebank)
     {
-      GNUNET_break (0);
-      result = GNUNET_SYSERR;
+      kill (fakebank,
+            SIGTERM);
+      waitpid (fakebank,
+               &wstatus,
+               0);
+      if ( (! WIFEXITED (wstatus)) ||
+           (0 != WEXITSTATUS (wstatus)) )
+      {
+        GNUNET_break (0);
+        result = GNUNET_SYSERR;
+      }
+    }
+    if (NULL != bankd)
+    {
+      GNUNET_OS_process_kill (bankd,
+                              SIGTERM);
+      GNUNET_OS_process_destroy (bankd);
     }
   }
   return result;
@@ -723,9 +934,6 @@ int
 main (int argc,
       char *const *argv)
 {
-  char *exchange_url = NULL;
-  char *auditor_url = NULL;
-  struct GNUNET_CONFIGURATION_Handle *cfg;
   struct GNUNET_GETOPT_CommandLineOption options[] = {
     GNUNET_GETOPT_option_mandatory
       (GNUNET_GETOPT_option_cfgfile (&cfg_filename)),
@@ -762,6 +970,14 @@ main (int argc,
                                  "LF",
                                  "will log to file LF",
                                  &logfile),
+    GNUNET_GETOPT_option_flag ('f',
+                               "fakebank",
+                               "start a fakebank instead of the Python bank",
+                               &use_fakebank),
+    GNUNET_GETOPT_option_flag ('F',
+                               "reserves-first",
+                               "should all reserves be created first, before starting normal operations",
+                               &reserves_first),
     GNUNET_GETOPT_option_flag ('K',
                                "linger",
                                "linger around until key press",
@@ -777,6 +993,7 @@ main (int argc,
                                    argc,
                                    argv)))
   {
+    GNUNET_free_non_null (cfg_filename);
     return BAD_CLI_ARG;
   }
   GNUNET_log_setup ("taler-exchange-benchmark",
@@ -793,6 +1010,7 @@ main (int argc,
   else
   {
     TALER_LOG_ERROR ("Unknown mode given: '%s'\n", mode_str);
+    GNUNET_free_non_null (cfg_filename);
     return BAD_CONFIG_FILE;
   }
   if (NULL == cfg_filename)
@@ -804,18 +1022,15 @@ main (int argc,
                                  cfg_filename))
   {
     TALER_LOG_ERROR ("Could not parse configuration\n");
+    GNUNET_free (cfg_filename);
     return BAD_CONFIG_FILE;
   }
   if (GNUNET_OK !=
-      GNUNET_CONFIGURATION_get_value_string (cfg,
-                                             "taler",
-                                             "currency",
-                                             &currency))
+      TALER_config_get_currency (cfg,
+                                 &currency))
   {
-    GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
-                               "taler",
-                               "currency");
     GNUNET_CONFIGURATION_destroy (cfg);
+    GNUNET_free (cfg_filename);
     return BAD_CONFIG_FILE;
   }
   if (howmany_clients > 10240)
@@ -823,85 +1038,79 @@ main (int argc,
     TALER_LOG_ERROR ("-p option value given is too large\n");
     return BAD_CLI_ARG;
   }
-
+  if (0 == howmany_clients)
+  {
+    TALER_LOG_ERROR ("-p option value must not be zero\n");
+    GNUNET_free (cfg_filename);
+    return BAD_CLI_ARG;
+  }
+  if (GNUNET_OK !=
+      GNUNET_CONFIGURATION_get_value_string (cfg,
+                                             "benchmark",
+                                             "USER_PAYTO_URI",
+                                             &user_payto_uri))
+  {
+    GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
+                               "benchmark",
+                               "USER_PAYTO_URI");
+    GNUNET_free (cfg_filename);
+    return BAD_CONFIG_FILE;
+  }
 
   {
     const char *bank_details_section;
-    char *exchange_payto_url;
-
-    GNUNET_CONFIGURATION_iterate_sections
-      (cfg,
-      &pick_exchange_account_cb,
-      &bank_details_section);
 
+    GNUNET_CONFIGURATION_iterate_sections (cfg,
+                                           &pick_exchange_account_cb,
+                                           &bank_details_section);
     if (NULL == bank_details_section)
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                  _ (
-                    "Missing specification of bank account in configuration\n"));
+                  "Missing specification of bank account in configuration\n");
+      GNUNET_free (cfg_filename);
       return BAD_CONFIG_FILE;
     }
     if (GNUNET_OK !=
-        GNUNET_CONFIGURATION_get_value_string
-          (cfg,
-          bank_details_section,
-          "url",
-          &exchange_payto_url))
-    {
-      GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
-                                 bank_details_section,
-                                 "url");
-      return BAD_CONFIG_FILE;
-    }
-
-    if (TALER_EC_NONE !=
-        TALER_WIRE_payto_to_account (exchange_payto_url,
-                                     &exchange_bank_account))
+        TALER_BANK_auth_parse_cfg (cfg,
+                                   bank_details_section,
+                                   &exchange_bank_account))
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                  _ ("Malformed payto:// URL `%s' in configuration\n"),
-                  exchange_payto_url);
-      GNUNET_free (exchange_payto_url);
+                  "Configuration fails to provide exchange bank details in section `%s'\n",
+                  bank_details_section);
+      GNUNET_free (cfg_filename);
       return BAD_CONFIG_FILE;
     }
-    if (TALER_PAC_X_TALER_BANK != exchange_bank_account.type)
-    {
-      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                  _ (
-                    "Malformed payto:// URL `%s' in configuration: x-taler-bank required\n"),
-                  exchange_payto_url);
-      GNUNET_free (exchange_payto_url);
-      return BAD_CONFIG_FILE;
-    }
-    GNUNET_free (exchange_payto_url);
   }
   if ( (MODE_EXCHANGE == mode) || (MODE_BOTH == mode) )
   {
     struct GNUNET_OS_Process *compute_wire_response;
 
-    compute_wire_response = GNUNET_OS_start_process
-                              (GNUNET_NO,
-                              GNUNET_OS_INHERIT_STD_ALL,
-                              NULL, NULL, NULL,
-                              "taler-exchange-wire",
-                              "taler-exchange-wire",
-                              "-c", cfg_filename,
-                              NULL);
+    compute_wire_response
+      = GNUNET_OS_start_process (GNUNET_NO,
+                                 GNUNET_OS_INHERIT_STD_ALL,
+                                 NULL, NULL, NULL,
+                                 "taler-exchange-wire",
+                                 "taler-exchange-wire",
+                                 "-c", cfg_filename,
+                                 NULL);
     if (NULL == compute_wire_response)
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                  "Failed to run `taler-exchange-wire`,"
-                  " is your PATH correct?\n");
-      return GNUNET_NO;
+                  "Failed to run `taler-exchange-wire`, is your PATH correct?\n");
+      GNUNET_free (cfg_filename);
+      return BAD_CONFIG_FILE;
     }
     GNUNET_OS_process_wait (compute_wire_response);
     GNUNET_OS_process_destroy (compute_wire_response);
-
-    GNUNET_assert
-      (GNUNET_OK == TALER_TESTING_prepare_exchange
-        (cfg_filename,
-        &auditor_url,
-        &exchange_url));
+    /* If we use the fakebank, we MUST reset the database as the fakebank
+       will have forgotten everything... */
+    GNUNET_assert (GNUNET_OK ==
+                   TALER_TESTING_prepare_exchange (cfg_filename,
+                                                   (GNUNET_YES == use_fakebank)
+                                                   ? GNUNET_YES
+                                                   : GNUNET_NO,
+                                                   &ec));
   }
   else
   {
@@ -909,12 +1118,13 @@ main (int argc,
         GNUNET_CONFIGURATION_get_value_string (cfg,
                                                "exchange",
                                                "BASE_URL",
-                                               &exchange_url))
+                                               &ec.exchange_url))
     {
       GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
                                  "exchange",
                                  "base_url");
       GNUNET_CONFIGURATION_destroy (cfg);
+      GNUNET_free (cfg_filename);
       return BAD_CONFIG_FILE;
     }
 
@@ -928,6 +1138,7 @@ main (int argc,
                                  "benchmark-remote-exchange",
                                  "host");
       GNUNET_CONFIGURATION_destroy (cfg);
+      GNUNET_free (cfg_filename);
       return BAD_CONFIG_FILE;
     }
 
@@ -941,24 +1152,22 @@ main (int argc,
                                  "benchmark-remote-exchange",
                                  "dir");
       GNUNET_CONFIGURATION_destroy (cfg);
+      GNUNET_free (cfg_filename);
       return BAD_CONFIG_FILE;
     }
   }
 
-  GNUNET_CONFIGURATION_destroy (cfg);
-
   result = parallel_benchmark (&run,
                                NULL,
-                               cfg_filename,
-                               exchange_url);
-
-  GNUNET_free_non_null (exchange_url);
-  GNUNET_free_non_null (auditor_url);
+                               cfg_filename);
+  GNUNET_CONFIGURATION_destroy (cfg);
+  GNUNET_free (cfg_filename);
 
   /* If we're the exchange worker, we're done now.  No need to print results */
   if (MODE_EXCHANGE == mode)
+  {
     return (GNUNET_OK == result) ? 0 : result;
-
+  }
   duration = GNUNET_TIME_absolute_get_duration (start_time);
   if (GNUNET_OK == result)
   {
@@ -994,5 +1203,10 @@ main (int argc,
              (unsigned long long) (usage.ru_utime.tv_sec * 1000 * 1000
                                    + usage.ru_utime.tv_usec));
   }
+  for (unsigned int i = 0; i<label_off; i++)
+    GNUNET_free (labels[i]);
+  GNUNET_array_grow (labels,
+                     label_len,
+                     0);
   return (GNUNET_OK == result) ? 0 : result;
 }
