@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014, 2015, 2016 GNUnet e.V.
+  Copyright (C) 2014, 2015, 2016 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -41,6 +41,12 @@ static const struct GNUNET_CONFIGURATION_Handle *kcfg;
  */
 static int global_ret;
 
+/**
+ * Option -i used to print full denomination key hashes for
+ * denominations of certain amounts.
+ */
+static struct TALER_Amount print_dk_amount;
+
 
 /**
  * Function called on each signing key.
@@ -95,8 +101,9 @@ signkeys_iter (void *cls,
              filename);
     return GNUNET_SYSERR;
   }
-  printf ("Signing key `%s' valid\n",
-          filename);
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+              "Signing key `%s' valid\n",
+              filename);
   return GNUNET_OK;
 }
 
@@ -132,9 +139,10 @@ static int
 denomkeys_iter (void *cls,
                 const char *alias,
                 const struct
-                TALER_EXCHANGEDB_DenominationKeyIssueInformation *dki)
+                TALER_EXCHANGEDB_DenominationKey *dki)
 {
   struct GNUNET_HashCode hc;
+  struct TALER_Amount value;
 
   (void) cls;
   if (ntohl (dki->issue.properties.purpose.size) !=
@@ -182,10 +190,33 @@ denomkeys_iter (void *cls,
              alias);
     return GNUNET_SYSERR;
   }
-  printf ("Denomination key `%s' (%s) is valid\n",
-          alias,
-          GNUNET_h2s (&hc));
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+              "Denomination key `%s' (%s) is valid\n",
+              alias,
+              GNUNET_h2s (&hc));
+  TALER_amount_ntoh (&value,
+                     &dki->issue.properties.value);
+  if ( (GNUNET_OK ==
+        TALER_amount_cmp_currency (&print_dk_amount,
+                                   &value)) &&
+       (0 ==
+        TALER_amount_cmp (&print_dk_amount,
+                          &value)) )
+  {
+    char *dh;
+    struct GNUNET_TIME_Absolute start;
 
+    start = GNUNET_TIME_absolute_ntoh (dki->issue.properties.start);
+    dh = GNUNET_STRINGS_data_to_string_alloc (&dki->issue.properties.denom_hash,
+                                              sizeof (struct GNUNET_HashCode));
+    /* output start time first for easy numeric sorting, then
+       the denomination hash, and finally the human-readable start time */
+    printf ("%020llu %s %s\n",
+            (unsigned long long) start.abs_value_us,
+            dh,
+            GNUNET_STRINGS_absolute_time_to_string (start));
+    GNUNET_free (dh);
+  }
   return GNUNET_OK;
 }
 
@@ -212,7 +243,6 @@ exchange_denomkeys_check ()
     GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
                                "exchange",
                                "master_public_key");
-    global_ret = 1;
     return GNUNET_NO;
   }
   if (0 > TALER_EXCHANGEDB_denomination_keys_iterate (exchange_directory,
@@ -271,12 +301,22 @@ run (void *cls,
  * @return 0 ok, 1 on error
  */
 int
-main (int argc, char *const *argv)
+main (int argc,
+      char *const *argv)
 {
   const struct GNUNET_GETOPT_CommandLineOption options[] = {
+    TALER_getopt_get_amount ('i',
+                             "denomination-info-hash",
+                             "AMOUNT",
+                             "print full denomination hashes of all denominations with the given AMOUNT value",
+                             &print_dk_amount),
     GNUNET_GETOPT_OPTION_END
   };
 
+  /* force linker to link against libtalerutil; if we do
+    not do this, the linker may "optimize" libtalerutil
+    away and skip #TALER_OS_init(), which we do need */
+  (void) TALER_project_data_default ();
   GNUNET_assert (GNUNET_OK ==
                  GNUNET_log_setup ("taler-exchange-keycheck",
                                    "WARNING",
