@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2017 GNUnet e.V.
+  Copyright (C) 2014-2017 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -30,6 +30,12 @@
 /**
  * How often should we retry a transaction before giving up
  * (for transactions resulting in serialization/dead locks only).
+ *
+ * The current value is likely too high for production. We might want to
+ * benchmark good values once we have a good database setup.  The code is
+ * expected to work correctly with any positive value, albeit inefficiently if
+ * we too aggressively force clients to retry the HTTP request merely because
+ * we have database serialization issues.
  */
 #define MAX_TRANSACTION_COMMIT_RETRIES 100
 
@@ -42,10 +48,10 @@
  * it returns the soft error code, the function MAY be called again to
  * retry and MUST not queue a MHD response.
  *
- * @param cls a `struct DepositContext`
- * @param connection MHD request context
+ * @param cls a `struct TEH_DB_KnowCoinContext`
+ * @param connection MHD request context, must not be NULL
  * @param session database session and transaction to use
- * @param[out] mhd_ret set to MHD status on error
+ * @param[out] mhd_ret set to MHD status on error, must not be NULL
  * @return transaction status
  */
 enum GNUNET_DB_QueryStatus
@@ -57,6 +63,7 @@ TEH_DB_know_coin_transaction (void *cls,
   struct TEH_DB_KnowCoinContext *kcc = cls;
   enum GNUNET_DB_QueryStatus qs;
 
+  GNUNET_assert (NULL != mhd_ret);
   qs = TEH_plugin->ensure_coin_known (TEH_plugin->cls,
                                       session,
                                       kcc->coin);
@@ -80,9 +87,11 @@ TEH_DB_know_coin_transaction (void *cls,
  * retries @a cb a few times.  Upon hard or persistent soft
  * errors, generates an error message for @a connection.
  *
- * @param connection MHD connection to run @a cb for
+ * @param connection MHD connection to run @a cb for, can be NULL
  * @param name name of the transaction (for debugging)
- * @param[out] set to MHD response code, if transaction failed
+ * @param[out] mhd_ret set to MHD response code, if transaction failed;
+ *             NULL if we are not running with a @a connection and thus
+ *             must not queue MHD replies
  * @param cb callback implementing transaction logic
  * @param cb_cls closure for @a cb, must be read-only!
  * @return #GNUNET_OK on success, #GNUNET_SYSERR on failure
@@ -97,7 +106,7 @@ TEH_DB_run_transaction (struct MHD_Connection *connection,
   struct TALER_EXCHANGEDB_Session *session;
 
   if (NULL != mhd_ret)
-    *mhd_ret = -1; /* invalid value */
+    *mhd_ret = -1; /* set to invalid value, to help detect bugs */
   if (NULL == (session = TEH_plugin->get_session (TEH_plugin->cls)))
   {
     GNUNET_break (0);
@@ -108,9 +117,8 @@ TEH_DB_run_transaction (struct MHD_Connection *connection,
                                              "could not establish database session");
     return GNUNET_SYSERR;
   }
-  TEH_plugin->preflight (TEH_plugin->cls,
-                         session);
-  for (unsigned int retries = 0; retries < MAX_TRANSACTION_COMMIT_RETRIES;
+  for (unsigned int retries = 0;
+       retries < MAX_TRANSACTION_COMMIT_RETRIES;
        retries++)
   {
     enum GNUNET_DB_QueryStatus qs;
@@ -167,123 +175,4 @@ TEH_DB_run_transaction (struct MHD_Connection *connection,
 }
 
 
-/**
- * Calculate the total value of all transactions performed.
- * Stores @a off plus the cost of all transactions in @a tl
- * in @a ret.
- *
- * @param tl transaction list to process
- * @param off offset to use as the starting value
- * @param[out] ret where the resulting total is to be stored
- * @return #GNUNET_OK on success, #GNUNET_SYSERR on errors
- */
-// FIXME: maybe move to another module, i.e. exchangedb???
-int
-TEH_DB_calculate_transaction_list_totals (struct
-                                          TALER_EXCHANGEDB_TransactionList *tl,
-                                          const struct TALER_Amount *off,
-                                          struct TALER_Amount *ret)
-{
-  struct TALER_Amount spent = *off;
-  struct TALER_Amount refunded;
-
-  GNUNET_assert (GNUNET_OK ==
-                 TALER_amount_get_zero (spent.currency,
-                                        &refunded));
-  for (struct TALER_EXCHANGEDB_TransactionList *pos = tl; NULL != pos; pos =
-         pos->next)
-  {
-    switch (pos->type)
-    {
-    case TALER_EXCHANGEDB_TT_DEPOSIT:
-      /* spent += pos->amount_with_fee */
-      if (GNUNET_OK !=
-          TALER_amount_add (&spent,
-                            &spent,
-                            &pos->details.deposit->amount_with_fee))
-      {
-        GNUNET_break (0);
-        return GNUNET_SYSERR;
-      }
-      break;
-    case TALER_EXCHANGEDB_TT_REFRESH_MELT:
-      /* spent += pos->amount_with_fee */
-      if (GNUNET_OK !=
-          TALER_amount_add (&spent,
-                            &spent,
-                            &pos->details.melt->session.amount_with_fee))
-      {
-        GNUNET_break (0);
-        return GNUNET_SYSERR;
-      }
-      break;
-    case TALER_EXCHANGEDB_TT_REFUND:
-      /* refunded += pos->refund_amount - pos->refund_fee */
-      if (GNUNET_OK !=
-          TALER_amount_add (&refunded,
-                            &refunded,
-                            &pos->details.refund->refund_amount))
-      {
-        GNUNET_break (0);
-        return GNUNET_SYSERR;
-      }
-      if (GNUNET_OK !=
-          TALER_amount_subtract (&refunded,
-                                 &refunded,
-                                 &pos->details.refund->refund_fee))
-      {
-        GNUNET_break (0);
-        return GNUNET_SYSERR;
-      }
-      break;
-    case TALER_EXCHANGEDB_TT_OLD_COIN_PAYBACK:
-      /* refunded += pos->value */
-      if (GNUNET_OK !=
-          TALER_amount_add (&refunded,
-                            &refunded,
-                            &pos->details.old_coin_payback->value))
-      {
-        GNUNET_break (0);
-        return GNUNET_SYSERR;
-      }
-      break;
-    case TALER_EXCHANGEDB_TT_PAYBACK:
-      /* spent += pos->value */
-      if (GNUNET_OK !=
-          TALER_amount_add (&spent,
-                            &spent,
-                            &pos->details.payback->value))
-      {
-        GNUNET_break (0);
-        return GNUNET_SYSERR;
-      }
-      break;
-    case TALER_EXCHANGEDB_TT_PAYBACK_REFRESH:
-      /* spent += pos->value */
-      if (GNUNET_OK !=
-          TALER_amount_add (&spent,
-                            &spent,
-                            &pos->details.payback_refresh->value))
-      {
-        GNUNET_break (0);
-        return GNUNET_SYSERR;
-      }
-      break;
-    }
-  }
-  /* spent = spent - refunded */
-  if (GNUNET_SYSERR ==
-      TALER_amount_subtract (&spent,
-                             &spent,
-                             &refunded))
-  {
-    GNUNET_break (0);
-    return GNUNET_SYSERR;
-  }
-
-  *ret = spent;
-  return GNUNET_OK;
-}
-
-
 /* end of taler-exchange-httpd_db.c */
