@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2017 GNUnet e.V.
+  Copyright (C) 2014-2017 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -50,10 +50,10 @@ struct TEH_DB_KnowCoinContext
  * it returns the soft error code, the function MAY be called again to
  * retry and MUST not queue a MHD response.
  *
- * @param cls a `struct DepositContext`
- * @param connection MHD request context
+ * @param cls a `struct TEH_DB_KnowCoinContext`
+ * @param connection MHD request context, must not be NULL
  * @param session database session and transaction to use
- * @param[out] mhd_ret set to MHD status on error
+ * @param[out] mhd_ret set to MHD status on error, must not be NULL
  * @return transaction status
  */
 enum GNUNET_DB_QueryStatus
@@ -92,9 +92,11 @@ typedef enum GNUNET_DB_QueryStatus
  * retries @a cb a few times.  Upon hard or persistent soft
  * errors, generates an error message for @a connection.
  *
- * @param connection MHD connection to run @a cb for
+ * @param connection MHD connection to run @a cb for, can be NULL
  * @param name name of the transaction (for debugging)
- * @param[out] set to MHD response code, if transaction failed
+ * @param[out] mhd_ret set to MHD response code, if transaction failed;
+ *             NULL if we are not running with a @a connection and thus
+ *             must not queue MHD replies
  * @param cb callback implementing transaction logic
  * @param cb_cls closure for @a cb, must be read-only!
  * @return #GNUNET_OK on success, #GNUNET_SYSERR on failure
@@ -107,23 +109,5 @@ TEH_DB_run_transaction (struct MHD_Connection *connection,
                         void *cb_cls);
 
 
-/**
- * Calculate the total value of all transactions performed.
- * Stores @a off plus the cost of all transactions in @a tl
- * in @a ret.
- *
- * @param tl transaction list to process
- * @param off offset to use as the starting value
- * @param[out] ret where the resulting total is to be stored
- * @return #GNUNET_OK on success, #GNUNET_SYSERR on errors
- */
-// FIXME: maybe move to another module, i.e. exchangedb???
-int
-TEH_DB_calculate_transaction_list_totals (struct
-                                          TALER_EXCHANGEDB_TransactionList *tl,
-                                          const struct TALER_Amount *off,
-                                          struct TALER_Amount *ret);
-
-
 #endif
 /* TALER_EXCHANGE_HTTPD_DB_H */
