@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014, 2015 GNUnet e.V.
+  Copyright (C) 2014, 2015 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -42,6 +42,7 @@ struct TEH_KS_StateHandle;
  * For every call to #TEH_KS_acquire(), a matching call
  * to #TEH_KS_release() must be made.
  *
+ * @param now for what timestamp should we acquire the key state
  * @param location name of the function in which the lock is acquired
  * @return the key state, NULL on error (usually pretty fatal)
  */
@@ -82,7 +83,16 @@ TEH_KS_release_ (const char *location,
 
 
 /**
- * Finally, release #internal_key_state.
+ * Setup initial #internal_key_state and our signal handlers.
+ *
+ * @return #GNUNET_OK on success
+ */
+int
+TEH_KS_init (void);
+
+
+/**
+ * Finally, release #internal_key_state and our signal handlers.
  */
 void
 TEH_KS_free (void);
@@ -98,25 +108,25 @@ enum TEH_KS_DenominationKeyUse
 {
 
   /**
-   * The key is to be used for a /reserve/withdraw or /refresh (exchange)
-   * operation.
+   * The denomination key is to be used for a withdraw or reveal operation.
    */
   TEH_KS_DKU_WITHDRAW,
 
   /**
-   * The key is to be used for a /deposit or /refresh (melt) operation.
+   * The denomination key is to be used for a deposit or melt operation.
    */
   TEH_KS_DKU_DEPOSIT,
 
   /**
-   * The key is to be used for a /payback operation.
+   * The denomination key is to be used for a recoup operation, or to
+   * melt a coin that was deposited (or melted) before the revocation.
    */
-  TEH_KS_DKU_PAYBACK,
+  TEH_KS_DKU_RECOUP,
 
   /**
-   * The key is to be used for a /refresh/payback operation,
+   * The key is to be used for a refresh + recoup operation,
    * i.e. it is an old coin that regained value from a
-   * payback on a new coin derived from the old coin.
+   * recoup on a new coin derived from the old coin.
    */
   TEH_KS_DKU_ZOMBIE
 
@@ -130,19 +140,18 @@ enum TEH_KS_DenominationKeyUse
  * @param key_state state to look in
  * @param denom_pub_hash hash of denomination public key
  * @param use purpose for which the key is being located
- * @param ec[out] set to the error code, in case the operation failed
- * @param hc[out] set to the HTTP status code to use
+ * @param[out] ec set to the error code, in case the operation failed
+ * @param[out] hc set to the HTTP status code to use
  * @return the denomination key issue,
  *         or NULL if denom_pub could not be found (or is not valid at this time for the given @a use)
  */
-struct TALER_EXCHANGEDB_DenominationKeyIssueInformation *
-TEH_KS_denomination_key_lookup_by_hash (const struct
-                                        TEH_KS_StateHandle *key_state,
-                                        const struct
-                                        GNUNET_HashCode *denom_pub_hash,
-                                        enum TEH_KS_DenominationKeyUse use,
-                                        enum TALER_ErrorCode *ec,
-                                        unsigned int *hc);
+struct TALER_EXCHANGEDB_DenominationKey *
+TEH_KS_denomination_key_lookup_by_hash (
+  const struct TEH_KS_StateHandle *key_state,
+  const struct GNUNET_HashCode *denom_pub_hash,
+  enum TEH_KS_DenominationKeyUse use,
+  enum TALER_ErrorCode *ec,
+  unsigned int *hc);
 
 
 /**
@@ -178,17 +187,13 @@ TEH_KS_sign (const struct GNUNET_CRYPTO_EccSignaturePurpose *purpose,
  *
  * @param rh context of the handler
  * @param connection the MHD connection to handle
- * @param[in,out] connection_cls the connection's closure (can be updated)
- * @param upload_data upload data
- * @param[in,out] upload_data_size number of bytes (left) in @a upload_data
+ * @param args array of additional options (must be empty for this function)
  * @return MHD result code
-  */
+ */
 int
-TEH_KS_handler_keys (struct TEH_RequestHandler *rh,
-                     struct MHD_Connection *connection,
-                     void **connection_cls,
-                     const char *upload_data,
-                     size_t *upload_data_size);
+TEH_handler_keys (const struct TEH_RequestHandler *rh,
+                  struct MHD_Connection *connection,
+                  const char *const args[]);
 
 
 #endif
