@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2017 Inria and GNUnet e.V.
+  Copyright (C) 2014-2020 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -15,7 +15,7 @@
 */
 /**
  * @file taler-exchange-httpd_refund.c
- * @brief Handle /refund requests; parses the POST and JSON and
+ * @brief Handle refund requests; parses the POST and JSON and
  *        verifies the coin signature before handing things off
  *        to the database.
  * @author Florian Dold
@@ -33,30 +33,32 @@
 #include "taler-exchange-httpd_refund.h"
 #include "taler-exchange-httpd_responses.h"
 #include "taler-exchange-httpd_keystate.h"
-#include "taler-exchange-httpd_validation.h"
 
 
 /**
  * Generate successful refund confirmation message.
  *
  * @param connection connection to the client
+ * @param coin_pub public key of the coin
  * @param refund details about the successful refund
  * @return MHD result code
  */
 static int
 reply_refund_success (struct MHD_Connection *connection,
-                      const struct TALER_EXCHANGEDB_Refund *refund)
+                      const struct TALER_CoinSpendPublicKeyP *coin_pub,
+                      const struct TALER_EXCHANGEDB_RefundListEntry *refund)
 {
-  struct TALER_RefundConfirmationPS rc;
   struct TALER_ExchangePublicKeyP pub;
   struct TALER_ExchangeSignatureP sig;
+  struct TALER_RefundConfirmationPS rc = {
+    .purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_CONFIRM_REFUND),
+    .purpose.size = htonl (sizeof (rc)),
+    .h_contract_terms = refund->h_contract_terms,
+    .coin_pub = *coin_pub,
+    .merchant = refund->merchant_pub,
+    .rtransaction_id = GNUNET_htonll (refund->rtransaction_id)
+  };
 
-  rc.purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_CONFIRM_REFUND);
-  rc.purpose.size = htonl (sizeof (struct TALER_RefundConfirmationPS));
-  rc.h_contract_terms = refund->h_contract_terms;
-  rc.coin_pub = refund->coin.coin_pub;
-  rc.merchant = refund->merchant_pub;
-  rc.rtransaction_id = GNUNET_htonll (refund->rtransaction_id);
   TALER_amount_hton (&rc.refund_amount,
                      &refund->refund_amount);
   TALER_amount_hton (&rc.refund_fee,
@@ -69,7 +71,7 @@ reply_refund_success (struct MHD_Connection *connection,
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_INTERNAL_SERVER_ERROR,
                                        TALER_EC_EXCHANGE_BAD_CONFIGURATION,
-                                       "no keys");
+                                       "no online signing key");
   }
   return TALER_MHD_reply_json_pack (connection,
                                     MHD_HTTP_OK,
@@ -80,52 +82,6 @@ reply_refund_success (struct MHD_Connection *connection,
 }
 
 
-/**
- * Generate generic refund failure message. All the details
- * are in the @a response_code.  The body can be empty.
- *
- * @param connection connection to the client
- * @param response_code response code to generate
- * @param ec taler error code to include
- * @return MHD result code
- */
-static int
-reply_refund_failure (struct MHD_Connection *connection,
-                      unsigned int response_code,
-                      enum TALER_ErrorCode ec)
-{
-  return TALER_MHD_reply_json_pack (connection,
-                                    response_code,
-                                    "{s:s, s:I}",
-                                    "hint", "refund failure",
-                                    "code", (json_int_t) ec);
-}
-
-
-/**
- * Generate refund conflict failure message. Returns the
- * transaction list @a tl with the details about the conflict.
- *
- * @param connection connection to the client
- * @param tl transaction list showing the conflict
- * @return MHD result code
- */
-static int
-reply_refund_conflict (struct MHD_Connection *connection,
-                       const struct TALER_EXCHANGEDB_TransactionList *tl)
-{
-  return TALER_MHD_reply_json_pack (connection,
-                                    MHD_HTTP_CONFLICT,
-                                    "{s:s, s:I, s:o}",
-                                    "hint", "conflicting refund",
-                                    "code",
-                                    (json_int_t) TALER_EC_REFUND_CONFLICT,
-                                    "history",
-                                    TEH_RESPONSE_compile_transaction_history (
-                                      tl));
-}
-
-
 /**
  * Execute a "/refund" transaction.  Returns a confirmation that the
  * refund was successful, or a failure if we are not aware of a
@@ -152,20 +108,12 @@ refund_transaction (void *cls,
 {
   const struct TALER_EXCHANGEDB_Refund *refund = cls;
   struct TALER_EXCHANGEDB_TransactionList *tl;
-  const struct TALER_EXCHANGEDB_Deposit *dep;
-  const struct TALER_EXCHANGEDB_Refund *ref;
-  struct TEH_KS_StateHandle *mks;
-  struct TALER_EXCHANGEDB_DenominationKeyIssueInformation *dki;
-  struct TALER_Amount expect_fee;
+  const struct TALER_EXCHANGEDB_DepositListEntry *dep;
+  const struct TALER_EXCHANGEDB_RefundListEntry *ref;
   enum GNUNET_DB_QueryStatus qs;
   int deposit_found;
   int refund_found;
-  int fee_cmp;
-  unsigned int hc;
-  enum TALER_ErrorCode ec;
 
-  dep = NULL;
-  ref = NULL;
   tl = NULL;
   qs = TEH_plugin->get_coin_transactions (TEH_plugin->cls,
                                           session,
@@ -175,11 +123,14 @@ refund_transaction (void *cls,
   if (0 > qs)
   {
     if (GNUNET_DB_STATUS_HARD_ERROR == qs)
-      *mhd_ret = reply_refund_failure (connection,
-                                       MHD_HTTP_NOT_FOUND,
-                                       TALER_EC_REFUND_COIN_NOT_FOUND);
+      *mhd_ret = TALER_MHD_reply_with_error (connection,
+                                             MHD_HTTP_NOT_FOUND,
+                                             TALER_EC_REFUND_COIN_NOT_FOUND,
+                                             "database transaction failure");
     return qs;
   }
+  dep = NULL;
+  ref = NULL;
   deposit_found = GNUNET_NO;
   refund_found = GNUNET_NO;
   for (struct TALER_EXCHANGEDB_TransactionList *tlp = tl;
@@ -192,10 +143,10 @@ refund_transaction (void *cls,
       if (GNUNET_NO == deposit_found)
       {
         if ( (0 == memcmp (&tlp->details.deposit->merchant_pub,
-                           &refund->merchant_pub,
+                           &refund->details.merchant_pub,
                            sizeof (struct TALER_MerchantPublicKeyP))) &&
              (0 == memcmp (&tlp->details.deposit->h_contract_terms,
-                           &refund->h_contract_terms,
+                           &refund->details.h_contract_terms,
                            sizeof (struct GNUNET_HashCode))) )
         {
           dep = tlp->details.deposit;
@@ -204,7 +155,7 @@ refund_transaction (void *cls,
         }
       }
       break;
-    case TALER_EXCHANGEDB_TT_REFRESH_MELT:
+    case TALER_EXCHANGEDB_TT_MELT:
       /* Melts cannot be refunded, ignore here */
       break;
     case TALER_EXCHANGEDB_TT_REFUND:
@@ -212,12 +163,13 @@ refund_transaction (void *cls,
       {
         /* First, check if existing refund request is identical */
         if ( (0 == memcmp (&tlp->details.refund->merchant_pub,
-                           &refund->merchant_pub,
+                           &refund->details.merchant_pub,
                            sizeof (struct TALER_MerchantPublicKeyP))) &&
              (0 == memcmp (&tlp->details.refund->h_contract_terms,
-                           &refund->h_contract_terms,
+                           &refund->details.h_contract_terms,
                            sizeof (struct GNUNET_HashCode))) &&
-             (tlp->details.refund->rtransaction_id == refund->rtransaction_id) )
+             (tlp->details.refund->rtransaction_id ==
+              refund->details.rtransaction_id) )
         {
           ref = tlp->details.refund;
           refund_found = GNUNET_YES;
@@ -225,14 +177,14 @@ refund_transaction (void *cls,
         }
         /* Second, check if existing refund request conflicts */
         if ( (0 == memcmp (&tlp->details.refund->merchant_pub,
-                           &refund->merchant_pub,
+                           &refund->details.merchant_pub,
                            sizeof (struct TALER_MerchantPublicKeyP))) &&
              (0 == memcmp (&tlp->details.refund->h_contract_terms,
-                           &refund->h_contract_terms,
+                           &refund->details.h_contract_terms,
                            sizeof (struct GNUNET_HashCode))) &&
-             (tlp->details.refund->rtransaction_id != refund->rtransaction_id) )
+             (tlp->details.refund->rtransaction_id !=
+              refund->details.rtransaction_id) )
         {
-          GNUNET_break_op (0); /* conflicting refund found */
           refund_found = GNUNET_SYSERR;
           /* NOTE: Alternatively we could total up all existing
              refunds and check if the sum still permits the
@@ -242,14 +194,14 @@ refund_transaction (void *cls,
         }
       }
       break;
-    case TALER_EXCHANGEDB_TT_OLD_COIN_PAYBACK:
-      /* Paybacks cannot be refunded, ignore here */
+    case TALER_EXCHANGEDB_TT_OLD_COIN_RECOUP:
+      /* Recoups cannot be refunded, ignore here */
       break;
-    case TALER_EXCHANGEDB_TT_PAYBACK:
-      /* Paybacks cannot be refunded, ignore here */
+    case TALER_EXCHANGEDB_TT_RECOUP:
+      /* Recoups cannot be refunded, ignore here */
       break;
-    case TALER_EXCHANGEDB_TT_PAYBACK_REFRESH:
-      /* Paybacks cannot be refunded, ignore here */
+    case TALER_EXCHANGEDB_TT_RECOUP_REFRESH:
+      /* Recoups cannot be refunded, ignore here */
       break;
     }
   }
@@ -268,8 +220,15 @@ refund_transaction (void *cls,
   /* handle if conflicting refund found */
   if (GNUNET_SYSERR == refund_found)
   {
-    *mhd_ret = reply_refund_conflict (connection,
-                                      tl);
+    *mhd_ret = TALER_MHD_reply_json_pack (
+      connection,
+      MHD_HTTP_CONFLICT,
+      "{s:s, s:I, s:o}",
+      "hint", "conflicting refund",
+      "code", (json_int_t) TALER_EC_REFUND_CONFLICT,
+      "history", TEH_RESPONSE_compile_transaction_history (
+        &refund->coin.coin_pub,
+        tl));
     TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
                                             tl);
     return GNUNET_DB_STATUS_HARD_ERROR;
@@ -279,6 +238,7 @@ refund_transaction (void *cls,
   {
     /* /refund already done, simply re-transmit confirmation */
     *mhd_ret = reply_refund_success (connection,
+                                     &refund->coin.coin_pub,
                                      ref);
     TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
                                             tl);
@@ -287,23 +247,29 @@ refund_transaction (void *cls,
 
   /* check currency is compatible */
   if ( (GNUNET_YES !=
-        TALER_amount_cmp_currency (&refund->refund_amount,
+        TALER_amount_cmp_currency (&refund->details.refund_amount,
                                    &dep->amount_with_fee)) ||
        (GNUNET_YES !=
-        TALER_amount_cmp_currency (&refund->refund_fee,
+        TALER_amount_cmp_currency (&refund->details.refund_fee,
                                    &dep->deposit_fee)) )
   {
-    GNUNET_break_op (0); /* currency missmatch */
-    *mhd_ret = reply_refund_failure (connection,
-                                     MHD_HTTP_PRECONDITION_FAILED,
-                                     TALER_EC_REFUND_CURRENCY_MISSMATCH);
+    GNUNET_break_op (0); /* currency mismatch */
+    TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
+                                            tl);
+    *mhd_ret = TALER_MHD_reply_with_error (connection,
+                                           MHD_HTTP_PRECONDITION_FAILED,
+                                           TALER_EC_REFUND_CURRENCY_MISSMATCH,
+                                           "currencies involved do not match");
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
 
   /* check if we already send the money for the /deposit */
   qs = TEH_plugin->test_deposit_done (TEH_plugin->cls,
                                       session,
-                                      dep);
+                                      &refund->coin.coin_pub,
+                                      &dep->merchant_pub,
+                                      &dep->h_contract_terms,
+                                      &dep->h_wire);
   if (GNUNET_DB_STATUS_HARD_ERROR == qs)
   {
     /* Internal error, we first had the deposit in the history,
@@ -314,7 +280,7 @@ refund_transaction (void *cls,
     *mhd_ret = TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_INTERNAL_SERVER_ERROR,
                                            TALER_EC_REFUND_DB_INCONSISTENT,
-                                           "database inconsistent");
+                                           "database inconsistent (deposit data became inaccessible during transaction)");
     return qs;
   }
   if (GNUNET_DB_STATUS_SOFT_ERROR == qs)
@@ -325,79 +291,26 @@ refund_transaction (void *cls,
     /* money was already transferred to merchant, can no longer refund */
     TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
                                             tl);
-    *mhd_ret = reply_refund_failure (connection,
-                                     MHD_HTTP_GONE,
-                                     TALER_EC_REFUND_MERCHANT_ALREADY_PAID);
+    *mhd_ret = TALER_MHD_reply_with_error (connection,
+                                           MHD_HTTP_GONE,
+                                           TALER_EC_REFUND_MERCHANT_ALREADY_PAID,
+                                           "money already sent to merchant");
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
 
   /* check refund amount is sufficiently low */
-  if (1 == TALER_amount_cmp (&refund->refund_amount,
+  if (1 == TALER_amount_cmp (&refund->details.refund_amount,
                              &dep->amount_with_fee) )
   {
     GNUNET_break_op (0); /* cannot refund more than original value */
-    TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
-                                            tl);
-    *mhd_ret = reply_refund_failure (connection,
-                                     MHD_HTTP_PRECONDITION_FAILED,
-                                     TALER_EC_REFUND_INSUFFICIENT_FUNDS);
-    return GNUNET_DB_STATUS_HARD_ERROR;
-  }
-
-  // FIXME: do this outside of transaction function?
-  /* Check refund fee matches fee of denomination key! */
-  mks = TEH_KS_acquire (GNUNET_TIME_absolute_get ());
-  if (NULL == mks)
-  {
-    TALER_LOG_ERROR ("Lacking keys to operate\n");
     TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
                                             tl);
     *mhd_ret = TALER_MHD_reply_with_error (connection,
-                                           MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                           TALER_EC_EXCHANGE_BAD_CONFIGURATION,
-                                           "no keys");
+                                           MHD_HTTP_PRECONDITION_FAILED,
+                                           TALER_EC_REFUND_INSUFFICIENT_FUNDS,
+                                           "refund requested exceeds original value");
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
-  dki = TEH_KS_denomination_key_lookup_by_hash (mks,
-                                                &dep->coin.denom_pub_hash,
-                                                TEH_KS_DKU_DEPOSIT,
-                                                &ec,
-                                                &hc);
-  if (NULL == dki)
-  {
-    /* DKI not found, but we do have a coin with this DK in our database;
-       not good... */
-    GNUNET_break (0);
-    TEH_KS_release (mks);
-    TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
-                                            tl);
-    *mhd_ret = TALER_MHD_reply_with_error (connection,
-                                           hc,
-                                           ec,
-                                           "denomination not found, but coin known");
-    return GNUNET_DB_STATUS_HARD_ERROR;
-  }
-  TALER_amount_ntoh (&expect_fee,
-                     &dki->issue.properties.fee_refund);
-  fee_cmp = TALER_amount_cmp (&refund->refund_fee,
-                              &expect_fee);
-  TEH_KS_release (mks);
-
-  if (-1 == fee_cmp)
-  {
-    TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
-                                            tl);
-    *mhd_ret = TALER_MHD_reply_with_error (connection,
-                                           MHD_HTTP_BAD_REQUEST,
-                                           TALER_EC_REFUND_FEE_TOO_LOW,
-                                           "refund_fee");
-    return GNUNET_DB_STATUS_HARD_ERROR;
-  }
-  if (1 == fee_cmp)
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "Refund fee proposed by merchant is higher than necessary.\n");
-  }
   TEH_plugin->free_coin_transaction_list (TEH_plugin->cls,
                                           tl);
 
@@ -433,22 +346,99 @@ static int
 verify_and_execute_refund (struct MHD_Connection *connection,
                            const struct TALER_EXCHANGEDB_Refund *refund)
 {
-  struct TALER_RefundRequestPS rr;
-  int mhd_ret;
+  struct GNUNET_HashCode denom_hash;
+  struct TALER_Amount expect_fee;
 
-  rr.purpose.purpose = htonl (TALER_SIGNATURE_MERCHANT_REFUND);
-  rr.purpose.size = htonl (sizeof (struct TALER_RefundRequestPS));
-  rr.h_contract_terms = refund->h_contract_terms;
-  rr.coin_pub = refund->coin.coin_pub;
-  rr.merchant = refund->merchant_pub;
-  rr.rtransaction_id = GNUNET_htonll (refund->rtransaction_id);
-  TALER_amount_hton (&rr.refund_amount,
-                     &refund->refund_amount);
-  TALER_amount_hton (&rr.refund_fee,
-                     &refund->refund_fee);
+  {
+    struct TALER_RefundRequestPS rr = {
+      .purpose.purpose = htonl (TALER_SIGNATURE_MERCHANT_REFUND),
+      .purpose.size = htonl (sizeof (rr)),
+      .h_contract_terms = refund->details.h_contract_terms,
+      .coin_pub = refund->coin.coin_pub,
+      .merchant = refund->details.merchant_pub,
+      .rtransaction_id = GNUNET_htonll (refund->details.rtransaction_id)
+    };
+
+    TALER_amount_hton (&rr.refund_amount,
+                       &refund->details.refund_amount);
+    TALER_amount_hton (&rr.refund_fee,
+                       &refund->details.refund_fee);
+    if (GNUNET_OK !=
+        GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_MERCHANT_REFUND,
+                                    &rr.purpose,
+                                    &refund->details.merchant_sig.eddsa_sig,
+                                    &refund->details.merchant_pub.eddsa_pub))
+    {
+      TALER_LOG_WARNING ("Invalid signature on refund request\n");
+      return TALER_MHD_reply_with_error (connection,
+                                         MHD_HTTP_FORBIDDEN,
+                                         TALER_EC_REFUND_MERCHANT_SIGNATURE_INVALID,
+                                         "merchant_sig");
+    }
+  }
+
+  /* Fetch the coin's denomination (hash) */
+  {
+    enum GNUNET_DB_QueryStatus qs;
+
+    qs = TEH_plugin->get_coin_denomination (TEH_plugin->cls,
+                                            NULL,
+                                            &refund->coin.coin_pub,
+                                            &denom_hash);
+    if (0 > qs)
+    {
+      GNUNET_break (GNUNET_DB_STATUS_HARD_ERROR == qs);
+      return TALER_MHD_reply_with_error (connection,
+                                         MHD_HTTP_NOT_FOUND,
+                                         TALER_EC_REFUND_COIN_NOT_FOUND,
+                                         "denomination of coin to be refunded not found in DB");
+    }
+  }
+
+  {
+    struct TEH_KS_StateHandle *key_state;
+
+    key_state = TEH_KS_acquire (GNUNET_TIME_absolute_get ());
+    if (NULL == key_state)
+    {
+      TALER_LOG_ERROR ("Lacking keys to operate\n");
+      return TALER_MHD_reply_with_error (connection,
+                                         MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                         TALER_EC_EXCHANGE_BAD_CONFIGURATION,
+                                         "no keys");
+    }
+    /* Obtain information about the coin's denomination! */
+    {
+      struct TALER_EXCHANGEDB_DenominationKey *dki;
+      unsigned int hc;
+      enum TALER_ErrorCode ec;
+
+      dki = TEH_KS_denomination_key_lookup_by_hash (key_state,
+                                                    &denom_hash,
+                                                    TEH_KS_DKU_DEPOSIT,
+                                                    &ec,
+                                                    &hc);
+      if (NULL == dki)
+      {
+        /* DKI not found, but we do have a coin with this DK in our database;
+           not good... */
+        GNUNET_break (0);
+        TEH_KS_release (key_state);
+        return TALER_MHD_reply_with_error (connection,
+                                           hc,
+                                           ec,
+                                           "denomination not found, but coin known");
+      }
+      TALER_amount_ntoh (&expect_fee,
+                         &dki->issue.properties.fee_refund);
+    }
+    TEH_KS_release (key_state);
+  }
+
+  /* Check refund fee matches fee of denomination key! */
   if (GNUNET_YES !=
-      TALER_amount_cmp_currency (&refund->refund_amount,
-                                 &refund->refund_fee) )
+      TALER_amount_cmp_currency (&expect_fee,
+                                 &refund->details.refund_fee) )
   {
     GNUNET_break_op (0);
     return TALER_MHD_reply_with_error (connection,
@@ -456,96 +446,116 @@ verify_and_execute_refund (struct MHD_Connection *connection,
                                        TALER_EC_REFUND_FEE_CURRENCY_MISSMATCH,
                                        "refund_fee");
   }
-  if (-1 == TALER_amount_cmp (&refund->refund_amount,
-                              &refund->refund_fee) )
+  {
+    int fee_cmp;
+
+    fee_cmp = TALER_amount_cmp (&refund->details.refund_fee,
+                                &expect_fee);
+    if (-1 == fee_cmp)
+    {
+      return TALER_MHD_reply_with_error (connection,
+                                         MHD_HTTP_BAD_REQUEST,
+                                         TALER_EC_REFUND_FEE_TOO_LOW,
+                                         "refund_fee");
+    }
+    if (1 == fee_cmp)
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                  "Refund fee proposed by merchant is higher than necessary.\n");
+    }
+  }
+
+
+  /* Finally run the actual transaction logic */
+  {
+    int mhd_ret;
+
+    if (GNUNET_OK !=
+        TEH_DB_run_transaction (connection,
+                                "run refund",
+                                &mhd_ret,
+                                &refund_transaction,
+                                (void *) refund))
+    {
+      return mhd_ret;
+    }
+  }
+  return reply_refund_success (connection,
+                               &refund->coin.coin_pub,
+                               &refund->details);
+}
+
+
+/**
+ * Handle a "/coins/$COIN_PUB/refund" request.  Parses the JSON, and, if
+ * successful, passes the JSON data to #verify_and_execute_refund() to further
+ * check the details of the operation specified.  If everything checks out,
+ * this will ultimately lead to the refund being executed, or rejected.
+ *
+ * @param connection the MHD connection to handle
+ * @param coin_pub public key of the coin
+ * @param root uploaded JSON data
+ * @return MHD result code
+  */
+int
+TEH_handler_refund (struct MHD_Connection *connection,
+                    const struct TALER_CoinSpendPublicKeyP *coin_pub,
+                    const json_t *root)
+{
+  struct TALER_EXCHANGEDB_Refund refund;
+  struct GNUNET_JSON_Specification spec[] = {
+    TALER_JSON_spec_amount ("refund_amount", &refund.details.refund_amount),
+    TALER_JSON_spec_amount ("refund_fee", &refund.details.refund_fee),
+    GNUNET_JSON_spec_fixed_auto ("h_contract_terms",
+                                 &refund.details.h_contract_terms),
+    GNUNET_JSON_spec_fixed_auto ("merchant_pub", &refund.details.merchant_pub),
+    GNUNET_JSON_spec_uint64 ("rtransaction_id",
+                             &refund.details.rtransaction_id),
+    GNUNET_JSON_spec_fixed_auto ("merchant_sig", &refund.details.merchant_sig),
+    GNUNET_JSON_spec_end ()
+  };
+
+  refund.coin.coin_pub = *coin_pub;
+  {
+    int res;
+
+    res = TALER_MHD_parse_json_data (connection,
+                                     root,
+                                     spec);
+    if (GNUNET_SYSERR == res)
+      return MHD_NO; /* hard failure */
+    if (GNUNET_NO == res)
+      return MHD_YES; /* failure */
+  }
+  if (GNUNET_YES !=
+      TALER_amount_cmp_currency (&refund.details.refund_amount,
+                                 &refund.details.refund_fee) )
   {
     GNUNET_break_op (0);
+    GNUNET_JSON_parse_free (spec);
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_BAD_REQUEST,
+                                       TALER_EC_REFUND_FEE_CURRENCY_MISSMATCH,
+                                       "refund_amount or refund_fee");
+  }
+  if (-1 == TALER_amount_cmp (&refund.details.refund_amount,
+                              &refund.details.refund_fee) )
+  {
+    GNUNET_break_op (0);
+    GNUNET_JSON_parse_free (spec);
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_BAD_REQUEST,
                                        TALER_EC_REFUND_FEE_ABOVE_AMOUNT,
                                        "refund_amount");
   }
-  if (GNUNET_OK !=
-      GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_MERCHANT_REFUND,
-                                  &rr.purpose,
-                                  &refund->merchant_sig.eddsa_sig,
-                                  &refund->merchant_pub.eddsa_pub))
   {
-    TALER_LOG_WARNING ("Invalid signature on /refund request\n");
-    return TALER_MHD_reply_with_error (connection,
-                                       MHD_HTTP_FORBIDDEN,
-                                       TALER_EC_REFUND_MERCHANT_SIGNATURE_INVALID,
-                                       "merchant_sig");
+    int res;
+
+    res = verify_and_execute_refund (connection,
+                                     &refund);
+    GNUNET_JSON_parse_free (spec);
+    return res;
   }
-  if (GNUNET_OK !=
-      TEH_DB_run_transaction (connection,
-                              "run refund",
-                              &mhd_ret,
-                              &refund_transaction,
-                              (void *) refund))
-    return mhd_ret;
-  return reply_refund_success (connection,
-                               refund);
-}
-
-
-/**
- * Handle a "/refund" request.  Parses the JSON, and, if successful,
- * passes the JSON data to #verify_and_execute_refund() to
- * further check the details of the operation specified.  If
- * everything checks out, this will ultimately lead to the "/refund"
- * being executed, or rejected.
- *
- * @param rh context of the handler
- * @param connection the MHD connection to handle
- * @param[in,out] connection_cls the connection's closure (can be updated)
- * @param upload_data upload data
- * @param[in,out] upload_data_size number of bytes (left) in @a upload_data
- * @return MHD result code
-  */
-int
-TEH_REFUND_handler_refund (struct TEH_RequestHandler *rh,
-                           struct MHD_Connection *connection,
-                           void **connection_cls,
-                           const char *upload_data,
-                           size_t *upload_data_size)
-{
-  json_t *json;
-  int res;
-  struct TALER_EXCHANGEDB_Refund refund;
-  struct GNUNET_JSON_Specification spec[] = {
-    TALER_JSON_spec_amount ("refund_amount", &refund.refund_amount),
-    TALER_JSON_spec_amount ("refund_fee", &refund.refund_fee),
-    GNUNET_JSON_spec_fixed_auto ("h_contract_terms", &refund.h_contract_terms),
-    GNUNET_JSON_spec_fixed_auto ("coin_pub", &refund.coin.coin_pub),
-    GNUNET_JSON_spec_fixed_auto ("merchant_pub", &refund.merchant_pub),
-    GNUNET_JSON_spec_uint64 ("rtransaction_id", &refund.rtransaction_id),
-    GNUNET_JSON_spec_fixed_auto ("merchant_sig", &refund.merchant_sig),
-    GNUNET_JSON_spec_end ()
-  };
-
-  (void) rh;
-  res = TALER_MHD_parse_post_json (connection,
-                                   connection_cls,
-                                   upload_data,
-                                   upload_data_size,
-                                   &json);
-  if (GNUNET_SYSERR == res)
-    return MHD_NO;
-  if ( (GNUNET_NO == res) || (NULL == json) )
-    return MHD_YES;
-  res = TALER_MHD_parse_json_data (connection,
-                                   json,
-                                   spec);
-  json_decref (json);
-  if (GNUNET_SYSERR == res)
-    return MHD_NO; /* hard failure */
-  if (GNUNET_NO == res)
-    return MHD_YES; /* failure */
-  res = verify_and_execute_refund (connection,
-                                   &refund);
-  GNUNET_JSON_parse_free (spec);
-  return res;
 }
 
 
