@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014, 2015, 2016 Inria and GNUnet e.V.
+  Copyright (C) 2014--2020 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -29,21 +29,19 @@
 /**
  * Initialize wire subsystem.
  *
+ * @param cfg configuration to use
  * @return #GNUNET_OK on success, #GNUNET_SYSERR if we found no valid
  *         wire methods
  */
 int
-TEH_WIRE_init (void);
+TEH_WIRE_init (const struct GNUNET_CONFIGURATION_Handle *cfg);
 
 
 /**
- * Obtain fee structure for @a wire_plugin_name wire transfers.
- *
- * @param method method to load fees for
- * @return JSON object (to be freed by caller) with fee structure
+ * Clean up wire subsystem.
  */
-json_t *
-TEH_WIRE_get_fees (const char *method);
+void
+TEH_WIRE_done (void);
 
 
 /**
@@ -51,17 +49,13 @@ TEH_WIRE_get_fees (const char *method);
  *
  * @param rh context of the handler
  * @param connection the MHD connection to handle
- * @param[in,out] connection_cls the connection's closure (can be updated)
- * @param upload_data upload data
- * @param[in,out] upload_data_size number of bytes (left) in @a upload_data
+ * @param args array of additional options (must be empty for this function)
  * @return MHD result code
-  */
+ */
 int
-TEH_WIRE_handler_wire (struct TEH_RequestHandler *rh,
-                       struct MHD_Connection *connection,
-                       void **connection_cls,
-                       const char *upload_data,
-                       size_t *upload_data_size);
+TEH_handler_wire (const struct TEH_RequestHandler *rh,
+                  struct MHD_Connection *connection,
+                  const char *const args[]);
 
 
 #endif
