@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2016, 2017, 2018 Taler Systems SA
+  Copyright (C) 2016--2020 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -27,31 +27,15 @@
 #include "taler_exchangedb_lib.h"
 #include "taler_exchangedb_plugin.h"
 #include "taler_json_lib.h"
-#include "taler_wire_lib.h"
+#include "taler_bank_service.h"
+
+#define DEBUG_LOGGING 0
 
 /**
- * How long do we sleep before trying again if there
- * are no transactions returned by the wire plugin?
+ * What is the initial batch size we use for credit history
+ * requests with the bank.  See `batch_size` below.
  */
-#define DELAY GNUNET_TIME_UNIT_SECONDS
-
-
-/**
- * Closure for #reject_cb().
- */
-struct RejectContext
-{
-  /**
-   * Wire transfer subject that was illformed.
-   */
-  char *wtid_s;
-
-  /**
-   * Database session that encountered the problem.
-   */
-  struct TALER_EXCHANGEDB_Session *session;
-};
-
+#define INITIAL_BATCH_SIZE 1024
 
 /**
  * Information we keep for each supported account.
@@ -68,16 +52,53 @@ struct WireAccount
    */
   struct WireAccount *prev;
 
-  /**
-   * Handle to the plugin.
-   */
-  struct TALER_WIRE_Plugin *wire_plugin;
-
   /**
    * Name of the section that configures this account.
    */
   char *section_name;
 
+  /**
+   * Database session we are using for the current transaction.
+   */
+  struct TALER_EXCHANGEDB_Session *session;
+
+  /**
+   * Active request for history.
+   */
+  struct TALER_BANK_CreditHistoryHandle *hh;
+
+  /**
+   * Authentication data.
+   */
+  struct TALER_BANK_AuthenticationData auth;
+
+  /**
+   * Until when is processing this wire plugin delayed?
+   */
+  struct GNUNET_TIME_Absolute delayed_until;
+
+  /**
+   * Encoded offset in the wire transfer list from where
+   * to start the next query with the bank.
+   */
+  uint64_t last_row_off;
+
+  /**
+   * Latest row offset seen in this transaction, becomes
+   * the new #last_row_off upon commit.
+   */
+  uint64_t latest_row_off;
+
+  /**
+   * How many transactions do we retrieve per batch?
+   */
+  unsigned int batch_size;
+
+  /**
+   * How many transactions did we see in the current batch?
+   */
+  unsigned int current_batch_size;
+
   /**
    * Are we running from scratch and should re-process all transactions
    * for this account?
@@ -85,9 +106,10 @@ struct WireAccount
   int reset_mode;
 
   /**
-   * Until when is processing this wire plugin delayed?
+   * Should we delay the next request to the wire plugin a bit?  Set to
+   * #GNUNET_NO if we actually did some work.
    */
-  struct GNUNET_TIME_Absolute delayed_until;
+  int delay;
 
 };
 
@@ -103,14 +125,20 @@ static struct WireAccount *wa_head;
 static struct WireAccount *wa_tail;
 
 /**
- * Wire plugin we are currently using.
+ * Wire account we are currently processing.  This would go away
+ * if we ever start processing all accounts in parallel.
  */
 static struct WireAccount *wa_pos;
 
 /**
- * Which currency is used by this exchange?
+ * Handle to the context for interacting with the bank.
  */
-static char *exchange_currency_string;
+static struct GNUNET_CURL_Context *ctx;
+
+/**
+ * Scheduler context for running the @e ctx.
+ */
+static struct GNUNET_CURL_RescheduleContext *rc;
 
 /**
  * The exchange's configuration (global)
@@ -123,37 +151,25 @@ static const struct GNUNET_CONFIGURATION_Handle *cfg;
 static struct TALER_EXCHANGEDB_Plugin *db_plugin;
 
 /**
- * Value to return from main(). #GNUNET_OK on success, #GNUNET_SYSERR
+ * How long should we sleep when idle before trying to find more work?
+ */
+static struct GNUNET_TIME_Relative wirewatch_idle_sleep_interval;
+
+/**
+ * Value to return from main(). 0 on success, non-zero on
  * on serious errors.
  */
-static int global_ret;
-
-/**
- * Encoded offset in the wire transfer list from where
- * to start the next query with the bank.
- */
-static void *last_row_off;
-
-/**
- * Number of bytes in #last_row_off.
- */
-static size_t last_row_off_size;
-
-/**
- * Latest row offset seen in this transaction, becomes
- * the new #last_row_off upon commit.
- */
-static void *latest_row_off;
-
-/**
- * Number of bytes in #latest_row_off.
- */
-static size_t latest_row_off_size;
-
-/**
- * Should we delay the next request to the wire plugin a bit?
- */
-static int delay;
+static enum
+{
+  GR_SUCCESS = 0,
+  GR_DATABASE_SESSION_FAIL = 1,
+  GR_DATABASE_TRANSACTION_BEGIN_FAIL = 2,
+  GR_DATABASE_SELECT_LATEST_HARD_FAIL = 3,
+  GR_BANK_REQUEST_HISTORY_FAIL = 4,
+  GR_CONFIGURATION_INVALID = 5,
+  GR_CMD_LINE_UTF8_ERROR = 6,
+  GR_CMD_LINE_OPTIONS_WRONG = 7,
+} global_ret;
 
 /**
  * Are we run in testing mode and should only do one pass?
@@ -166,30 +182,10 @@ static int test_mode;
 static int reset_mode;
 
 /**
- * How many transactions do we retrieve per batch?
- */
-static unsigned int batch_size = 1024;
-
-/**
- * How many transactions did we see in the current batch?
- */
-static unsigned int current_batch_size;
-
-/**
- * Next task to run, if any.
+ * Current task waiting for execution, if any.
  */
 static struct GNUNET_SCHEDULER_Task *task;
 
-/**
- * Active request for history.
- */
-static struct TALER_WIRE_HistoryHandle *hh;
-
-/**
- * Active request to reject a wire transfer.
- */
-static struct TALER_WIRE_RejectHandle *rt;
-
 
 /**
  * We're being aborted with CTRL-C (or SIGTERM). Shut down.
@@ -199,45 +195,44 @@ static struct TALER_WIRE_RejectHandle *rt;
 static void
 shutdown_task (void *cls)
 {
-  struct WireAccount *wa;
-
   (void) cls;
+  {
+    struct WireAccount *wa;
+
+    while (NULL != (wa = wa_head))
+    {
+      if (NULL != wa->hh)
+      {
+        TALER_BANK_credit_history_cancel (wa->hh);
+        wa->hh = NULL;
+      }
+      GNUNET_CONTAINER_DLL_remove (wa_head,
+                                   wa_tail,
+                                   wa);
+      TALER_BANK_auth_free (&wa->auth);
+      GNUNET_free (wa->section_name);
+      GNUNET_free (wa);
+    }
+  }
+  wa_pos = NULL;
+
+  if (NULL != ctx)
+  {
+    GNUNET_CURL_fini (ctx);
+    ctx = NULL;
+  }
+  if (NULL != rc)
+  {
+    GNUNET_CURL_gnunet_rc_destroy (rc);
+    rc = NULL;
+  }
   if (NULL != task)
   {
     GNUNET_SCHEDULER_cancel (task);
     task = NULL;
   }
-  if (NULL != hh)
-  {
-    wa_pos->wire_plugin->get_history_cancel (wa_pos->wire_plugin->cls,
-                                             hh);
-    hh = NULL;
-  }
-  if (NULL != rt)
-  {
-    char *wtid_s;
-
-    wtid_s = wa_pos->wire_plugin->reject_transfer_cancel (
-      wa_pos->wire_plugin->cls,
-      rt);
-    rt = NULL;
-    GNUNET_free (wtid_s);
-  }
   TALER_EXCHANGEDB_plugin_unload (db_plugin);
   db_plugin = NULL;
-  while (NULL != (wa = wa_head))
-  {
-    GNUNET_CONTAINER_DLL_remove (wa_head,
-                                 wa_tail,
-                                 wa);
-    TALER_WIRE_plugin_unload (wa->wire_plugin);
-    GNUNET_free (wa->section_name);
-    GNUNET_free (wa);
-  }
-  wa_pos = NULL;
-  GNUNET_free_non_null (last_row_off);
-  last_row_off = NULL;
-  last_row_off_size = 0;
 }
 
 
@@ -259,17 +254,19 @@ add_account_cb (void *cls,
     return; /* not enabled for us, skip */
   wa = GNUNET_new (struct WireAccount);
   wa->reset_mode = reset_mode;
-  wa->wire_plugin = TALER_WIRE_plugin_load (cfg,
-                                            ai->plugin_name);
-  if (NULL == wa->wire_plugin)
+  if (GNUNET_OK !=
+      TALER_BANK_auth_parse_cfg (cfg,
+                                 ai->section_name,
+                                 &wa->auth))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_MESSAGE,
-                "Failed to load wire plugin for `%s'\n",
-                ai->plugin_name);
+                "Failed to load account `%s'\n",
+                ai->section_name);
     GNUNET_free (wa);
     return;
   }
   wa->section_name = GNUNET_strdup (ai->section_name);
+  wa->batch_size = INITIAL_BATCH_SIZE;
   GNUNET_CONTAINER_DLL_insert (wa_head,
                                wa_tail,
                                wa);
@@ -283,28 +280,19 @@ add_account_cb (void *cls,
  * @return #GNUNET_OK on success
  */
 static int
-exchange_serve_process_config ()
+exchange_serve_process_config (void)
 {
   if (GNUNET_OK !=
-      GNUNET_CONFIGURATION_get_value_string (cfg,
-                                             "taler",
-                                             "CURRENCY",
-                                             &exchange_currency_string))
+      GNUNET_CONFIGURATION_get_value_time (cfg,
+                                           "exchange",
+                                           "WIREWATCH_IDLE_SLEEP_INTERVAL",
+                                           &wirewatch_idle_sleep_interval))
   {
     GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
-                               "taler",
-                               "CURRENCY");
+                               "exchange",
+                               "WIREWATCH_IDLE_SLEEP_INTERVAL");
     return GNUNET_SYSERR;
   }
-  if (strlen (exchange_currency_string) >= TALER_CURRENCY_LEN)
-  {
-    fprintf (stderr,
-             "Currency `%s' longer than the allowed limit of %u characters.",
-             exchange_currency_string,
-             (unsigned int) TALER_CURRENCY_LEN);
-    return GNUNET_SYSERR;
-  }
-
   if (NULL ==
       (db_plugin = TALER_EXCHANGEDB_plugin_load (cfg)))
   {
@@ -335,78 +323,40 @@ static void
 find_transfers (void *cls);
 
 
-/**
- * Function called upon completion of the rejection of a wire transfer.
- *
- * @param cls closure with the `struct RejectContext`
- * @param ec error code for the operation
- */
-static void
-reject_cb (void *cls,
-           enum TALER_ErrorCode ec)
-{
-  struct RejectContext *rtc = cls;
-  enum GNUNET_DB_QueryStatus qs;
-
-  rt = NULL;
-  if (TALER_EC_NONE != ec)
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Failed to wire back transfer `%s': %d\n",
-                rtc->wtid_s,
-                ec);
-    GNUNET_free (rtc->wtid_s);
-    db_plugin->rollback (db_plugin->cls,
-                         rtc->session);
-    GNUNET_free (rtc);
-    GNUNET_SCHEDULER_shutdown ();
-    return;
-  }
-  GNUNET_free (rtc->wtid_s);
-  qs = db_plugin->commit (db_plugin->cls,
-                          rtc->session);
-  GNUNET_break (0 <= qs);
-  GNUNET_free (rtc);
-  task = GNUNET_SCHEDULER_add_now (&find_transfers,
-                                   NULL);
-}
-
-
 /**
  * Callbacks of this type are used to serve the result of asking
  * the bank for the transaction history.
  *
- * @param cls closure with the `struct TALER_EXCHANGEDB_Session *`
+ * @param cls closure with the `struct WioreAccount *` we are processing
+ * @param http_status HTTP status code from the server
  * @param ec taler error code
- * @param dir direction of the transfer
- * @param row_off identification of the position at which we are querying
- * @param row_off_size number of bytes in @a row_off
+ * @param serial_id identification of the position at which we are querying
  * @param details details about the wire transfer
+ * @param json raw JSON response
  * @return #GNUNET_OK to continue, #GNUNET_SYSERR to abort iteration
  */
 static int
 history_cb (void *cls,
+            unsigned int http_status,
             enum TALER_ErrorCode ec,
-            enum TALER_BANK_Direction dir,
-            const void *row_off,
-            size_t row_off_size,
-            const struct TALER_WIRE_TransferDetails *details)
+            uint64_t serial_id,
+            const struct TALER_BANK_CreditDetails *details,
+            const json_t *json)
 {
-  struct TALER_EXCHANGEDB_Session *session = cls;
+  struct WireAccount *wa = cls;
+  struct TALER_EXCHANGEDB_Session *session = wa->session;
   enum GNUNET_DB_QueryStatus qs;
-  struct TALER_ReservePublicKeyP reserve_pub;
 
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "Got history callback, direction %u!\n",
-              (unsigned int) dir);
-  if (TALER_BANK_DIRECTION_NONE == dir)
+  (void) json;
+  if (NULL == details)
   {
-    hh = NULL;
+    wa->hh = NULL;
     if (TALER_EC_NONE != ec)
     {
-      GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                  "Error fetching history: %u!\n",
-                  (unsigned int) ec);
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Error fetching history: ec=%u, http_status=%u\n",
+                  (unsigned int) ec,
+                  http_status);
     }
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                 "End of list. Committing progress!\n");
@@ -417,8 +367,8 @@ history_cb (void *cls,
       GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                   "Got DB soft error for commit\n");
       /* reduce transaction size to reduce rollback probability */
-      if (2 > current_batch_size)
-        current_batch_size /= 2;
+      if (2 > wa->current_batch_size)
+        wa->current_batch_size /= 2;
       /* try again */
       GNUNET_assert (NULL == task);
       task = GNUNET_SCHEDULER_add_now (&find_transfers,
@@ -428,30 +378,28 @@ history_cb (void *cls,
     if (0 < qs)
     {
       /* transaction success, update #last_row_off */
-      GNUNET_free_non_null (last_row_off);
-      last_row_off = latest_row_off;
-      last_row_off_size = latest_row_off_size;
-      latest_row_off = NULL;
-      latest_row_off_size = 0;
-
+      wa->last_row_off = wa->latest_row_off;
+      wa->latest_row_off = 0; /* should not be needed */
+      wa->session = NULL; /* should not be needed */
       /* if successful at limit, try increasing transaction batch size (AIMD) */
-      if (current_batch_size == batch_size)
-        batch_size++;
+      if ( (wa->current_batch_size == wa->batch_size) &&
+           (UINT_MAX > wa->batch_size) )
+        wa->batch_size++;
     }
     GNUNET_break (0 <= qs);
-    if ( (GNUNET_YES == delay) &&
+    if ( (GNUNET_YES == wa->delay) &&
          (test_mode) &&
-         (NULL == wa_pos->next) )
+         (NULL == wa->next) )
     {
       GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                   "Shutdown due to test mode!\n");
       GNUNET_SCHEDULER_shutdown ();
       return GNUNET_OK;
     }
-    if (GNUNET_YES == delay)
+    if (GNUNET_YES == wa->delay)
     {
-      wa_pos->delayed_until
-        = GNUNET_TIME_relative_to_absolute (DELAY);
+      wa->delayed_until
+        = GNUNET_TIME_relative_to_absolute (wirewatch_idle_sleep_interval);
       wa_pos = wa_pos->next;
       if (NULL == wa_pos)
         wa_pos = wa_head;
@@ -462,83 +410,41 @@ history_cb (void *cls,
                                     NULL);
     return GNUNET_OK; /* will be ignored anyway */
   }
-  if (NULL != details->wtid_s)
-  {
-    struct RejectContext *rtc;
-
-    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "Wire transfer over %s has invalid subject `%s', sending it back!\n",
-                TALER_amount2s (&details->amount),
-                details->wtid_s);
-    GNUNET_break (0 != row_off_size);
-    if (latest_row_off_size != row_off_size)
-    {
-      GNUNET_free_non_null (latest_row_off);
-      latest_row_off = GNUNET_malloc (row_off_size);
-      latest_row_off_size = row_off_size;
-    }
-    memcpy (latest_row_off,
-            row_off,
-            row_off_size);
-    rtc = GNUNET_new (struct RejectContext);
-    rtc->session = session;
-    rtc->wtid_s = GNUNET_strdup (details->wtid_s);
-    rt = wa_pos->wire_plugin->reject_transfer (wa_pos->wire_plugin->cls,
-                                               wa_pos->section_name,
-                                               row_off,
-                                               row_off_size,
-                                               &reject_cb,
-                                               rtc);
-    if (NULL == rt)
-    {
-      GNUNET_break (0);
-      db_plugin->rollback (db_plugin->cls,
-                           session);
-      GNUNET_assert (NULL == task);
-      task = GNUNET_SCHEDULER_add_now (&find_transfers,
-                                       NULL);
-    }
-    return GNUNET_SYSERR; /* will continue later... */
-  }
-
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Adding wire transfer over %s with (hashed) subject `%s'\n",
               TALER_amount2s (&details->amount),
-              TALER_B2S (&details->wtid));
+              TALER_B2S (&details->reserve_pub));
 
   /**
    * Debug block.
    */
+#if DEBUG_LOGGING
   {
-/* Should be 53, give 80 just to be redundant.  */
+    /** Should be 53, give 80 just to be extra conservative (and aligned).  */
 #define PUBSIZE 80
     char wtid_s[PUBSIZE];
 
-    GNUNET_break
-      (NULL != GNUNET_STRINGS_data_to_string (&details->wtid,
-                                              sizeof (details->wtid),
-                                              &wtid_s[0],
-                                              PUBSIZE));
+    GNUNET_break (NULL !=
+                  GNUNET_STRINGS_data_to_string (&details->reserve_pub,
+                                                 sizeof (details->reserve_pub),
+                                                 &wtid_s[0],
+                                                 PUBSIZE));
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "Plain text subject (= reserve_pub): %s\n",
                 wtid_s);
   }
+#endif
 
-  current_batch_size++;
-  /* Wire transfer identifier == reserve public key */
-  GNUNET_assert (sizeof (reserve_pub) == sizeof (details->wtid));
-  memcpy (&reserve_pub,
-          &details->wtid,
-          sizeof (reserve_pub));
+  if (wa->current_batch_size < UINT_MAX)
+    wa->current_batch_size++;
   qs = db_plugin->reserves_in_insert (db_plugin->cls,
                                       session,
-                                      &reserve_pub,
+                                      &details->reserve_pub,
                                       &details->amount,
                                       details->execution_date,
-                                      details->account_url,
-                                      wa_pos->section_name,
-                                      row_off,
-                                      row_off_size);
+                                      details->debit_account_url,
+                                      wa->section_name,
+                                      serial_id);
   if (GNUNET_DB_STATUS_HARD_ERROR == qs)
   {
     GNUNET_break (0);
@@ -549,8 +455,8 @@ history_cb (void *cls,
   }
   if (GNUNET_DB_STATUS_SOFT_ERROR == qs)
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-                "Got DB soft error for reserve_in_insert\n");
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Got DB soft error for reserves_in_insert. Rolling back.\n");
     db_plugin->rollback (db_plugin->cls,
                          session);
     /* try again */
@@ -559,16 +465,8 @@ history_cb (void *cls,
                                      NULL);
     return GNUNET_SYSERR;
   }
-
-  if (latest_row_off_size != row_off_size)
-  {
-    GNUNET_free_non_null (latest_row_off);
-    latest_row_off = GNUNET_malloc (row_off_size);
-    latest_row_off_size = row_off_size;
-  }
-  memcpy (latest_row_off,
-          row_off,
-          row_off_size);
+  wa->delay = GNUNET_NO;
+  wa->latest_row_off = serial_id;
   return GNUNET_OK;
 }
 
@@ -588,12 +486,11 @@ find_transfers (void *cls)
   task = NULL;
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Checking for incoming wire transfers\n");
-
   if (NULL == (session = db_plugin->get_session (db_plugin->cls)))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Failed to obtain database session!\n");
-    global_ret = GNUNET_SYSERR;
+    global_ret = GR_DATABASE_SESSION_FAIL;
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
@@ -606,7 +503,7 @@ find_transfers (void *cls)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Failed to start database transaction!\n");
-    global_ret = GNUNET_SYSERR;
+    global_ret = GR_DATABASE_TRANSACTION_BEGIN_FAIL;
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
@@ -615,15 +512,14 @@ find_transfers (void *cls)
     qs = db_plugin->get_latest_reserve_in_reference (db_plugin->cls,
                                                      session,
                                                      wa_pos->section_name,
-                                                     &last_row_off,
-                                                     &last_row_off_size);
+                                                     &wa_pos->last_row_off);
     if (GNUNET_DB_STATUS_HARD_ERROR == qs)
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                   "Failed to obtain starting point for montoring from database!\n");
       db_plugin->rollback (db_plugin->cls,
                            session);
-      global_ret = GNUNET_SYSERR;
+      global_ret = GR_DATABASE_SELECT_LATEST_HARD_FAIL;
       GNUNET_SCHEDULER_shutdown ();
       return;
     }
@@ -636,29 +532,28 @@ find_transfers (void *cls)
                                        NULL);
       return;
     }
+    wa_pos->reset_mode = GNUNET_NO;
   }
-  wa_pos->reset_mode = GNUNET_NO;
-  GNUNET_assert ( (NULL == last_row_off) ||
-                  ( (NULL != last_row_off) &&
-                    (0 != last_row_off_size) ) );
-  delay = GNUNET_YES;
-  current_batch_size = 0;
+  wa_pos->delay = GNUNET_YES;
+  wa_pos->current_batch_size = 0; /* reset counter */
 
-  hh = wa_pos->wire_plugin->get_history (wa_pos->wire_plugin->cls,
-                                         wa_pos->section_name,
-                                         TALER_BANK_DIRECTION_CREDIT,
-                                         last_row_off,
-                                         last_row_off_size,
-                                         batch_size,
-                                         &history_cb,
-                                         session);
-  if (NULL == hh)
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "wirewatch: requesting incoming history from %s\n",
+              wa_pos->auth.wire_gateway_url);
+  wa_pos->session = session;
+  wa_pos->hh = TALER_BANK_credit_history (ctx,
+                                          &wa_pos->auth,
+                                          wa_pos->last_row_off,
+                                          wa_pos->batch_size,
+                                          &history_cb,
+                                          wa_pos);
+  if (NULL == wa_pos->hh)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Failed to start request for account history!\n");
     db_plugin->rollback (db_plugin->cls,
                          session);
-    global_ret = GNUNET_SYSERR;
+    global_ret = GR_BANK_REQUEST_HISTORY_FAIL;
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
@@ -686,7 +581,7 @@ run (void *cls,
   if (GNUNET_OK !=
       exchange_serve_process_config ())
   {
-    global_ret = 1;
+    global_ret = GR_CONFIGURATION_INVALID;
     return;
   }
   wa_pos = wa_head;
@@ -695,6 +590,14 @@ run (void *cls,
                                    NULL);
   GNUNET_SCHEDULER_add_shutdown (&shutdown_task,
                                  cls);
+  ctx = GNUNET_CURL_init (&GNUNET_CURL_gnunet_scheduler_reschedule,
+                          &rc);
+  rc = GNUNET_CURL_gnunet_rc_create (ctx);
+  if (NULL == ctx)
+  {
+    GNUNET_break (0);
+    return;
+  }
 }
 
 
@@ -703,14 +606,16 @@ run (void *cls,
  *
  * @param argc number of arguments from the command line
  * @param argv command line arguments
- * @return 0 ok, 1 on error
+ * @return 0 ok, non-zero on error
  */
 int
 main (int argc,
       char *const *argv)
 {
   struct GNUNET_GETOPT_CommandLineOption options[] = {
-    GNUNET_GETOPT_option_flag ('T',
+    GNUNET_GETOPT_option_timetravel ('T',
+                                     "timetravel"),
+    GNUNET_GETOPT_option_flag ('t',
                                "test",
                                "run in test mode and exit when idle",
                                &test_mode),
@@ -724,7 +629,7 @@ main (int argc,
   if (GNUNET_OK !=
       GNUNET_STRINGS_get_utf8_args (argc, argv,
                                     &argc, &argv))
-    return 2;
+    return GR_CMD_LINE_UTF8_ERROR;
   if (GNUNET_OK !=
       GNUNET_PROGRAM_run (argc, argv,
                           "taler-exchange-wirewatch",
@@ -734,7 +639,7 @@ main (int argc,
                           &run, NULL))
   {
     GNUNET_free ((void *) argv);
-    return 1;
+    return GR_CMD_LINE_OPTIONS_WRONG;
   }
   GNUNET_free ((void *) argv);
   return global_ret;
