@@ -75,6 +75,11 @@ dead_prepare_cb (void *cls,
                  const char *buf,
                  size_t buf_size)
 {
+  (void) cls;
+  (void) rowid;
+  (void) wire_method;
+  (void) buf;
+  (void) buf_size;
   GNUNET_assert (0);
 }
 
@@ -163,9 +168,9 @@ check_reserve (struct TALER_EXCHANGEDB_Session *session,
 
   reserve.pub = *pub;
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
-          plugin->reserve_get (plugin->cls,
-                               session,
-                               &reserve));
+          plugin->reserves_get (plugin->cls,
+                                session,
+                                &reserve));
   FAILIF (value != reserve.balance.value);
   FAILIF (fraction != reserve.balance.fraction);
   FAILIF (0 != strcmp (currency, reserve.balance.currency));
@@ -220,7 +225,7 @@ create_denom_key_pair (unsigned int size,
                        const struct TALER_Amount *fee_refund)
 {
   struct DenomKeyPair *dkp;
-  struct TALER_EXCHANGEDB_DenominationKeyIssueInformation dki;
+  struct TALER_EXCHANGEDB_DenominationKey dki;
   struct TALER_EXCHANGEDB_DenominationKeyInformationP issue2;
 
   dkp = GNUNET_new (struct DenomKeyPair);
@@ -233,7 +238,7 @@ create_denom_key_pair (unsigned int size,
      are not properly initialized for this test. */
   memset (&dki,
           0,
-          sizeof (struct TALER_EXCHANGEDB_DenominationKeyIssueInformation));
+          sizeof (struct TALER_EXCHANGEDB_DenominationKey));
   dki.denom_pub = dkp->pub;
   GNUNET_TIME_round_abs (&now);
   dki.issue.properties.start = GNUNET_TIME_absolute_hton (now);
@@ -330,20 +335,26 @@ static struct TALER_TransferPublicKeyP tpub;
  *
  * @param cls closure
  * @param rowid unique serial ID for the row in our database
- * @param num_newcoins size of the @a rrcs array
- * @param rrcs array of @a num_newcoins information about coins to be created
+ * @param num_freshcoins size of the @a rrcs array
+ * @param rrcs array of @a num_freshcoins information about coins to be created
  * @param num_tprivs number of entries in @a tprivs, should be #TALER_CNC_KAPPA - 1
  * @param tprivs array of @e num_tprivs transfer private keys
  * @param tp transfer public key information
  */
 static void
 never_called_cb (void *cls,
-                 uint32_t num_newcoins,
+                 uint32_t num_freshcoins,
                  const struct TALER_EXCHANGEDB_RefreshRevealedCoin *rrcs,
                  unsigned int num_tprivs,
                  const struct TALER_TransferPrivateKeyP *tprivs,
                  const struct TALER_TransferPublicKeyP *tp)
 {
+  (void) cls;
+  (void) num_freshcoins;
+  (void) rrcs;
+  (void) num_tprivs;
+  (void) tprivs;
+  (void) tp;
   GNUNET_assert (0); /* should never be called! */
 }
 
@@ -354,23 +365,24 @@ never_called_cb (void *cls,
  *
  * @param cls closure
  * @param rowid unique serial ID for the row in our database
- * @param num_newcoins size of the @a rrcs array
- * @param rrcs array of @a num_newcoins information about coins to be created
+ * @param num_freshcoins size of the @a rrcs array
+ * @param rrcs array of @a num_freshcoins information about coins to be created
  * @param num_tprivs number of entries in @a tprivs, should be #TALER_CNC_KAPPA - 1
  * @param tprivsr array of @e num_tprivs transfer private keys
  * @param tpr transfer public key information
  */
 static void
-check_refresh_reveal_cb (void *cls,
-                         uint32_t num_newcoins,
-                         const struct
-                         TALER_EXCHANGEDB_RefreshRevealedCoin *rrcs,
-                         unsigned int num_tprivs,
-                         const struct TALER_TransferPrivateKeyP *tprivsr,
-                         const struct TALER_TransferPublicKeyP *tpr)
+check_refresh_reveal_cb (
+  void *cls,
+  uint32_t num_freshcoins,
+  const struct TALER_EXCHANGEDB_RefreshRevealedCoin *rrcs,
+  unsigned int num_tprivs,
+  const struct TALER_TransferPrivateKeyP *tprivsr,
+  const struct TALER_TransferPublicKeyP *tpr)
 {
+  (void) cls;
   /* compare the refresh commit coin arrays */
-  for (unsigned int cnt = 0; cnt < num_newcoins; cnt++)
+  for (unsigned int cnt = 0; cnt < num_freshcoins; cnt++)
   {
     const struct TALER_EXCHANGEDB_RefreshRevealedCoin *acoin =
       &revealed_coins[cnt];
@@ -411,7 +423,7 @@ static unsigned int auditor_row_cnt;
  * @param coin_pub public key of the coin
  * @param coin_sig signature from the coin
  * @param amount_with_fee amount that was deposited including fee
- * @param num_newcoins how many coins were issued
+ * @param num_freshcoins how many coins were issued
  * @param noreveal_index which index was picked by the exchange in cut-and-choose
  * @param rc what is the session hash
  * @return #GNUNET_OK to continue to iterate, #GNUNET_SYSERR to stop
@@ -426,6 +438,13 @@ audit_refresh_session_cb (void *cls,
                           uint32_t noreveal_index,
                           const struct TALER_RefreshCommitmentP *rc)
 {
+  (void) cls;
+  (void) rowid;
+  (void) denom_pub;
+  (void) coin_sig;
+  (void) amount_with_fee;
+  (void) noreveal_index;
+  (void) rc;
   auditor_row_cnt++;
   return GNUNET_OK;
 }
@@ -448,9 +467,11 @@ static struct DenomKeyPair **new_dkp;
 static void
 handle_link_data_cb (void *cls,
                      const struct TALER_TransferPublicKeyP *transfer_pub,
-                     const struct TALER_EXCHANGEDB_LinkDataList *ldl)
+                     const struct TALER_EXCHANGEDB_LinkList *ldl)
 {
-  for (const struct TALER_EXCHANGEDB_LinkDataList *ldlp = ldl;
+  (void) cls;
+  (void) transfer_pub;
+  for (const struct TALER_EXCHANGEDB_LinkList *ldlp = ldl;
        NULL != ldlp;
        ldlp = ldlp->next)
   {
@@ -488,8 +509,8 @@ handle_link_data_cb (void *cls,
 static int
 test_melting (struct TALER_EXCHANGEDB_Session *session)
 {
-  struct TALER_EXCHANGEDB_RefreshSession refresh_session;
-  struct TALER_EXCHANGEDB_RefreshMelt ret_refresh_session;
+  struct TALER_EXCHANGEDB_Refresh refresh_session;
+  struct TALER_EXCHANGEDB_Melt ret_refresh_session;
   struct DenomKeyPair *dkp;
   struct TALER_DenominationPublicKey *new_denom_pubs;
   int ret;
@@ -572,14 +593,14 @@ test_melting (struct TALER_EXCHANGEDB_Session *session)
           GNUNET_memcmp (&refresh_session.coin.denom_pub_hash,
                          &ret_refresh_session.session.coin.denom_pub_hash));
 
-  /* test 'select_refreshs_above_serial_id' */
+  /* test 'select_refreshes_above_serial_id' */
   auditor_row_cnt = 0;
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
-          plugin->select_refreshs_above_serial_id (plugin->cls,
-                                                   session,
-                                                   0,
-                                                   &audit_refresh_session_cb,
-                                                   NULL));
+          plugin->select_refreshes_above_serial_id (plugin->cls,
+                                                    session,
+                                                    0,
+                                                    &audit_refresh_session_cb,
+                                                    NULL));
   FAILIF (1 != auditor_row_cnt);
 
   new_dkp = GNUNET_new_array (MELT_NEW_COINS,
@@ -760,7 +781,7 @@ cb_wt_check (void *cls,
   GNUNET_assert (0 == GNUNET_memcmp (merchant_pub,
                                      &merchant_pub_wt));
   GNUNET_assert (0 == strcmp (json_string_value (json_object_get (wire,
-                                                                  "url")),
+                                                                  "payto_uri")),
                               "payto://sepa/DE67830654080004822650"));
   GNUNET_assert (0 == GNUNET_memcmp (h_wire,
                                      &h_wire_wt));
@@ -931,6 +952,15 @@ audit_refund_cb (void *cls,
                  uint64_t rtransaction_id,
                  const struct TALER_Amount *amount_with_fee)
 {
+  (void) cls;
+  (void) rowid;
+  (void) denom_pub;
+  (void) coin_pub;
+  (void) merchant_pub;
+  (void) merchant_sig;
+  (void) h_contract_terms;
+  (void) rtransaction_id;
+  (void) amount_with_fee;
   auditor_row_cnt++;
   return GNUNET_OK;
 }
@@ -944,8 +974,7 @@ audit_refund_cb (void *cls,
  * @param reserve_pub public key of the reserve (also the WTID)
  * @param credit amount that was received
  * @param sender_account_details information about the sender's bank account
- * @param wire_reference unique reference identifying the wire transfer (binary blob)
- * @param wire_reference_size number of bytes in @a wire_reference
+ * @param wire_reference unique reference identifying the wire transfer
  * @param execution_date when did we receive the funds
  * @return #GNUNET_OK to continue to iterate, #GNUNET_SYSERR to stop
  */
@@ -955,10 +984,16 @@ audit_reserve_in_cb (void *cls,
                      const struct TALER_ReservePublicKeyP *reserve_pub,
                      const struct TALER_Amount *credit,
                      const char *sender_account_details,
-                     const void *wire_reference,
-                     size_t wire_reference_size,
+                     uint64_t wire_reference,
                      struct GNUNET_TIME_Absolute execution_date)
 {
+  (void) cls;
+  (void) rowid;
+  (void) reserve_pub;
+  (void) credit;
+  (void) sender_account_details;
+  (void) wire_reference;
+  (void) execution_date;
   auditor_row_cnt++;
   return GNUNET_OK;
 }
@@ -971,7 +1006,6 @@ audit_reserve_in_cb (void *cls,
  * @param rowid unique serial ID for the refresh session in our DB
  * @param h_blind_ev blinded hash of the coin's public key
  * @param denom_pub public denomination key of the deposited coin
- * @param denom_sig signature over the deposited coin
  * @param reserve_pub public key of the reserve
  * @param reserve_sig signature over the withdraw operation
  * @param execution_date when did the wallet withdraw the coin
@@ -983,12 +1017,19 @@ audit_reserve_out_cb (void *cls,
                       uint64_t rowid,
                       const struct GNUNET_HashCode *h_blind_ev,
                       const struct TALER_DenominationPublicKey *denom_pub,
-                      const struct TALER_DenominationSignature *denom_sig,
                       const struct TALER_ReservePublicKeyP *reserve_pub,
                       const struct TALER_ReserveSignatureP *reserve_sig,
                       struct GNUNET_TIME_Absolute execution_date,
                       const struct TALER_Amount *amount_with_fee)
 {
+  (void) cls;
+  (void) rowid;
+  (void) h_blind_ev;
+  (void) denom_pub;
+  (void) reserve_pub;
+  (void) reserve_sig;
+  (void) execution_date;
+  (void) amount_with_fee;
   auditor_row_cnt++;
   return GNUNET_OK;
 }
@@ -1239,14 +1280,14 @@ test_wire_out (struct TALER_EXCHANGEDB_Session *session,
 
     h_contract_terms_wt2.bits[0]++;
     FAILIF (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS !=
-            plugin->wire_lookup_deposit_wtid (plugin->cls,
-                                              session,
-                                              &h_contract_terms_wt2,
-                                              &h_wire_wt,
-                                              &coin_pub_wt,
-                                              &merchant_pub_wt,
-                                              &cb_wtid_never,
-                                              NULL));
+            plugin->lookup_transfer_by_deposit (plugin->cls,
+                                                session,
+                                                &h_contract_terms_wt2,
+                                                &h_wire_wt,
+                                                &coin_pub_wt,
+                                                &merchant_pub_wt,
+                                                &cb_wtid_never,
+                                                NULL));
   }
   /* insert WT data */
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
@@ -1261,7 +1302,7 @@ test_wire_out (struct TALER_EXCHANGEDB_Session *session,
     json_t *wire_out_account;
 
     wire_out_account = json_pack ("{s:s,s:s}",
-                                  "url",
+                                  "payto_uri",
                                   "payto://x-taler-bank/localhost:8080/1",
                                   "salt", "this-is-my-salt");
     if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
@@ -1290,14 +1331,14 @@ test_wire_out (struct TALER_EXCHANGEDB_Session *session,
                                         &cb_wt_check,
                                         &cb_wt_never));
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
-          plugin->wire_lookup_deposit_wtid (plugin->cls,
-                                            session,
-                                            &h_contract_terms_wt,
-                                            &h_wire_wt,
-                                            &coin_pub_wt,
-                                            &merchant_pub_wt,
-                                            &cb_wtid_check,
-                                            &cb_wtid_never));
+          plugin->lookup_transfer_by_deposit (plugin->cls,
+                                              session,
+                                              &h_contract_terms_wt,
+                                              &h_wire_wt,
+                                              &coin_pub_wt,
+                                              &merchant_pub_wt,
+                                              &cb_wtid_check,
+                                              &cb_wtid_never));
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
           plugin->select_wire_out_above_serial_id (plugin->cls,
                                                    session,
@@ -1313,31 +1354,33 @@ drop:
 
 
 /**
- * Function called about paybacks the exchange has to perform.
+ * Function called about recoups the exchange has to perform.
  *
  * @param cls closure with the expected value for @a coin_blind
- * @param rowid row identifier used to uniquely identify the payback operation
- * @param timestamp when did we receive the payback request
+ * @param rowid row identifier used to uniquely identify the recoup operation
+ * @param timestamp when did we receive the recoup request
  * @param amount how much should be added back to the reserve
  * @param reserve_pub public key of the reserve
  * @param coin public information about the coin
- * @param coin_sig signature with @e coin_pub of type #TALER_SIGNATURE_WALLET_COIN_PAYBACK
+ * @param denom_pub denomination key of @a coin
+ * @param coin_sig signature with @e coin_pub of type #TALER_SIGNATURE_WALLET_COIN_RECOUP
  * @param coin_blind blinding factor used to blind the coin
  * @return #GNUNET_OK to continue to iterate, #GNUNET_SYSERR to stop
  */
 static int
-payback_cb (void *cls,
-            uint64_t rowid,
-            struct GNUNET_TIME_Absolute timestamp,
-            const struct TALER_Amount *amount,
-            const struct TALER_ReservePublicKeyP *reserve_pub,
-            const struct TALER_CoinPublicInfo *coin,
-            const struct TALER_DenominationPublicKey *denom_pub,
-            const struct TALER_CoinSpendSignatureP *coin_sig,
-            const struct TALER_DenominationBlindingKeyP *coin_blind)
+recoup_cb (void *cls,
+           uint64_t rowid,
+           struct GNUNET_TIME_Absolute timestamp,
+           const struct TALER_Amount *amount,
+           const struct TALER_ReservePublicKeyP *reserve_pub,
+           const struct TALER_CoinPublicInfo *coin,
+           const struct TALER_DenominationPublicKey *denom_pub,
+           const struct TALER_CoinSpendSignatureP *coin_sig,
+           const struct TALER_DenominationBlindingKeyP *coin_blind)
 {
   const struct TALER_DenominationBlindingKeyP *cb = cls;
 
+  FAILIF (NULL == cb);
   FAILIF (0 != GNUNET_memcmp (cb,
                               coin_blind));
   return GNUNET_OK;
@@ -1372,6 +1415,7 @@ wire_missing_cb (void *cls,
   const struct TALER_EXCHANGEDB_Deposit *deposit = cls;
   struct GNUNET_HashCode h_wire;
 
+  (void) done;
   if (NULL != wire)
     GNUNET_assert (GNUNET_OK ==
                    TALER_JSON_merchant_wire_signature_hash (wire,
@@ -1416,56 +1460,17 @@ wire_missing_cb (void *cls,
  * to a particular coin.
  *
  * @param cls closure with the `struct TALER_EXCHANGEDB_Refund *` we expect to get
- * @param merchant_pub public key of merchant who authorized refund
- * @param merchant_sig signature of merchant authorizing refund
- * @param h_contract hash of contract being refunded
- * @param rtransaction_id refund transaction ID
  * @param amount_with_fee amount being refunded
- * @param refund_fee fee the exchange keeps for the refund processing
  * @return #GNUNET_OK to continue to iterate, #GNUNET_SYSERR to stop
  */
 static int
 check_refund_cb (void *cls,
-                 const struct TALER_MerchantPublicKeyP *merchant_pub,
-                 const struct TALER_MerchantSignatureP *merchant_sig,
-                 const struct GNUNET_HashCode *h_contract,
-                 uint64_t rtransaction_id,
-                 const struct TALER_Amount *amount_with_fee,
-                 const struct TALER_Amount *refund_fee)
+                 const struct TALER_Amount *amount_with_fee)
 {
   const struct TALER_EXCHANGEDB_Refund *refund = cls;
 
-  if (0 != GNUNET_memcmp (merchant_pub,
-                          &refund->merchant_pub))
-  {
-    GNUNET_break (0);
-    result = 66;
-  }
-  if (0 != GNUNET_memcmp (merchant_sig,
-                          &refund->merchant_sig))
-  {
-    GNUNET_break (0);
-    result = 66;
-  }
-  if (0 != GNUNET_memcmp (h_contract,
-                          &refund->h_contract_terms))
-  {
-    GNUNET_break (0);
-    result = 66;
-  }
-  if (rtransaction_id != refund->rtransaction_id)
-  {
-    GNUNET_break (0);
-    result = 66;
-  }
   if (0 != TALER_amount_cmp (amount_with_fee,
-                             &refund->refund_amount))
-  {
-    GNUNET_break (0);
-    result = 66;
-  }
-  if (0 != TALER_amount_cmp (refund_fee,
-                             &refund->refund_fee))
+                             &refund->details.refund_amount))
   {
     GNUNET_break (0);
     result = 66;
@@ -1507,8 +1512,7 @@ run (void *cls)
   const char *sndr = "payto://x-taler-bank/localhost:8080/1";
   unsigned int matched;
   unsigned int cnt;
-  void *rr;
-  size_t rr_size;
+  uint64_t rr;
   enum GNUNET_DB_QueryStatus qs;
   struct GNUNET_TIME_Absolute now;
 
@@ -1517,7 +1521,7 @@ run (void *cls)
   session = NULL;
   deposit.coin.denom_sig.rsa_signature = NULL;
   wire = json_pack ("{s:s, s:s}",
-                    "url", "payto://sepa/DE67830654080004822650",
+                    "payto_uri", "payto://sepa/DE67830654080004822650",
                     "salt", "this-is-a-salt-value");
   ZR_BLK (&cbc);
   ZR_BLK (&cbc2);
@@ -1548,11 +1552,11 @@ run (void *cls)
 
   /* test DB is empty */
   FAILIF (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS !=
-          plugin->select_payback_above_serial_id (plugin->cls,
-                                                  session,
-                                                  0,
-                                                  &payback_cb,
-                                                  NULL));
+          plugin->select_recoup_above_serial_id (plugin->cls,
+                                                 session,
+                                                 0,
+                                                 &recoup_cb,
+                                                 NULL));
   RND_BLK (&reserve_pub);
   GNUNET_assert (GNUNET_OK ==
                  TALER_string_to_amount (CURRENCY ":1.000010",
@@ -1577,9 +1581,8 @@ run (void *cls)
   FAILIF (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS !=
           plugin->get_latest_reserve_in_reference (plugin->cls,
                                                    session,
-                                                   "account-1",
-                                                   &rr,
-                                                   &rr_size));
+                                                   "exchange-account-1",
+                                                   &rr));
   now = GNUNET_TIME_absolute_get ();
   (void) GNUNET_TIME_round_abs (&now);
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
@@ -1589,18 +1592,14 @@ run (void *cls)
                                       &value,
                                       now,
                                       sndr,
-                                      "account-1",
-                                      "TEST",
+                                      "exchange-account-1",
                                       4));
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
           plugin->get_latest_reserve_in_reference (plugin->cls,
                                                    session,
-                                                   "account-1",
-                                                   &rr,
-                                                   &rr_size));
-  FAILIF (4 != rr_size);
-  FAILIF (0 != memcmp ("TEST", rr, 4));
-  GNUNET_free (rr);
+                                                   "exchange-account-1",
+                                                   &rr));
+  FAILIF (4 != rr);
   FAILIF (GNUNET_OK !=
           check_reserve (session,
                          &reserve_pub,
@@ -1616,25 +1615,19 @@ run (void *cls)
                                       &value,
                                       now,
                                       sndr,
-                                      "account-1",
-                                      "TEST2",
+                                      "exchange-account-1",
                                       5));
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
           plugin->get_latest_reserve_in_reference (plugin->cls,
                                                    session,
-                                                   "account-1",
-                                                   &rr,
-                                                   &rr_size));
-  GNUNET_free (rr);
+                                                   "exchange-account-1",
+                                                   &rr));
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
           plugin->get_latest_reserve_in_reference (plugin->cls,
                                                    session,
-                                                   "account-1",
-                                                   &rr,
-                                                   &rr_size));
-  FAILIF (5 != rr_size);
-  FAILIF (0 != memcmp ("TEST2", rr, 5));
-  GNUNET_free (rr);
+                                                   "exchange-account-1",
+                                                   &rr));
+  FAILIF (5 != rr);
   FAILIF (GNUNET_OK !=
           check_reserve (session,
                          &reserve_pub,
@@ -1710,22 +1703,45 @@ run (void *cls)
           plugin->ensure_coin_known (plugin->cls,
                                      session,
                                      &deposit.coin));
+  {
+    struct TALER_EXCHANGEDB_Reserve pre_reserve;
+    struct TALER_EXCHANGEDB_Reserve post_reserve;
+    struct TALER_Amount delta;
+
+    pre_reserve.pub = reserve_pub;
+    FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
+            plugin->reserves_get (plugin->cls,
+                                  session,
+                                  &pre_reserve));
+    FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
+            plugin->insert_recoup_request (plugin->cls,
+                                           session,
+                                           &reserve_pub,
+                                           &deposit.coin,
+                                           &coin_sig,
+                                           &coin_blind,
+                                           &value,
+                                           &cbc.h_coin_envelope,
+                                           deadline));
+    post_reserve.pub = reserve_pub;
+    FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
+            plugin->reserves_get (plugin->cls,
+                                  session,
+                                  &post_reserve));
+    FAILIF (GNUNET_OK !=
+            TALER_amount_subtract (&delta,
+                                   &post_reserve.balance,
+                                   &pre_reserve.balance));
+    FAILIF (0 !=
+            TALER_amount_cmp (&delta,
+                              &value));
+  }
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
-          plugin->insert_payback_request (plugin->cls,
-                                          session,
-                                          &reserve_pub,
-                                          &deposit.coin,
-                                          &coin_sig,
-                                          &coin_blind,
-                                          &value,
-                                          &cbc.h_coin_envelope,
-                                          deadline));
-  FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
-          plugin->select_payback_above_serial_id (plugin->cls,
-                                                  session,
-                                                  0,
-                                                  &payback_cb,
-                                                  &coin_blind));
+          plugin->select_recoup_above_serial_id (plugin->cls,
+                                                 session,
+                                                 0,
+                                                 &recoup_cb,
+                                                 &coin_blind));
 
   GNUNET_assert (GNUNET_OK ==
                  TALER_amount_add (&amount_with_fee,
@@ -1784,23 +1800,23 @@ run (void *cls)
                            &cbc.h_coin_envelope,
                            sizeof (cbc.h_coin_envelope)));
       break;
-    case TALER_EXCHANGEDB_RO_PAYBACK_COIN:
+    case TALER_EXCHANGEDB_RO_RECOUP_COIN:
       {
-        struct TALER_EXCHANGEDB_Payback *payback = rh_head->details.payback;
+        struct TALER_EXCHANGEDB_Recoup *recoup = rh_head->details.recoup;
 
-        FAILIF (0 != memcmp (&payback->coin_sig,
+        FAILIF (0 != memcmp (&recoup->coin_sig,
                              &coin_sig,
                              sizeof (coin_sig)));
-        FAILIF (0 != memcmp (&payback->coin_blind,
+        FAILIF (0 != memcmp (&recoup->coin_blind,
                              &coin_blind,
                              sizeof (coin_blind)));
-        FAILIF (0 != memcmp (&payback->reserve_pub,
+        FAILIF (0 != memcmp (&recoup->reserve_pub,
                              &reserve_pub,
                              sizeof (reserve_pub)));
-        FAILIF (0 != memcmp (&payback->coin.coin_pub,
+        FAILIF (0 != memcmp (&recoup->coin.coin_pub,
                              &deposit.coin.coin_pub,
                              sizeof (deposit.coin.coin_pub)));
-        FAILIF (0 != TALER_amount_cmp (&payback->value,
+        FAILIF (0 != TALER_amount_cmp (&recoup->value,
                                        &value));
       }
       break;
@@ -1830,11 +1846,11 @@ run (void *cls)
                                                       &audit_reserve_in_cb,
                                                       NULL));
   FAILIF (0 >=
-          plugin->select_reserves_out_above_serial_id (plugin->cls,
-                                                       session,
-                                                       0,
-                                                       &audit_reserve_out_cb,
-                                                       NULL));
+          plugin->select_withdrawals_above_serial_id (plugin->cls,
+                                                      session,
+                                                      0,
+                                                      &audit_reserve_out_cb,
+                                                      NULL));
   FAILIF (3 != auditor_row_cnt);
 
   /* Tests for deposits */
@@ -1941,7 +1957,10 @@ run (void *cls)
   FAILIF (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS !=
           plugin->test_deposit_done (plugin->cls,
                                      session,
-                                     &deposit));
+                                     &deposit.coin.coin_pub,
+                                     &deposit.merchant_pub,
+                                     &deposit.h_contract_terms,
+                                     &deposit.h_wire));
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
           plugin->mark_deposit_done (plugin->cls,
                                      session,
@@ -1952,7 +1971,10 @@ run (void *cls)
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
           plugin->test_deposit_done (plugin->cls,
                                      session,
-                                     &deposit));
+                                     &deposit.coin.coin_pub,
+                                     &deposit.merchant_pub,
+                                     &deposit.h_contract_terms,
+                                     &deposit.h_wire));
 
   result = 10;
   deposit2 = deposit;
@@ -1982,13 +2004,14 @@ run (void *cls)
 
   /* test insert_refund! */
   refund.coin = deposit.coin;
-  refund.merchant_pub = deposit.merchant_pub;
-  RND_BLK (&refund.merchant_sig);
-  refund.h_contract_terms = deposit.h_contract_terms;
-  refund.rtransaction_id = GNUNET_CRYPTO_random_u64 (GNUNET_CRYPTO_QUALITY_WEAK,
-                                                     UINT64_MAX);
-  refund.refund_amount = deposit.amount_with_fee;
-  refund.refund_fee = fee_refund;
+  refund.details.merchant_pub = deposit.merchant_pub;
+  RND_BLK (&refund.details.merchant_sig);
+  refund.details.h_contract_terms = deposit.h_contract_terms;
+  refund.details.rtransaction_id
+    = GNUNET_CRYPTO_random_u64 (GNUNET_CRYPTO_QUALITY_WEAK,
+                                UINT64_MAX);
+  refund.details.refund_amount = deposit.amount_with_fee;
+  refund.details.refund_fee = fee_refund;
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
           plugin->insert_refund (plugin->cls,
                                  session,
@@ -1997,10 +2020,12 @@ run (void *cls)
           plugin->select_refunds_by_coin (plugin->cls,
                                           session,
                                           &refund.coin.coin_pub,
+                                          &refund.details.merchant_pub,
+                                          &refund.details.h_contract_terms,
                                           &check_refund_cb,
                                           &refund));
 
-  /* test payback / revocation */
+  /* test recoup / revocation */
   RND_BLK (&master_sig);
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
           plugin->insert_denomination_revocation (plugin->cls,
@@ -2047,15 +2072,15 @@ run (void *cls)
   RND_BLK (&coin_sig);
   RND_BLK (&coin_blind);
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
-          plugin->insert_payback_request (plugin->cls,
-                                          session,
-                                          &reserve_pub,
-                                          &deposit.coin,
-                                          &coin_sig,
-                                          &coin_blind,
-                                          &value,
-                                          &cbc.h_coin_envelope,
-                                          deadline));
+          plugin->insert_recoup_request (plugin->cls,
+                                         session,
+                                         &reserve_pub,
+                                         &deposit.coin,
+                                         &coin_sig,
+                                         &coin_blind,
+                                         &value,
+                                         &cbc.h_coin_envelope,
+                                         deadline));
 
   auditor_row_cnt = 0;
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
@@ -2080,11 +2105,8 @@ run (void *cls)
     {
     case TALER_EXCHANGEDB_TT_DEPOSIT:
       {
-        struct TALER_EXCHANGEDB_Deposit *have = tlp->details.deposit;
+        struct TALER_EXCHANGEDB_DepositListEntry *have = tlp->details.deposit;
 
-        FAILIF (0 != memcmp (&have->coin.coin_pub,
-                             &deposit.coin.coin_pub,
-                             sizeof (struct TALER_CoinSpendPublicKeyP)));
         /* Note: we're not comparing the denomination keys, as there is
            still the question of whether we should even bother exporting
            them here. */
@@ -2115,51 +2137,46 @@ run (void *cls)
       }
 #if 0
     /* this coin pub was actually never melted... */
-    case TALER_EXCHANGEDB_TT_REFRESH_MELT:
+    case TALER_EXCHANGEDB_TT_MELT:
       FAILIF (0 != memcmp (&melt,
                            &tlp->details.melt,
-                           sizeof (struct TALER_EXCHANGEDB_RefreshMelt)));
+                           sizeof (struct TALER_EXCHANGEDB_Melt)));
       matched |= 2;
       break;
 #endif
     case TALER_EXCHANGEDB_TT_REFUND:
       {
-        struct TALER_EXCHANGEDB_Refund *have = tlp->details.refund;
+        struct TALER_EXCHANGEDB_RefundListEntry *have = tlp->details.refund;
 
-        FAILIF (0 != memcmp (&have->coin.coin_pub,
-                             &refund.coin.coin_pub,
-                             sizeof (struct TALER_CoinSpendPublicKeyP)));
         /* Note: we're not comparing the denomination keys, as there is
            still the question of whether we should even bother exporting
            them here. */
         FAILIF (0 != GNUNET_memcmp (&have->merchant_pub,
-                                    &refund.merchant_pub));
+                                    &refund.details.merchant_pub));
         FAILIF (0 != GNUNET_memcmp (&have->merchant_sig,
-                                    &refund.merchant_sig));
+                                    &refund.details.merchant_sig));
         FAILIF (0 != GNUNET_memcmp (&have->h_contract_terms,
-                                    &refund.h_contract_terms));
-        FAILIF (have->rtransaction_id != refund.rtransaction_id);
+                                    &refund.details.h_contract_terms));
+        FAILIF (have->rtransaction_id != refund.details.rtransaction_id);
         FAILIF (0 != TALER_amount_cmp (&have->refund_amount,
-                                       &refund.refund_amount));
+                                       &refund.details.refund_amount));
         FAILIF (0 != TALER_amount_cmp (&have->refund_fee,
-                                       &refund.refund_fee));
+                                       &refund.details.refund_fee));
         matched |= 4;
         break;
       }
-    case TALER_EXCHANGEDB_TT_PAYBACK:
+    case TALER_EXCHANGEDB_TT_RECOUP:
       {
-        struct TALER_EXCHANGEDB_Payback *payback = tlp->details.payback;
+        struct TALER_EXCHANGEDB_RecoupListEntry *recoup =
+          tlp->details.recoup;
 
-        FAILIF (0 != GNUNET_memcmp (&payback->coin_sig,
+        FAILIF (0 != GNUNET_memcmp (&recoup->coin_sig,
                                     &coin_sig));
-        FAILIF (0 != GNUNET_memcmp (&payback->coin_blind,
+        FAILIF (0 != GNUNET_memcmp (&recoup->coin_blind,
                                     &coin_blind));
-        FAILIF (0 != GNUNET_memcmp (&payback->reserve_pub,
+        FAILIF (0 != GNUNET_memcmp (&recoup->reserve_pub,
                                     &reserve_pub));
-        FAILIF (0 != memcmp (&payback->coin.coin_pub,
-                             &deposit.coin.coin_pub,
-                             sizeof (deposit.coin.coin_pub)));
-        FAILIF (0 != TALER_amount_cmp (&payback->value,
+        FAILIF (0 != TALER_amount_cmp (&recoup->value,
                                        &value));
         matched |= 8;
         break;
