@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2017 GNUnet e. V. (and other contributing authors)
+  Copyright (C) 2014-2017 Taler Systems SA (and other contributing authors)
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -45,10 +45,11 @@
 static int
 dki_iter (void *cls,
           const char *alias,
-          const struct TALER_EXCHANGEDB_DenominationKeyIssueInformation *dki)
+          const struct TALER_EXCHANGEDB_DenominationKey *dki)
 {
-  const struct TALER_EXCHANGEDB_DenominationKeyIssueInformation *exp = cls;
+  const struct TALER_EXCHANGEDB_DenominationKey *exp = cls;
 
+  (void) alias;
   if (0 != GNUNET_memcmp (&exp->issue,
                           &dki->issue))
   {
@@ -88,7 +89,7 @@ dki_iter_revoked (void *cls,
                   const struct GNUNET_HashCode *denom_hash,
                   const struct TALER_MasterSignatureP *revocation_master_sig)
 {
-  const struct TALER_EXCHANGEDB_DenominationKeyIssueInformation *exp = cls;
+  const struct TALER_EXCHANGEDB_DenominationKey *exp = cls;
 
   if (NULL == revocation_master_sig)
   {
@@ -109,14 +110,14 @@ int
 main (int argc,
       const char *const argv[])
 {
-  struct TALER_EXCHANGEDB_DenominationKeyIssueInformation dki;
-  char *enc;
+  struct TALER_EXCHANGEDB_DenominationKey dki;
+  void *enc;
   size_t enc_size;
-  struct TALER_EXCHANGEDB_DenominationKeyIssueInformation dki_read;
+  struct TALER_EXCHANGEDB_DenominationKey dki_read;
   struct GNUNET_CRYPTO_EddsaPrivateKey *pk;
   struct TALER_MasterPrivateKeyP master_priv;
   struct TALER_MasterPublicKeyP master_pub;
-  char *enc_read;
+  void *enc_read;
   size_t enc_read_size;
   char *tmpfile;
   char *tmpdir;
@@ -124,6 +125,8 @@ main (int argc,
   int ret;
   struct GNUNET_TIME_Absolute start;
 
+  (void) argc;
+  (void) argv;
   ret = 1;
   GNUNET_log_setup ("test-exchangedb-denomkeys",
                     "WARNING",
