@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2017 Inria & GNUnet e. V.
+  Copyright (C) 2017 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -60,6 +60,8 @@ main (int argc,
   int ret;
   unsigned int year;
 
+  (void) argc;
+  (void) argv;
   GNUNET_log_setup ("test-exchangedb-fees",
                     "WARNING",
                     NULL);
