@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014, 2015 GNUnet e.V.
+  Copyright (C) 2014, 2015 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -113,42 +113,43 @@ struct TALER_Amount
 
 
 /**
- * Parse denomination description, in the format "T:V.F".
+ * Parse monetary amount, in the format "T:V.F".
  *
- * @param str denomination description
- * @param denom denomination to write the result to
- * @return #GNUNET_OK if the string is a valid denomination specification,
+ * @param str amount string
+ * @param[out] amount amount to write the result to
+ * @return #GNUNET_OK if the string is a valid monetary amount specification,
  *         #GNUNET_SYSERR if it is invalid.
  */
 int
 TALER_string_to_amount (const char *str,
-                        struct TALER_Amount *denom);
+                        struct TALER_Amount *amount);
 
 
 /**
- * Parse denomination description, in the format "T:V.F".
+ * Parse monetary amount, in the format "T:V.F".
+ * The result is stored in network byte order (NBO).
  *
- * @param str denomination description
- * @param denom denomination to write the result to, in NBO
- * @return #GNUNET_OK if the string is a valid denomination specification,
+ * @param str amount string
+ * @param[out] amount_nbo amount to write the result to
+ * @return #GNUNET_OK if the string is a valid amount specification,
  *         #GNUNET_SYSERR if it is invalid.
  */
 int
 TALER_string_to_amount_nbo (const char *str,
-                            struct TALER_AmountNBO *denom);
+                            struct TALER_AmountNBO *amount_nbo);
 
 
 /**
  * Get the value of "zero" in a particular currency.
  *
  * @param cur currency description
- * @param denom denomination to write the result to
+ * @param[out] amount amount to write the result to
  * @return #GNUNET_OK if @a cur is a valid currency specification,
  *         #GNUNET_SYSERR if it is invalid.
  */
 int
 TALER_amount_get_zero (const char *cur,
-                       struct TALER_Amount *denom);
+                       struct TALER_Amount *amount);
 
 
 /**
@@ -164,7 +165,7 @@ TALER_amount_is_valid (const struct TALER_Amount *amount);
 /**
  * Convert amount from host to network representation.
  *
- * @param res where to store amount in network representation
+ * @param[out] res where to store amount in network representation
  * @param d amount in host representation
  */
 void
@@ -175,7 +176,7 @@ TALER_amount_hton (struct TALER_AmountNBO *res,
 /**
  * Convert amount from network to host representation.
  *
- * @param res where to store amount in host representation
+ * @param[out] res where to store amount in host representation
  * @param dn amount in network representation
  */
 void
@@ -232,7 +233,7 @@ TALER_amount_cmp_currency_nbo (const struct TALER_AmountNBO *a1,
 /**
  * Perform saturating subtraction of amounts.
  *
- * @param diff where to store (@a a1 - @a a2), or invalid if @a a2 > @a a1
+ * @param[out] diff where to store (@a a1 - @a a2), or invalid if @a a2 > @a a1
  * @param a1 amount to subtract from
  * @param a2 amount to subtract
  * @return #GNUNET_OK if the subtraction worked,
@@ -249,7 +250,7 @@ TALER_amount_subtract (struct TALER_Amount *diff,
 /**
  * Perform addition of amounts.
  *
- * @param sum where to store @a a1 + @a a2, set to "invalid" on overflow
+ * @param[out] sum where to store @a a1 + @a a2, set to "invalid" on overflow
  * @param a1 first amount to add
  * @param a2 second amount to add
  * @return #GNUNET_OK if the addition worked,
@@ -262,10 +263,10 @@ TALER_amount_add (struct TALER_Amount *sum,
 
 
 /**
- * Divide an amount by a float.  Note that this function
+ * Divide an amount by a @ divisor.  Note that this function
  * may introduce a rounding error!
  *
- * @param result where to store @a dividend / @a divisor
+ * @param[out] result where to store @a dividend / @a divisor
  * @param dividend amount to divide
  * @param divisor by what to divide, must be positive
  */
@@ -278,7 +279,7 @@ TALER_amount_divide (struct TALER_Amount *result,
 /**
  * Normalize the given amount.
  *
- * @param amount amount to normalize
+ * @param[in,out] amount amount to normalize
  * @return #GNUNET_OK if normalization worked
  *         #GNUNET_NO if value was already normalized
  *         #GNUNET_SYSERR if value was invalid or could not be normalized
@@ -309,6 +310,23 @@ const char *
 TALER_amount2s (const struct TALER_Amount *amount);
 
 
+/**
+ * Round the amount to something that can be transferred on the wire.
+ * The rounding mode is specified via the smallest transferable unit,
+ * which must only have a fractional part *or* only a value (either
+ * of the two must be zero!).
+ *
+ * @param[in,out] amount amount to round down
+ * @param[in] round_unit unit that should be rounded down to, and
+ *            either value part or the faction must be zero (but not both)
+ * @return #GNUNET_OK on success, #GNUNET_NO if rounding was unnecessary,
+ *         #GNUNET_SYSERR if the amount or currency or @a round_unit was invalid
+ */
+int
+TALER_amount_round_down (struct TALER_Amount *amount,
+                         const struct TALER_Amount *round_unit);
+
+
 #if 0                           /* keep Emacsens' auto-indent happy */
 {
 #endif
