@@ -38,10 +38,10 @@
  * @param exchange_url base URL of the exchange's API
  */
 typedef void
-(*TALER_AUDITORDB_ExchangeCallback)(void *cls,
-                                    const struct
-                                    TALER_MasterPublicKeyP *master_pub,
-                                    const char *exchange_url);
+(*TALER_AUDITORDB_ExchangeCallback)(
+  void *cls,
+  const struct TALER_MasterPublicKeyP *master_pub,
+  const char *exchange_url);
 
 
 /**
@@ -56,10 +56,9 @@ typedef void
  *         #GNUNET_SYSERR or other values on error.
  */
 typedef int
-(*TALER_AUDITORDB_DenominationInfoDataCallback)(void *cls,
-                                                const struct
-                                                TALER_DenominationKeyValidityPS
-                                                *issue);
+(*TALER_AUDITORDB_DenominationInfoDataCallback)(
+  void *cls,
+  const struct TALER_DenominationKeyValidityPS *issue);
 
 
 /**
@@ -78,19 +77,12 @@ typedef int
  *         #GNUNET_SYSERR or other values on error.
  */
 typedef int
-(*TALER_AUDITORDB_HistoricDenominationRevenueDataCallback)(void *cls,
-                                                           const struct
-                                                           GNUNET_HashCode *
-                                                           denom_pub_hash,
-                                                           struct
-                                                           GNUNET_TIME_Absolute
-                                                           revenue_timestamp,
-                                                           const struct
-                                                           TALER_Amount *
-                                                           revenue_balance,
-                                                           const struct
-                                                           TALER_Amount *
-                                                           loss_balance);
+(*TALER_AUDITORDB_HistoricDenominationRevenueDataCallback)(
+  void *cls,
+  const struct GNUNET_HashCode *denom_pub_hash,
+  struct GNUNET_TIME_Absolute revenue_timestamp,
+  const struct TALER_Amount *revenue_balance,
+  const struct TALER_Amount *loss_balance);
 
 
 /**
@@ -107,16 +99,11 @@ typedef int
  *         #GNUNET_SYSERR or other values on error.
  */
 typedef int
-(*TALER_AUDITORDB_HistoricReserveRevenueDataCallback)(void *cls,
-                                                      struct
-                                                      GNUNET_TIME_Absolute
-                                                      start_time,
-                                                      struct
-                                                      GNUNET_TIME_Absolute
-                                                      end_time,
-                                                      const struct
-                                                      TALER_Amount *
-                                                      reserve_profits);
+(*TALER_AUDITORDB_HistoricReserveRevenueDataCallback)(
+  void *cls,
+  struct GNUNET_TIME_Absolute start_time,
+  struct GNUNET_TIME_Absolute end_time,
+  const struct TALER_Amount *reserve_profits);
 
 
 /**
@@ -176,10 +163,10 @@ struct TALER_AUDITORDB_ProgressPointReserve
   uint64_t last_reserve_out_serial_id;
 
   /**
-   * serial ID of the last payback entry the auditor processed when
+   * serial ID of the last recoup entry the auditor processed when
    * considering reserves.
    */
-  uint64_t last_reserve_payback_serial_id;
+  uint64_t last_reserve_recoup_serial_id;
 
   /**
    * serial ID of the last reserve_close
@@ -246,14 +233,14 @@ struct TALER_AUDITORDB_ProgressPointCoin
   uint64_t last_refund_serial_id;
 
   /**
-   * Serial ID of the last payback operation the auditor processed.
+   * Serial ID of the last recoup operation the auditor processed.
    */
-  uint64_t last_payback_serial_id;
+  uint64_t last_recoup_serial_id;
 
   /**
-   * Serial ID of the last payback-of-refresh operation the auditor processed.
+   * Serial ID of the last recoup-of-refresh operation the auditor processed.
    */
-  uint64_t last_payback_refresh_serial_id;
+  uint64_t last_recoup_refresh_serial_id;
 
 };
 
@@ -379,11 +366,10 @@ struct TALER_AUDITORDB_DepositConfirmation
  * @param dc the deposit confirmation itself
  */
 typedef void
-(*TALER_AUDITORDB_DepositConfirmationCallback)(void *cls,
-                                               uint64_t serial_id,
-                                               const struct
-                                               TALER_AUDITORDB_DepositConfirmation
-                                               *dc);
+(*TALER_AUDITORDB_DepositConfirmationCallback)(
+  void *cls,
+  uint64_t serial_id,
+  const struct TALER_AUDITORDB_DepositConfirmation *dc);
 
 
 /**
@@ -425,10 +411,14 @@ struct TALER_AUDITORDB_Plugin
 
 
   /**
-   * Drop the Taler tables.  This should only be used in testcases.
+   * Drop all auditor tables OR deletes recoverable auditor state.
+   * This should only be used by testcases or when restarting the
+   * auditor from scratch.
    *
-   * @param cls the @e cls of this struct with the plugin-specific state
-   * @param drop_exchangelist should we also drop the exchange and deposit_confirmations table?
+   * @param cls the `struct PostgresClosure` with the plugin-specific state
+   * @param drop_exchangelist drop all tables, including schema versioning
+   *        and the exchange and deposit_confirmations table; NOT to be
+   *        used when restarting the auditor
    * @return #GNUNET_OK upon success; #GNUNET_SYSERR upon failure
    */
   int
@@ -530,8 +520,8 @@ struct TALER_AUDITORDB_Plugin
    *
    * @param cls the @e cls of this struct with the plugin-specific state
    * @param session connection to the database
-   * @param master_pub master public key of the exchange
-   * @param exchange_url public (base) URL of the API of the exchange
+   * @param cb function to call with the results
+   * @param cb_cls closure for @a cb
    * @return query result status
    */
   enum GNUNET_DB_QueryStatus
@@ -549,10 +539,10 @@ struct TALER_AUDITORDB_Plugin
    * @return query result status
    */
   enum GNUNET_DB_QueryStatus
-  (*insert_exchange_signkey)(void *cls,
-                             struct TALER_AUDITORDB_Session *session,
-                             const struct
-                             TALER_AUDITORDB_ExchangeSigningKey *sk);
+  (*insert_exchange_signkey)(
+    void *cls,
+    struct TALER_AUDITORDB_Session *session,
+    const struct TALER_AUDITORDB_ExchangeSigningKey *sk);
 
 
   /**
@@ -564,10 +554,10 @@ struct TALER_AUDITORDB_Plugin
    * @return query result status
    */
   enum GNUNET_DB_QueryStatus
-  (*insert_deposit_confirmation)(void *cls,
-                                 struct TALER_AUDITORDB_Session *session,
-                                 const struct
-                                 TALER_AUDITORDB_DepositConfirmation *dc);
+  (*insert_deposit_confirmation)(
+    void *cls,
+    struct TALER_AUDITORDB_Session *session,
+    const struct TALER_AUDITORDB_DepositConfirmation *dc);
 
 
   /**
@@ -575,7 +565,7 @@ struct TALER_AUDITORDB_Plugin
    *
    * @param cls the @e cls of this struct with the plugin-specific state
    * @param session connection to the database
-   * @param master_pub for which exchange do we want to get deposit confirmations
+   * @param master_public_key for which exchange do we want to get deposit confirmations
    * @param start_id row/serial ID where to start the iteration (0 from
    *                  the start, exclusive, i.e. serial_ids must start from 1)
    * @param cb function to call with results
@@ -583,13 +573,13 @@ struct TALER_AUDITORDB_Plugin
    * @return query result status
    */
   enum GNUNET_DB_QueryStatus
-  (*get_deposit_confirmations)(void *cls,
-                               struct TALER_AUDITORDB_Session *session,
-                               const struct
-                               TALER_MasterPublicKeyP *master_public_key,
-                               uint64_t start_id,
-                               TALER_AUDITORDB_DepositConfirmationCallback cb,
-                               void *cb_cls);
+  (*get_deposit_confirmations)(
+    void *cls,
+    struct TALER_AUDITORDB_Session *session,
+    const struct TALER_MasterPublicKeyP *master_public_key,
+    uint64_t start_id,
+    TALER_AUDITORDB_DepositConfirmationCallback cb,
+    void *cb_cls);
 
 
   /**
@@ -603,10 +593,10 @@ struct TALER_AUDITORDB_Plugin
    * @return status of database operation
    */
   enum GNUNET_DB_QueryStatus
-  (*insert_denomination_info)(void *cls,
-                              struct TALER_AUDITORDB_Session *session,
-                              const struct
-                              TALER_DenominationKeyValidityPS *issue);
+  (*insert_denomination_info)(
+    void *cls,
+    struct TALER_AUDITORDB_Session *session,
+    const struct TALER_DenominationKeyValidityPS *issue);
 
 
   /**
@@ -638,12 +628,11 @@ struct TALER_AUDITORDB_Plugin
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
-  (*insert_auditor_progress_coin)(void *cls,
-                                  struct TALER_AUDITORDB_Session *session,
-                                  const struct
-                                  TALER_MasterPublicKeyP *master_pub,
-                                  const struct
-                                  TALER_AUDITORDB_ProgressPointCoin *ppc);
+  (*insert_auditor_progress_coin)(
+    void *cls,
+    struct TALER_AUDITORDB_Session *session,
+    const struct TALER_MasterPublicKeyP *master_pub,
+    const struct TALER_AUDITORDB_ProgressPointCoin *ppc);
 
 
   /**
@@ -657,12 +646,11 @@ struct TALER_AUDITORDB_Plugin
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
-  (*update_auditor_progress_coin)(void *cls,
-                                  struct TALER_AUDITORDB_Session *session,
-                                  const struct
-                                  TALER_MasterPublicKeyP *master_pub,
-                                  const struct
-                                  TALER_AUDITORDB_ProgressPointCoin *ppc);
+  (*update_auditor_progress_coin)(
+    void *cls,
+    struct TALER_AUDITORDB_Session *session,
+    const struct TALER_MasterPublicKeyP *master_pub,
+    const struct TALER_AUDITORDB_ProgressPointCoin *ppc);
 
 
   /**
@@ -691,12 +679,11 @@ struct TALER_AUDITORDB_Plugin
  * @return transaction status code
  */
   enum GNUNET_DB_QueryStatus
-  (*insert_auditor_progress_reserve)(void *cls,
-                                     struct TALER_AUDITORDB_Session *session,
-                                     const struct
-                                     TALER_MasterPublicKeyP *master_pub,
-                                     const struct
-                                     TALER_AUDITORDB_ProgressPointReserve *ppr);
+  (*insert_auditor_progress_reserve)(
+    void *cls,
+    struct TALER_AUDITORDB_Session *session,
+    const struct TALER_MasterPublicKeyP *master_pub,
+    const struct TALER_AUDITORDB_ProgressPointReserve *ppr);
 
 
   /**
@@ -710,12 +697,11 @@ struct TALER_AUDITORDB_Plugin
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
-  (*update_auditor_progress_reserve)(void *cls,
-                                     struct TALER_AUDITORDB_Session *session,
-                                     const struct
-                                     TALER_MasterPublicKeyP *master_pub,
-                                     const struct
-                                     TALER_AUDITORDB_ProgressPointReserve *ppr);
+  (*update_auditor_progress_reserve)(
+    void *cls,
+    struct TALER_AUDITORDB_Session *session,
+    const struct TALER_MasterPublicKeyP *master_pub,
+    const struct TALER_AUDITORDB_ProgressPointReserve *ppr);
 
 
   /**
@@ -728,12 +714,11 @@ struct TALER_AUDITORDB_Plugin
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
-  (*get_auditor_progress_reserve)(void *cls,
-                                  struct TALER_AUDITORDB_Session *session,
-                                  const struct
-                                  TALER_MasterPublicKeyP *master_pub,
-                                  struct TALER_AUDITORDB_ProgressPointReserve *
-                                  ppr);
+  (*get_auditor_progress_reserve)(
+    void *cls,
+    struct TALER_AUDITORDB_Session *session,
+    const struct TALER_MasterPublicKeyP *master_pub,
+    struct TALER_AUDITORDB_ProgressPointReserve *ppr);
 
   /**
    * Insert information about the auditor's progress with an exchange's
@@ -746,15 +731,11 @@ struct TALER_AUDITORDB_Plugin
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
-  (*insert_auditor_progress_deposit_confirmation)(void *cls,
-                                                  struct TALER_AUDITORDB_Session
-                                                  *session,
-                                                  const struct
-                                                  TALER_MasterPublicKeyP *
-                                                  master_pub,
-                                                  const struct
-                                                  TALER_AUDITORDB_ProgressPointDepositConfirmation
-                                                  *ppdc);
+  (*insert_auditor_progress_deposit_confirmation)(
+    void *cls,
+    struct TALER_AUDITORDB_Session *session,
+    const struct TALER_MasterPublicKeyP *master_pub,
+    const struct TALER_AUDITORDB_ProgressPointDepositConfirmation *ppdc);
 
 
   /**
@@ -768,15 +749,11 @@ struct TALER_AUDITORDB_Plugin
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
-  (*update_auditor_progress_deposit_confirmation)(void *cls,
-                                                  struct TALER_AUDITORDB_Session
-                                                  *session,
-                                                  const struct
-                                                  TALER_MasterPublicKeyP *
-                                                  master_pub,
-                                                  const struct
-                                                  TALER_AUDITORDB_ProgressPointDepositConfirmation
-                                                  *ppdc);
+  (*update_auditor_progress_deposit_confirmation)(
+    void *cls,
+    struct TALER_AUDITORDB_Session *session,
+    const struct TALER_MasterPublicKeyP *master_pub,
+    const struct TALER_AUDITORDB_ProgressPointDepositConfirmation *ppdc);
 
 
   /**
@@ -789,15 +766,11 @@ struct TALER_AUDITORDB_Plugin
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
-  (*get_auditor_progress_deposit_confirmation)(void *cls,
-                                               struct TALER_AUDITORDB_Session *
-                                               session,
-                                               const struct
-                                               TALER_MasterPublicKeyP *
-                                               master_pub,
-                                               struct
-                                               TALER_AUDITORDB_ProgressPointDepositConfirmation
-                                               *ppdc);
+  (*get_auditor_progress_deposit_confirmation)(
+    void *cls,
+    struct TALER_AUDITORDB_Session *session,
+    const struct TALER_MasterPublicKeyP *master_pub,
+    struct TALER_AUDITORDB_ProgressPointDepositConfirmation *ppdc);
 
 
   /**
@@ -811,13 +784,11 @@ struct TALER_AUDITORDB_Plugin
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
-  (*insert_auditor_progress_aggregation)(void *cls,
-                                         struct TALER_AUDITORDB_Session *session,
-                                         const struct
-                                         TALER_MasterPublicKeyP *master_pub,
-                                         const struct
-                                         TALER_AUDITORDB_ProgressPointAggregation
-                                         *ppa);
+  (*insert_auditor_progress_aggregation)(
+    void *cls,
+    struct TALER_AUDITORDB_Session *session,
+    const struct TALER_MasterPublicKeyP *master_pub,
+    const struct TALER_AUDITORDB_ProgressPointAggregation *ppa);
 
 
   /**
@@ -831,13 +802,11 @@ struct TALER_AUDITORDB_Plugin
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
-  (*update_auditor_progress_aggregation)(void *cls,
-                                         struct TALER_AUDITORDB_Session *session,
-                                         const struct
-                                         TALER_MasterPublicKeyP *master_pub,
-                                         const struct
-                                         TALER_AUDITORDB_ProgressPointAggregation
-                                         *ppa);
+  (*update_auditor_progress_aggregation)(
+    void *cls,
+    struct TALER_AUDITORDB_Session *session,
+    const struct TALER_MasterPublicKeyP *master_pub,
+    const struct TALER_AUDITORDB_ProgressPointAggregation *ppa);
 
 
   /**
@@ -850,13 +819,11 @@ struct TALER_AUDITORDB_Plugin
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
-  (*get_auditor_progress_aggregation)(void *cls,
-                                      struct TALER_AUDITORDB_Session *session,
-                                      const struct
-                                      TALER_MasterPublicKeyP *master_pub,
-                                      struct
-                                      TALER_AUDITORDB_ProgressPointAggregation *
-                                      ppa);
+  (*get_auditor_progress_aggregation)(
+    void *cls,
+    struct TALER_AUDITORDB_Session *session,
+    const struct TALER_MasterPublicKeyP *master_pub,
+    struct TALER_AUDITORDB_ProgressPointAggregation *ppa);
 
 
   /**
@@ -870,22 +837,17 @@ struct TALER_AUDITORDB_Plugin
    * @param pp where is the auditor in processing
    * @param in_wire_off how far are we in the incoming wire transaction history
    * @param out_wire_off how far are we in the outgoing wire transaction history
-   * @param wire_off_size how many bytes do @a in_wire_off and @a out_wire_off take?
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
-  (*insert_wire_auditor_account_progress)(void *cls,
-                                          struct TALER_AUDITORDB_Session *
-                                          session,
-                                          const struct
-                                          TALER_MasterPublicKeyP *master_pub,
-                                          const char *account_name,
-                                          const struct
-                                          TALER_AUDITORDB_WireAccountProgressPoint
-                                          *pp,
-                                          const void *in_wire_off,
-                                          const void *out_wire_off,
-                                          size_t wire_off_size);
+  (*insert_wire_auditor_account_progress)(
+    void *cls,
+    struct TALER_AUDITORDB_Session *session,
+    const struct TALER_MasterPublicKeyP *master_pub,
+    const char *account_name,
+    const struct TALER_AUDITORDB_WireAccountProgressPoint *pp,
+    uint64_t in_wire_off,
+    uint64_t out_wire_off);
 
 
   /**
@@ -899,22 +861,17 @@ struct TALER_AUDITORDB_Plugin
    * @param pp where is the auditor in processing
    * @param in_wire_off how far are we in the incoming wire transaction history
    * @param out_wire_off how far are we in the outgoing wire transaction history
-   * @param wire_off_size how many bytes do @a in_wire_off and @a out_wire_off take?
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
-  (*update_wire_auditor_account_progress)(void *cls,
-                                          struct TALER_AUDITORDB_Session *
-                                          session,
-                                          const struct
-                                          TALER_MasterPublicKeyP *master_pub,
-                                          const char *account_name,
-                                          const struct
-                                          TALER_AUDITORDB_WireAccountProgressPoint
-                                          *pp,
-                                          const void *in_wire_off,
-                                          const void *out_wire_off,
-                                          size_t wire_off_size);
+  (*update_wire_auditor_account_progress)(
+    void *cls,
+    struct TALER_AUDITORDB_Session *session,
+    const struct TALER_MasterPublicKeyP *master_pub,
+    const char *account_name,
+    const struct TALER_AUDITORDB_WireAccountProgressPoint *pp,
+    uint64_t in_wire_off,
+    uint64_t out_wire_off);
 
 
   /**
@@ -927,21 +884,17 @@ struct TALER_AUDITORDB_Plugin
    * @param[out] pp where is the auditor in processing
    * @param[out] in_wire_off how far are we in the incoming wire transaction history
    * @param[out] out_wire_off how far are we in the outgoing wire transaction history
-   * @param[out] wire_off_size how many bytes do @a in_wire_off and @a out_wire_off take?
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
-  (*get_wire_auditor_account_progress)(void *cls,
-                                       struct TALER_AUDITORDB_Session *session,
-                                       const struct
-                                       TALER_MasterPublicKeyP *master_pub,
-                                       const char *account_name,
-                                       struct
-                                       TALER_AUDITORDB_WireAccountProgressPoint
-                                       *pp,
-                                       void **in_wire_off,
-                                       void **out_wire_off,
-                                       size_t *wire_off_size);
+  (*get_wire_auditor_account_progress)(
+    void *cls,
+    struct TALER_AUDITORDB_Session *session,
+    const struct TALER_MasterPublicKeyP *master_pub,
+    const char *account_name,
+    struct TALER_AUDITORDB_WireAccountProgressPoint *pp,
+    uint64_t *in_wire_off,
+    uint64_t *out_wire_off);
 
 
   /**
@@ -956,12 +909,11 @@ struct TALER_AUDITORDB_Plugin
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
-  (*insert_wire_auditor_progress)(void *cls,
-                                  struct TALER_AUDITORDB_Session *session,
-                                  const struct
-                                  TALER_MasterPublicKeyP *master_pub,
-                                  const struct
-                                  TALER_AUDITORDB_WireProgressPoint *pp);
+  (*insert_wire_auditor_progress)(
+    void *cls,
+    struct TALER_AUDITORDB_Session *session,
+    const struct TALER_MasterPublicKeyP *master_pub,
+    const struct TALER_AUDITORDB_WireProgressPoint *pp);
 
 
   /**
@@ -976,12 +928,11 @@ struct TALER_AUDITORDB_Plugin
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
-  (*update_wire_auditor_progress)(void *cls,
-                                  struct TALER_AUDITORDB_Session *session,
-                                  const struct
-                                  TALER_MasterPublicKeyP *master_pub,
-                                  const struct
-                                  TALER_AUDITORDB_WireProgressPoint *pp);
+  (*update_wire_auditor_progress)(
+    void *cls,
+    struct TALER_AUDITORDB_Session *session,
+    const struct TALER_MasterPublicKeyP *master_pub,
+    const struct TALER_AUDITORDB_WireProgressPoint *pp);
 
 
   /**
@@ -1213,7 +1164,7 @@ struct TALER_AUDITORDB_Plugin
    * @param denom_balance value of coins outstanding with this denomination key
    * @param denom_loss value of coins redeemed that were not outstanding (effectively, negative @a denom_balance)
    * @param denom_risk value of coins issued with this denomination key
-   * @param denom_payback value of coins paid back if this denomination key was revoked
+   * @param denom_recoup value of coins paid back if this denomination key was revoked
    * @param num_issued how many coins of this denomination did the exchange blind-sign
    * @return transaction status code
    */
@@ -1224,7 +1175,7 @@ struct TALER_AUDITORDB_Plugin
                                  const struct TALER_Amount *denom_balance,
                                  const struct TALER_Amount *denom_loss,
                                  const struct TALER_Amount *denom_risk,
-                                 const struct TALER_Amount *payback_loss,
+                                 const struct TALER_Amount *recoup_loss,
                                  uint64_t num_issued);
 
 
@@ -1238,7 +1189,7 @@ struct TALER_AUDITORDB_Plugin
    * @param denom_balance value of coins outstanding with this denomination key
    * @param denom_loss value of coins redeemed that were not outstanding (effectively, negative @a denom_balance)
    * @param denom_risk value of coins issued with this denomination key
-   * @param denom_payback value of coins paid back if this denomination key was revoked
+   * @param denom_recoup value of coins paid back if this denomination key was revoked
    * @param num_issued how many coins of this denomination did the exchange blind-sign
    * @return transaction status code
    */
@@ -1249,7 +1200,7 @@ struct TALER_AUDITORDB_Plugin
                                  const struct TALER_Amount *denom_balance,
                                  const struct TALER_Amount *denom_loss,
                                  const struct TALER_Amount *denom_risk,
-                                 const struct TALER_Amount *payback_loss,
+                                 const struct TALER_Amount *recoup_loss,
                                  uint64_t num_issued);
 
 
@@ -1262,7 +1213,7 @@ struct TALER_AUDITORDB_Plugin
    * @param[out] denom_balance value of coins outstanding with this denomination key
    * @param[out] denom_loss value of coins redeemed that were not outstanding (effectively, negative @a denom_balance)
    * @param[out] denom_risk value of coins issued with this denomination key
-   * @param[out] denom_payback value of coins paid back if this denomination key was revoked
+   * @param[out] denom_recoup value of coins paid back if this denomination key was revoked
    * @param[out] num_issued how many coins of this denomination did the exchange blind-sign
    * @return transaction status code
    */
@@ -1273,7 +1224,7 @@ struct TALER_AUDITORDB_Plugin
                               struct TALER_Amount *denom_balance,
                               struct TALER_Amount *denom_loss,
                               struct TALER_Amount *denom_risk,
-                              struct TALER_Amount *payback_loss,
+                              struct TALER_Amount *recoup_loss,
                               uint64_t *num_issued);
 
 
@@ -1303,7 +1254,9 @@ struct TALER_AUDITORDB_Plugin
    * @param melt_fee_balance total melt fees collected for this DK
    * @param refund_fee_balance total refund fees collected for this DK
    * @param risk maximum risk exposure of the exchange
-   * @param payback_loss actual losses from payback (actualized @a risk)
+   * @param recoup_loss actual losses from recoup (actualized @a risk)
+   * @param irregular_recoups recoups made of non-revoked coins (reduces
+   *             risk, but should never happen)
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
@@ -1315,7 +1268,8 @@ struct TALER_AUDITORDB_Plugin
                             const struct TALER_Amount *melt_fee_balance,
                             const struct TALER_Amount *refund_fee_balance,
                             const struct TALER_Amount *risk,
-                            const struct TALER_Amount *payback_loss);
+                            const struct TALER_Amount *recoup_loss,
+                            const struct TALER_Amount *irregular_recoups);
 
 
   /**
@@ -1330,7 +1284,9 @@ struct TALER_AUDITORDB_Plugin
    * @param melt_fee_balance total melt fees collected for this DK
    * @param refund_fee_balance total refund fees collected for this DK
    * @param risk maximum risk exposure of the exchange
-   * @param payback_loss actual losses from payback (actualized @a risk)
+   * @param recoup_loss actual losses from recoup (actualized @a risk)
+   * @param irregular_recoups recoups made of non-revoked coins (reduces
+   *             risk, but should never happen)
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
@@ -1342,7 +1298,8 @@ struct TALER_AUDITORDB_Plugin
                             const struct TALER_Amount *melt_fee_balance,
                             const struct TALER_Amount *refund_fee_balance,
                             const struct TALER_Amount *risk,
-                            const struct TALER_Amount *payback_loss);
+                            const struct TALER_Amount *recoup_loss,
+                            const struct TALER_Amount *irregular_recoups);
 
 
   /**
@@ -1356,7 +1313,9 @@ struct TALER_AUDITORDB_Plugin
    * @param[out] melt_fee_balance total melt fees collected for this DK
    * @param[out] refund_fee_balance total refund fees collected for this DK
    * @param[out] risk maximum risk exposure of the exchange
-   * @param[out] payback_loss actual losses from payback (actualized @a risk)
+   * @param[out] recoup_loss actual losses from recoup (actualized @a risk)
+   * @param[out] irregular_recoups recoups made of non-revoked coins (reduces
+   *             risk, but should never happen)
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
@@ -1368,7 +1327,8 @@ struct TALER_AUDITORDB_Plugin
                          struct TALER_Amount *melt_fee_balance,
                          struct TALER_Amount *refund_fee_balance,
                          struct TALER_Amount *risk,
-                         struct TALER_Amount *payback_loss);
+                         struct TALER_Amount *recoup_loss,
+                         struct TALER_Amount *irregular_recoup);
 
 
   /**
@@ -1383,20 +1343,18 @@ struct TALER_AUDITORDB_Plugin
    * @param revenue_balance what was the total profit made from
    *                        deposit fees, melting fees, refresh fees
    *                        and coins that were never returned?
-   * @param payback_loss_balance total losses from paybacks of revoked denominations
+   * @param recoup_loss_balance total losses from recoups of revoked denominations
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
-  (*insert_historic_denom_revenue)(void *cls,
-                                   struct TALER_AUDITORDB_Session *session,
-                                   const struct
-                                   TALER_MasterPublicKeyP *master_pub,
-                                   const struct GNUNET_HashCode *denom_pub_hash,
-                                   struct GNUNET_TIME_Absolute
-                                   revenue_timestamp,
-                                   const struct TALER_Amount *revenue_balance,
-                                   const struct
-                                   TALER_Amount *payback_loss_balance);
+  (*insert_historic_denom_revenue)(
+    void *cls,
+    struct TALER_AUDITORDB_Session *session,
+    const struct TALER_MasterPublicKeyP *master_pub,
+    const struct GNUNET_HashCode *denom_pub_hash,
+    struct GNUNET_TIME_Absolute revenue_timestamp,
+    const struct TALER_Amount *revenue_balance,
+    const struct TALER_Amount *recoup_loss_balance);
 
 
   /**
@@ -1411,13 +1369,12 @@ struct TALER_AUDITORDB_Plugin
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
-  (*select_historic_denom_revenue)(void *cls,
-                                   struct TALER_AUDITORDB_Session *session,
-                                   const struct
-                                   TALER_MasterPublicKeyP *master_pub,
-                                   TALER_AUDITORDB_HistoricDenominationRevenueDataCallback
-                                   cb,
-                                   void *cb_cls);
+  (*select_historic_denom_revenue)(
+    void *cls,
+    struct TALER_AUDITORDB_Session *session,
+    const struct TALER_MasterPublicKeyP *master_pub,
+    TALER_AUDITORDB_HistoricDenominationRevenueDataCallback cb,
+    void *cb_cls);
 
 
   /**
@@ -1432,14 +1389,13 @@ struct TALER_AUDITORDB_Plugin
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
-  (*insert_historic_reserve_revenue)(void *cls,
-                                     struct TALER_AUDITORDB_Session *session,
-                                     const struct
-                                     TALER_MasterPublicKeyP *master_pub,
-                                     struct GNUNET_TIME_Absolute start_time,
-                                     struct GNUNET_TIME_Absolute end_time,
-                                     const struct
-                                     TALER_Amount *reserve_profits);
+  (*insert_historic_reserve_revenue)(
+    void *cls,
+    struct TALER_AUDITORDB_Session *session,
+    const struct TALER_MasterPublicKeyP *master_pub,
+    struct GNUNET_TIME_Absolute start_time,
+    struct GNUNET_TIME_Absolute end_time,
+    const struct TALER_Amount *reserve_profits);
 
 
   /**
@@ -1453,13 +1409,12 @@ struct TALER_AUDITORDB_Plugin
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
-  (*select_historic_reserve_revenue)(void *cls,
-                                     struct TALER_AUDITORDB_Session *session,
-                                     const struct
-                                     TALER_MasterPublicKeyP *master_pub,
-                                     TALER_AUDITORDB_HistoricReserveRevenueDataCallback
-                                     cb,
-                                     void *cb_cls);
+  (*select_historic_reserve_revenue)(
+    void *cls,
+    struct TALER_AUDITORDB_Session *session,
+    const struct TALER_MasterPublicKeyP *master_pub,
+    TALER_AUDITORDB_HistoricReserveRevenueDataCallback cb,
+    void *cb_cls);
 
 
   /**
