@@ -22,13 +22,8 @@
 #ifndef TALER_CRYPTO_LIB_H
 #define TALER_CRYPTO_LIB_H
 
-#if HAVE_GNUNET_GNUNET_UTIL_LIB_H
 #include <gnunet/gnunet_util_lib.h>
 #include "taler_util.h"
-#elif HAVE_GNUNET_GNUNET_UTIL_TALER_WALLET_LIB_H
-#include <gnunet/gnunet_util_taler_wallet_lib.h>
-#include "taler_util_wallet.h"
-#endif
 
 #include <gcrypt.h>
 
@@ -460,7 +455,7 @@ struct TALER_PlanchetDetail
   /**
    * Blinded coin (see GNUNET_CRYPTO_rsa_blind()).  Note: is malloc()'ed!
    */
-  char *coin_ev;
+  void *coin_ev;
 
   /**
    * Number of bytes in @a coin_ev.
@@ -510,12 +505,12 @@ struct TALER_TransferSecretP
  * Length of the raw value in the Taler wire transfer identifier
  * (in binary representation).
  */
-#define TALER_WIRE_TRANSFER_IDENTIFIER_LEN 32
+#define TALER_BANK_TRANSFER_IDENTIFIER_LEN 32
 
 /**
- * #TALER_WIRE_TRANSFER_IDENTIFIER_LEN as a string.
+ * #TALER_BANK_TRANSFER_IDENTIFIER_LEN as a string.
  */
-#define TALER_WIRE_TRANSFER_IDENTIFIER_LEN_STR "32"
+#define TALER_BANK_TRANSFER_IDENTIFIER_LEN_STR "32"
 
 
 /**
@@ -536,7 +531,7 @@ struct TALER_WireTransferIdentifierRawP
    * encode the actual value (i.e. a 256-bit / 32-byte public key or
    * a hash code), and the last byte for a minimalistic checksum.
    */
-  uint8_t raw[TALER_WIRE_TRANSFER_IDENTIFIER_LEN];
+  uint8_t raw[TALER_BANK_TRANSFER_IDENTIFIER_LEN];
 };
 
 
@@ -638,11 +633,10 @@ TALER_planchet_to_coin (const struct TALER_DenominationPublicKey *dk,
  * @param[out] ts computed transfer secret
  */
 void
-TALER_link_derive_transfer_secret (const struct
-                                   TALER_CoinSpendPrivateKeyP *coin_priv,
-                                   const struct
-                                   TALER_TransferPrivateKeyP *trans_priv,
-                                   struct TALER_TransferSecretP *ts);
+TALER_link_derive_transfer_secret (
+  const struct TALER_CoinSpendPrivateKeyP *coin_priv,
+  const struct TALER_TransferPrivateKeyP *trans_priv,
+  struct TALER_TransferSecretP *ts);
 
 
 /**
@@ -654,11 +648,10 @@ TALER_link_derive_transfer_secret (const struct
  * @param[out] transfer_secret set to the shared secret
  */
 void
-TALER_link_reveal_transfer_secret (const struct
-                                   TALER_TransferPrivateKeyP *trans_priv,
-                                   const struct
-                                   TALER_CoinSpendPublicKeyP *coin_pub,
-                                   struct TALER_TransferSecretP *transfer_secret);
+TALER_link_reveal_transfer_secret (
+  const struct TALER_TransferPrivateKeyP *trans_priv,
+  const struct TALER_CoinSpendPublicKeyP *coin_pub,
+  struct TALER_TransferSecretP *transfer_secret);
 
 
 /**
@@ -670,12 +663,10 @@ TALER_link_reveal_transfer_secret (const struct
  * @param[out] transfer_secret set to the shared secret
  */
 void
-TALER_link_recover_transfer_secret (const struct
-                                    TALER_TransferPublicKeyP *trans_pub,
-                                    const struct
-                                    TALER_CoinSpendPrivateKeyP *coin_priv,
-                                    struct TALER_TransferSecretP *
-                                    transfer_secret);
+TALER_link_recover_transfer_secret (
+  const struct TALER_TransferPublicKeyP *trans_pub,
+  const struct TALER_CoinSpendPrivateKeyP *coin_priv,
+  struct TALER_TransferSecretP *transfer_secret);
 
 
 /**
@@ -692,7 +683,7 @@ struct TALER_RefreshCoinData
   /**
    * The envelope with the blinded coin.
    */
-  char *coin_ev;
+  void *coin_ev;
 
   /**
    * Number of bytes in @a coin_ev
@@ -726,7 +717,7 @@ struct TALER_RefreshCommitmentEntry
  * @param[out] rc set to the value the wallet must commit to
  * @param kappa number of transfer public keys involved (must be #TALER_CNC_KAPPA)
  * @param num_new_coins number of new coins to be created
- * @param commitments array of @a kappa commitments
+ * @param rcs array of @a kappa commitments
  * @param coin_pub public key of the coin to be melted
  * @param amount_with_fee amount to be melted, including fee
  */
@@ -744,56 +735,56 @@ TALER_refresh_get_commitment (struct TALER_RefreshCommitmentP *rc,
 
 /**
  * Compute the hash of the given wire details.   The resulting
- * hash is what is put into the contract.
+ * hash is what is signed by the master key.
  *
- * @param payto_url bank account
- * @param hc[out] set to the hash
+ * @param payto_uri bank account
+ * @param[out] hc set to the hash
  */
 void
-TALER_exchange_wire_signature_hash (const char *payto_url,
+TALER_exchange_wire_signature_hash (const char *payto_uri,
                                     struct GNUNET_HashCode *hc);
 
 
 /**
- * Check the signature in @a wire_s.
+ * Check the signature in @a master_sig.
  *
- * @param payto_url URL that is signed
+ * @param payto_uri URL that is signed
  * @param master_pub master public key of the exchange
  * @param master_sig signature of the exchange
  * @return #GNUNET_OK if signature is valid
  */
 int
-TALER_exchange_wire_signature_check (const char *payto_url,
-                                     const struct
-                                     TALER_MasterPublicKeyP *master_pub,
-                                     const struct
-                                     TALER_MasterSignatureP *master_sig);
+TALER_exchange_wire_signature_check (
+  const char *payto_uri,
+  const struct TALER_MasterPublicKeyP *master_pub,
+  const struct TALER_MasterSignatureP *master_sig);
 
 
 /**
  * Create a signed wire statement for the given account.
  *
- * @param payto_url account specification
+ * @param payto_uri account specification
  * @param master_priv private key to sign with
- * @param master_sig[out] where to write the signature
+ * @param[out] master_sig where to write the signature
  */
 void
-TALER_exchange_wire_signature_make (const char *payto_url,
-                                    const struct
-                                    TALER_MasterPrivateKeyP *master_priv,
-                                    struct TALER_MasterSignatureP *master_sig);
+TALER_exchange_wire_signature_make (
+  const char *payto_uri,
+  const struct TALER_MasterPrivateKeyP *master_priv,
+  struct TALER_MasterSignatureP *master_sig);
 
 
 /**
  * Compute the hash of the given wire details.   The resulting
- * hash is what is put into the contract.
+ * @a hc is what will be put into the contract between customer
+ * and merchant for signing by both parties.
  *
- * @param payto_url bank account
+ * @param payto_uri bank account
  * @param salt salt used to eliminate brute-force inversion
- * @param hc[out] set to the hash
+ * @param[out] hc set to the hash
  */
 void
-TALER_merchant_wire_signature_hash (const char *payto_url,
+TALER_merchant_wire_signature_hash (const char *payto_uri,
                                     const char *salt,
                                     struct GNUNET_HashCode *hc);
 
@@ -801,35 +792,34 @@ TALER_merchant_wire_signature_hash (const char *payto_url,
 /**
  * Check the signature in @a wire_s.
  *
- * @param payto_url URL that is signed
- * @param salt the salt used to salt the @a payto_url when hashing
+ * @param payto_uri URL that is signed
+ * @param salt the salt used to salt the @a payto_uri when hashing
  * @param merch_pub public key of the merchant
  * @param merch_sig signature of the merchant
  * @return #GNUNET_OK if signature is valid
  */
 int
-TALER_merchant_wire_signature_check (const char *payto_url,
-                                     const char *salt,
-                                     const struct
-                                     TALER_MerchantPublicKeyP *merch_pub,
-                                     const struct
-                                     TALER_MerchantSignatureP *merch_sig);
+TALER_merchant_wire_signature_check (
+  const char *payto_uri,
+  const char *salt,
+  const struct TALER_MerchantPublicKeyP *merch_pub,
+  const struct TALER_MerchantSignatureP *merch_sig);
 
 
 /**
  * Create a signed wire statement for the given account.
  *
- * @param payto_url account specification
- * @param salt the salt used to salt the @a payto_url when hashing
+ * @param payto_uri account specification
+ * @param salt the salt used to salt the @a payto_uri when hashing
  * @param merch_priv private key to sign with
- * @param merch_sig[out] where to write the signature
+ * @param[out] merch_sig where to write the signature
  */
 void
-TALER_merchant_wire_signature_make (const char *payto_url,
-                                    const char *salt,
-                                    const struct
-                                    TALER_MerchantPrivateKeyP *merch_priv,
-                                    struct TALER_MerchantSignatureP *merch_sig);
+TALER_merchant_wire_signature_make (
+  const char *payto_uri,
+  const char *salt,
+  const struct TALER_MerchantPrivateKeyP *merch_priv,
+  struct TALER_MerchantSignatureP *merch_sig);
 
 
 #endif
