@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2017 Inria & GNUnet e.V.
+  Copyright (C) 2014-2017 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -23,9 +23,9 @@
 #ifndef TALER_EXCHANGEDB_LIB_H
 #define TALER_EXCHANGEDB_LIB_H
 
-
 #include "taler_signatures.h"
-
+#include "taler_exchangedb_plugin.h"
+#include "taler_bank_service.h"
 
 /**
  * Subdirectroy under the exchange's base directory which contains
@@ -40,81 +40,6 @@
 #define TALER_EXCHANGEDB_DIR_DENOMINATION_KEYS "denomkeys"
 
 
-GNUNET_NETWORK_STRUCT_BEGIN
-
-/**
- * @brief On disk format used for a exchange signing key.  Signing keys are used
- * by the exchange to affirm its messages, but not to create coins.
- * Includes the private key followed by the public information about
- * the signing key.
- */
-struct TALER_EXCHANGEDB_PrivateSigningKeyInformationP
-{
-  /**
-   * Private key part of the exchange's signing key.
-   */
-  struct TALER_ExchangePrivateKeyP signkey_priv;
-
-  /**
-   * Signature over @e issue
-   */
-  struct TALER_MasterSignatureP master_sig;
-
-  /**
-   * Public information about a exchange signing key.
-   */
-  struct TALER_ExchangeSigningKeyValidityPS issue;
-
-};
-
-
-/**
- * Information about a denomination key.
- */
-struct TALER_EXCHANGEDB_DenominationKeyInformationP
-{
-
-  /**
-   * Signature over this struct to affirm the validity of the key.
-   */
-  struct TALER_MasterSignatureP signature;
-
-  /**
-   * Signed properties of the denomination key.
-   */
-  struct TALER_DenominationKeyValidityPS properties;
-};
-
-
-GNUNET_NETWORK_STRUCT_END
-
-
-/**
- * @brief All information about a denomination key (which is used to
- * sign coins into existence).
- */
-struct TALER_EXCHANGEDB_DenominationKeyIssueInformation
-{
-  /**
-   * The private key of the denomination.  Will be NULL if the private
-   * key is not available (this is the case after the key has expired
-   * for signing coins, but is still valid for depositing coins).
-   */
-  struct TALER_DenominationPrivateKey denom_priv;
-
-  /**
-   * Decoded denomination public key (the hash of it is in
-   * @e issue, but we sometimes need the full public key as well).
-   */
-  struct TALER_DenominationPublicKey denom_pub;
-
-  /**
-   * Signed public information about a denomination key.
-   */
-  struct TALER_EXCHANGEDB_DenominationKeyInformationP issue;
-};
-
-
 /**
  * @brief Iterator over signing keys.
  *
@@ -126,11 +51,10 @@ struct TALER_EXCHANGEDB_DenominationKeyIssueInformation
  *  #GNUNET_SYSERR to abort iteration with error!
  */
 typedef int
-(*TALER_EXCHANGEDB_SigningKeyIterator)(void *cls,
-                                       const char *filename,
-                                       const struct
-                                       TALER_EXCHANGEDB_PrivateSigningKeyInformationP
-                                       *ski);
+(*TALER_EXCHANGEDB_SigningKeyIterator)(
+  void *cls,
+  const char *filename,
+  const struct TALER_EXCHANGEDB_PrivateSigningKeyInformationP *ski);
 
 
 /**
@@ -160,11 +84,10 @@ TALER_EXCHANGEDB_signing_keys_iterate (const char *exchange_base_dir,
  * @return #GNUNET_OK upon success; #GNUNET_SYSERR upon failure.
  */
 int
-TALER_EXCHANGEDB_signing_key_write (const char *exchange_base_dir,
-                                    struct GNUNET_TIME_Absolute start,
-                                    const struct
-                                    TALER_EXCHANGEDB_PrivateSigningKeyInformationP
-                                    *ski);
+TALER_EXCHANGEDB_signing_key_write (
+  const char *exchange_base_dir,
+  struct GNUNET_TIME_Absolute start,
+  const struct TALER_EXCHANGEDB_PrivateSigningKeyInformationP *ski);
 
 
 /**
@@ -178,11 +101,10 @@ TALER_EXCHANGEDB_signing_key_write (const char *exchange_base_dir,
  *  #GNUNET_SYSERR to abort iteration with error!
  */
 typedef int
-(*TALER_EXCHANGEDB_DenominationKeyIterator)(void *cls,
-                                            const char *alias,
-                                            const struct
-                                            TALER_EXCHANGEDB_DenominationKeyIssueInformation
-                                            *dki);
+(*TALER_EXCHANGEDB_DenominationKeyIterator)(
+  void *cls,
+  const char *alias,
+  const struct TALER_EXCHANGEDB_DenominationKey *dki);
 
 
 /**
@@ -196,11 +118,10 @@ typedef int
  *  #GNUNET_SYSERR to abort iteration with error!
  */
 typedef int
-(*TALER_EXCHANGEDB_RevocationIterator)(void *cls,
-                                       const struct GNUNET_HashCode *denom_hash,
-                                       const struct
-                                       TALER_MasterSignatureP *
-                                       revocation_master_sig);
+(*TALER_EXCHANGEDB_RevocationIterator)(
+  void *cls,
+  const struct GNUNET_HashCode *denom_hash,
+  const struct TALER_MasterSignatureP *revocation_master_sig);
 
 
 /**
@@ -217,10 +138,10 @@ typedef int
  *         as maybe none of the files were well-formed)
  */
 int
-TALER_EXCHANGEDB_denomination_keys_iterate (const char *exchange_base_dir,
-                                            TALER_EXCHANGEDB_DenominationKeyIterator
-                                            it,
-                                            void *it_cls);
+TALER_EXCHANGEDB_denomination_keys_iterate (
+  const char *exchange_base_dir,
+  TALER_EXCHANGEDB_DenominationKeyIterator it,
+  void *it_cls);
 
 
 /**
@@ -236,16 +157,16 @@ TALER_EXCHANGEDB_denomination_keys_iterate (const char *exchange_base_dir,
  *         as maybe none of the files were well-formed)
  */
 int
-TALER_EXCHANGEDB_revocations_iterate (const char *revocation_dir,
-                                      const struct
-                                      TALER_MasterPublicKeyP *master_pub,
-                                      TALER_EXCHANGEDB_RevocationIterator it,
-                                      void *it_cls);
+TALER_EXCHANGEDB_revocations_iterate (
+  const char *revocation_dir,
+  const struct TALER_MasterPublicKeyP *master_pub,
+  TALER_EXCHANGEDB_RevocationIterator it,
+  void *it_cls);
 
 
 /**
  * Mark the given denomination key as revoked and request the wallets
- * to initiate /payback.
+ * to initiate recoup.
  *
  * @param revocation_dir where to write the revocation certificate
  * @param denom_hash hash of the denomination key to revoke
@@ -253,11 +174,10 @@ TALER_EXCHANGEDB_revocations_iterate (const char *revocation_dir,
  * @return #GNUNET_OK upon success; #GNUNET_SYSERR upon failure.
  */
 int
-TALER_EXCHANGEDB_denomination_key_revoke (const char *revocation_dir,
-                                          const struct
-                                          GNUNET_HashCode *denom_hash,
-                                          const struct
-                                          TALER_MasterPrivateKeyP *mpriv);
+TALER_EXCHANGEDB_denomination_key_revoke (
+  const char *revocation_dir,
+  const struct GNUNET_HashCode *denom_hash,
+  const struct TALER_MasterPrivateKeyP *mpriv);
 
 
 /**
@@ -268,10 +188,9 @@ TALER_EXCHANGEDB_denomination_key_revoke (const char *revocation_dir,
  * @return #GNUNET_OK upon success; #GNUNET_SYSERR upon failure.
  */
 int
-TALER_EXCHANGEDB_denomination_key_write (const char *filename,
-                                         const struct
-                                         TALER_EXCHANGEDB_DenominationKeyIssueInformation
-                                         *dki);
+TALER_EXCHANGEDB_denomination_key_write (
+  const char *filename,
+  const struct TALER_EXCHANGEDB_DenominationKey *dki);
 
 
 /**
@@ -282,10 +201,9 @@ TALER_EXCHANGEDB_denomination_key_write (const char *filename,
  * @return #GNUNET_OK upon success; #GNUNET_SYSERR upon failure
  */
 int
-TALER_EXCHANGEDB_denomination_key_read (const char *filename,
-                                        struct
-                                        TALER_EXCHANGEDB_DenominationKeyIssueInformation
-                                        *dki);
+TALER_EXCHANGEDB_denomination_key_read (
+  const char *filename,
+  struct TALER_EXCHANGEDB_DenominationKey *dki);
 
 
 /**
@@ -303,14 +221,14 @@ TALER_EXCHANGEDB_denomination_key_read (const char *filename,
  *  #GNUNET_SYSERR to abort iteration with error!
  */
 typedef int
-(*TALER_EXCHANGEDB_AuditorIterator)(void *cls,
-                                    const struct TALER_AuditorPublicKeyP *apub,
-                                    const char *auditor_url,
-                                    const struct TALER_MasterPublicKeyP *mpub,
-                                    unsigned int dki_len,
-                                    const struct TALER_AuditorSignatureP *asigs,
-                                    const struct
-                                    TALER_DenominationKeyValidityPS *dki);
+(*TALER_EXCHANGEDB_AuditorIterator)(
+  void *cls,
+  const struct TALER_AuditorPublicKeyP *apub,
+  const char *auditor_url,
+  const struct TALER_MasterPublicKeyP *mpub,
+  unsigned int dki_len,
+  const struct TALER_AuditorSignatureP *asigs,
+  const struct TALER_DenominationKeyValidityPS *dki);
 
 
 /**
@@ -344,14 +262,14 @@ TALER_EXCHANGEDB_auditor_iterate (const struct GNUNET_CONFIGURATION_Handle *cfg,
  * @return #GNUNET_OK upon success; #GNUNET_SYSERR upon failure.
  */
 int
-TALER_EXCHANGEDB_auditor_write (const char *filename,
-                                const struct TALER_AuditorPublicKeyP *apub,
-                                const char *auditor_url,
-                                const struct TALER_AuditorSignatureP *asigs,
-                                const struct TALER_MasterPublicKeyP *mpub,
-                                unsigned int dki_len,
-                                const struct
-                                TALER_DenominationKeyValidityPS *dki);
+TALER_EXCHANGEDB_auditor_write (
+  const char *filename,
+  const struct TALER_AuditorPublicKeyP *apub,
+  const char *auditor_url,
+  const struct TALER_AuditorSignatureP *asigs,
+  const struct TALER_MasterPublicKeyP *mpub,
+  uint32_t dki_len,
+  const struct TALER_DenominationKeyValidityPS *dki);
 
 
 /**
@@ -375,6 +293,8 @@ TALER_EXCHANGEDB_plugin_unload (struct TALER_EXCHANGEDB_Plugin *plugin);
 
 /**
  * Sorted list of fees to be paid for aggregate wire transfers.
+ * Sorted by @e start_date or @e end_date --- both work fine as
+ * the resulting order must be the same.
  */
 struct TALER_EXCHANGEDB_AggregateFees
 {
@@ -425,12 +345,12 @@ TALER_EXCHANGEDB_fees_read (const struct GNUNET_CONFIGURATION_Handle *cfg,
 /**
  * Convert @a af to @a wf.
  *
- * @param wireplugin name of the wire plugin the fees are for
+ * @param wiremethod name of the wire method the fees are for
  * @param[in,out] af aggregate fees, host format (updated to round time)
  * @param[out] wf aggregate fees, disk / signature format
  */
 void
-TALER_EXCHANGEDB_fees_2_wf (const char *wireplugin,
+TALER_EXCHANGEDB_fees_2_wf (const char *wiremethod,
                             struct TALER_EXCHANGEDB_AggregateFees *af,
                             struct TALER_MasterWireFeePS *wf);
 
@@ -465,20 +385,19 @@ struct TALER_EXCHANGEDB_AccountInfo
 {
   /**
    * Section in the configuration file that specifies the
-   * account. Must start with "account-".
+   * account. Must start with "exchange-account-".
    */
   const char *section_name;
 
   /**
-   * Name of the wire plugin that should be used to access
-   * the account.
+   * Name of the wire method used by this account.
    */
-  const char *plugin_name;
+  const char *method;
 
   /**
    * payto://-URL of the account.
    */
-  const char *payto_url;
+  const char *payto_uri;
 
   /**
    * Filename containing the signed /wire response, or NULL
@@ -509,9 +428,9 @@ struct TALER_EXCHANGEDB_AccountInfo
  * @param ai account information
  */
 typedef void
-(*TALER_EXCHANGEDB_AccountCallback)(void *cls,
-                                    const struct
-                                    TALER_EXCHANGEDB_AccountInfo *ai);
+(*TALER_EXCHANGEDB_AccountCallback)(
+  void *cls,
+  const struct TALER_EXCHANGEDB_AccountInfo *ai);
 
 /**
  * Parse the configuration to find account information.
@@ -526,4 +445,121 @@ TALER_EXCHANGEDB_find_accounts (const struct GNUNET_CONFIGURATION_Handle *cfg,
                                 void *cb_cls);
 
 
+/**
+ * Calculate the total value of all transactions performed.
+ * Stores @a off plus the cost of all transactions in @a tl
+ * in @a ret.
+ *
+ * @param tl transaction list to process
+ * @param off offset to use as the starting value
+ * @param[out] ret where the resulting total is to be stored
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR on errors
+ */
+int
+TALER_EXCHANGEDB_calculate_transaction_list_totals (
+  struct TALER_EXCHANGEDB_TransactionList *tl,
+  const struct TALER_Amount *off,
+  struct TALER_Amount *ret);
+
+
+/* ***************** convenience functions ******** */
+
+/**
+ * Information we keep for each supported account of the exchange.
+ */
+struct TALER_EXCHANGEDB_WireAccount
+{
+  /**
+   * Accounts are kept in a DLL.
+   */
+  struct TALER_EXCHANGEDB_WireAccount *next;
+
+  /**
+   * Plugins are kept in a DLL.
+   */
+  struct TALER_EXCHANGEDB_WireAccount *prev;
+
+  /**
+   * Authentication data.
+   */
+  struct TALER_BANK_AuthenticationData auth;
+
+  /**
+   * Wire transfer fee structure.
+   */
+  struct TALER_EXCHANGEDB_AggregateFees *af;
+
+  /**
+   * Name of the section that configures this account.
+   */
+  char *section_name;
+
+  /**
+   * Name of the wire method underlying the account.
+   */
+  char *method;
+
+};
+
+
+/**
+ * Update wire transfer fee data structure in @a wa.
+ *
+ * @param cfg configuration to use
+ * @param db_plugin database plugin to use
+ * @param wa wire account data structure to update
+ * @param now timestamp to update fees to
+ * @param session DB session to use
+ * @return fee valid at @a now, or NULL if unknown
+ */
+struct TALER_EXCHANGEDB_AggregateFees *
+TALER_EXCHANGEDB_update_fees (const struct GNUNET_CONFIGURATION_Handle *cfg,
+                              struct TALER_EXCHANGEDB_Plugin *db_plugin,
+                              struct TALER_EXCHANGEDB_WireAccount *wa,
+                              struct GNUNET_TIME_Absolute now,
+                              struct TALER_EXCHANGEDB_Session *session);
+
+
+/**
+ * Find the wire plugin for the given payto:// URL.
+ * Only useful after the accounts have been loaded
+ * using #TALER_EXCHANGEDB_load_accounts().
+ *
+ * @param method wire method we need an account for
+ * @return NULL on error
+ */
+struct TALER_EXCHANGEDB_WireAccount *
+TALER_EXCHANGEDB_find_account_by_method (const char *method);
+
+
+/**
+ * Find the wire plugin for the given payto:// URL
+ * Only useful after the accounts have been loaded
+ * using #TALER_EXCHANGEDB_load_accounts().
+ *
+ * @param url wire address we need an account for
+ * @return NULL on error
+ */
+struct TALER_EXCHANGEDB_WireAccount *
+TALER_EXCHANGEDB_find_account_by_payto_uri (const char *url);
+
+
+/**
+ * Load account information opf the exchange from
+ * @a cfg.
+ *
+ * @param cfg configuration to load from
+ * @return #GNUNET_OK on success, #GNUNET_NO if no accounts are configured
+ */
+int
+TALER_EXCHANGEDB_load_accounts (const struct GNUNET_CONFIGURATION_Handle *cfg);
+
+
+/**
+ * Free resources allocated by
+ * #TALER_EXCHANGEDB_load_accounts().
+ */
+void
+TALER_EXCHANGEDB_unload_accounts (void);
+
 #endif
