@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014, 2015, 2016 GNUnet e.V.
+  Copyright (C) 2014, 2015, 2016 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -107,6 +107,12 @@ TALER_JSON_spec_denomination_signature (const char *field,
 /**
  * Hash a JSON for binary signing.
  *
+ * See https://tools.ietf.org/html/draft-rundgren-json-canonicalization-scheme-15
+ * for fun JSON canonicalization problems.  Callers must ensure that
+ * those are avoided in the input. We will use libjanson's "JSON_COMPACT"
+ * encoding for whitespace and "JSON_SORT_KEYS" to canonicalize as best
+ * as we can.
+ *
  * @param[in] json some JSON value to hash
  * @param[out] hc resulting hash code
  * @return #GNUNET_OK on success, #GNUNET_SYSERR on error
@@ -143,10 +149,11 @@ TALER_JSON_get_error_code2 (const void *data,
 
 /**
  * Compute the hash of the given wire details.   The resulting
- * hash is what is put into the contract.
+ * hash is what is put into the contract.  Also performs rudimentary
+ * checks on the account data *if* supported.
  *
  * @param wire_s wire details to hash
- * @param hc[out] set to the hash
+ * @param[out] hc set to the hash
  * @return #GNUNET_OK on success, #GNUNET_SYSERR if @a wire_s is malformed
  */
 int
@@ -155,41 +162,30 @@ TALER_JSON_merchant_wire_signature_hash (const json_t *wire_s,
 
 
 /**
- * Compute the hash of the given wire details.   The resulting
- * hash is what is signed by the master public key.
- *
- * @param wire_s wire details to hash
- * @param hc[out] set to the hash
- * @return #GNUNET_OK on success, #GNUNET_SYSERR if @a wire_s is malformed
- */
-int
-TALER_JSON_exchange_wire_signature_hash (const json_t *wire_s,
-                                         struct GNUNET_HashCode *hc);
-
-
-/**
- * Check the signature in @a wire_s.
+ * Check the signature in @a wire_s.  Also performs rudimentary
+ * checks on the account data *if* supported.
  *
  * @param wire_s signed wire information of an exchange
  * @param master_pub master public key of the exchange
  * @return #GNUNET_OK if signature is valid
  */
 int
-TALER_JSON_exchange_wire_signature_check (const json_t *wire_s,
-                                          const struct
-                                          TALER_MasterPublicKeyP *master_pub);
+TALER_JSON_exchange_wire_signature_check (
+  const json_t *wire_s,
+  const struct TALER_MasterPublicKeyP *master_pub);
 
 
 /**
  * Create a signed wire statement for the given account.
  *
- * @param payto_url account specification
- * @param master_priv private key to sign with, NULL to not sign
+ * @param payto_uri account specification
+ * @param master_priv private key to sign with
+ * @return NULL if @a payto_uri is malformed
  */
 json_t *
-TALER_JSON_exchange_wire_signature_make (const char *payto_url,
-                                         const struct
-                                         TALER_MasterPrivateKeyP *master_priv);
+TALER_JSON_exchange_wire_signature_make (
+  const char *payto_uri,
+  const struct TALER_MasterPrivateKeyP *master_priv);
 
 
 /**
@@ -206,7 +202,7 @@ TALER_JSON_wire_to_method (const json_t *wire_s);
 /**
  * Obtain the payto://-URL associated with the given
  * wire account details.  @a wire_s must contain a payto://-URL
- * under 'url'.
+ * under 'payto_uri'.
  *
  * @return NULL on error
  */
