@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2017 GNUnet e.V.
+  Copyright (C) 2014-2017 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -27,12 +27,7 @@
 #ifndef TALER_SIGNATURES_H
 #define TALER_SIGNATURES_H
 
-#if HAVE_GNUNET_GNUNET_UTIL_LIB_H
 #include <gnunet/gnunet_util_lib.h>
-#elif HAVE_GNUNET_GNUNET_UTIL_TALER_WALLET_LIB_H
-#include <gnunet/gnunet_util_taler_wallet_lib.h>
-#endif
-
 #include "taler_amount_lib.h"
 #include "taler_crypto_lib.h"
 
@@ -120,9 +115,9 @@
 #define TALER_SIGNATURE_EXCHANGE_CONFIRM_REFUND 1038
 
 /**
- * Signature where the Exchange confirms a payback.
+ * Signature where the Exchange confirms a recoup.
  */
-#define TALER_SIGNATURE_EXCHANGE_CONFIRM_PAYBACK 1039
+#define TALER_SIGNATURE_EXCHANGE_CONFIRM_RECOUP 1039
 
 /**
  * Signature where the Exchange confirms it closed a reserve.
@@ -130,9 +125,9 @@
 #define TALER_SIGNATURE_EXCHANGE_RESERVE_CLOSED 1040
 
 /**
- * Signature where the Exchange confirms a payback-refresh operation.
+ * Signature where the Exchange confirms a recoup-refresh operation.
  */
-#define TALER_SIGNATURE_EXCHANGE_CONFIRM_PAYBACK_REFRESH 1041
+#define TALER_SIGNATURE_EXCHANGE_CONFIRM_RECOUP_REFRESH 1041
 
 
 /**********************/
@@ -210,9 +205,9 @@
 #define TALER_SIGNATURE_WALLET_COIN_MELT 1202
 
 /**
- * Signature using a coin key requesting payback.
+ * Signature using a coin key requesting recoup.
  */
-#define TALER_SIGNATURE_WALLET_COIN_PAYBACK 1203
+#define TALER_SIGNATURE_WALLET_COIN_RECOUP 1203
 
 /**
  * Signature using a coin key authenticating link data.
@@ -328,8 +323,8 @@ struct TALER_WithdrawRequestPS
 
   /**
    * Withdrawl fee charged by the exchange.  This must match the Exchange's
-   * denomination key's withdrawl fee.  If the client puts in an
-   * invalid withdrawl fee (too high or too low) that does not match
+   * denomination key's withdrawal fee.  If the client puts in an
+   * invalid withdrawal fee (too high or too low) that does not match
    * the Exchange's denomination key, the withdraw operation is invalid
    * and will be rejected by the exchange.  The @e amount_with_fee minus
    * the @e withdraw_fee is must match the value of the generated
@@ -960,7 +955,7 @@ struct TALER_MasterWireDetailsPS
 
   /**
    * Hash over the account holder's payto:// URL and
-   * the salt, as done by #TALER_wire_signature_hash().
+   * the salt, as done by #TALER_exchange_wire_signature_hash().
    */
   struct GNUNET_HashCode h_wire_details GNUNET_PACKED;
 
@@ -1228,13 +1223,13 @@ struct TALER_ConfirmWirePS
 
 /**
  * Signed data to request that a coin should be refunded as part of
- * the "emergency" /payback protocol.  The refund will go back to the bank
+ * the "emergency" /recoup protocol.  The refund will go back to the bank
  * account that created the reserve.
  */
-struct TALER_PaybackRequestPS
+struct TALER_RecoupRequestPS
 {
   /**
-   * Purpose is #TALER_SIGNATURE_WALLET_COIN_PAYBACK
+   * Purpose is #TALER_SIGNATURE_WALLET_COIN_RECOUP
    */
   struct GNUNET_CRYPTO_EccSignaturePurpose purpose;
 
@@ -1257,20 +1252,20 @@ struct TALER_PaybackRequestPS
 
 /**
  * Response by which the exchange affirms that it will
- * refund a coin as part of the emergency /payback
- * protocol.  The payback will go back to the bank
+ * refund a coin as part of the emergency /recoup
+ * protocol.  The recoup will go back to the bank
  * account that created the reserve.
  */
-struct TALER_PaybackConfirmationPS
+struct TALER_RecoupConfirmationPS
 {
 
   /**
-   * Purpose is #TALER_SIGNATURE_EXCHANGE_CONFIRM_PAYBACK
+   * Purpose is #TALER_SIGNATURE_EXCHANGE_CONFIRM_RECOUP
    */
   struct GNUNET_CRYPTO_EccSignaturePurpose purpose;
 
   /**
-   * When did the exchange receive the payback request?
+   * When did the exchange receive the recoup request?
    * Indirectly determines when the wire transfer is (likely)
    * to happen.
    */
@@ -1280,7 +1275,7 @@ struct TALER_PaybackConfirmationPS
    * How much of the coin's value will the exchange transfer?
    * (Needed in case the coin was partially spent.)
    */
-  struct TALER_AmountNBO payback_amount;
+  struct TALER_AmountNBO recoup_amount;
 
   /**
    * Public key of the coin.
@@ -1288,7 +1283,7 @@ struct TALER_PaybackConfirmationPS
   struct TALER_CoinSpendPublicKeyP coin_pub;
 
   /**
-   * Public key of the reserve that will receive the payback.
+   * Public key of the reserve that will receive the recoup.
    */
   struct TALER_ReservePublicKeyP reserve_pub;
 };
@@ -1296,19 +1291,19 @@ struct TALER_PaybackConfirmationPS
 
 /**
  * Response by which the exchange affirms that it will refund a refreshed coin
- * as part of the emergency /payback protocol.  The payback will go back to the
+ * as part of the emergency /recoup protocol.  The recoup will go back to the
  * old coin's balance.
  */
-struct TALER_PaybackRefreshConfirmationPS
+struct TALER_RecoupRefreshConfirmationPS
 {
 
   /**
-   * Purpose is #TALER_SIGNATURE_EXCHANGE_CONFIRM_PAYBACK_REFRESH
+   * Purpose is #TALER_SIGNATURE_EXCHANGE_CONFIRM_RECOUP_REFRESH
    */
   struct GNUNET_CRYPTO_EccSignaturePurpose purpose;
 
   /**
-   * When did the exchange receive the payback request?
+   * When did the exchange receive the recoup request?
    * Indirectly determines when the wire transfer is (likely)
    * to happen.
    */
@@ -1318,7 +1313,7 @@ struct TALER_PaybackRefreshConfirmationPS
    * How much of the coin's value will the exchange transfer?
    * (Needed in case the coin was partially spent.)
    */
-  struct TALER_AmountNBO payback_amount;
+  struct TALER_AmountNBO recoup_amount;
 
   /**
    * Public key of the refreshed coin.
@@ -1326,7 +1321,7 @@ struct TALER_PaybackRefreshConfirmationPS
   struct TALER_CoinSpendPublicKeyP coin_pub;
 
   /**
-   * Public key of the old coin that will receive the payback.
+   * Public key of the old coin that will receive the recoup.
    */
   struct TALER_CoinSpendPublicKeyP old_coin_pub;
 };
@@ -1360,7 +1355,7 @@ struct TALER_ReserveCloseConfirmationPS
   struct TALER_AmountNBO closing_fee;
 
   /**
-   * Public key of the reserve that received the payback.
+   * Public key of the reserve that received the recoup.
    */
   struct TALER_ReservePublicKeyP reserve_pub;
 
