@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014, 2015 GNUnet e.V.
+  Copyright (C) 2014-2020 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -26,6 +26,22 @@
 #include "taler_amount_lib.h"
 #include "taler_crypto_lib.h"
 
+/**
+ * Stringify operator.
+ *
+ * @param a some expression to stringify. Must NOT be a macro.
+ * @return same expression as a constant string.
+ */
+#define TALER_S(a) #a
+
+/**
+ * Stringify operator.
+ *
+ * @param a some expression to stringify. Can be a macro.
+ * @return macro-expanded expression as a constant string.
+ */
+#define TALER_QUOTE(a) TALER_S (a)
+
 
 /* Define logging functions */
 #define TALER_LOG_DEBUG(...)                                  \
@@ -74,152 +90,6 @@
                        __LINE__, gcry_strerror (rc)); abort (); } while (0)
 
 
-/**
- * Dynamically growing buffer.  Can be used to construct
- * strings and other objects with dynamic size.
- *
- * This structure should, in most cases, be stack-allocated and
- * zero-initialized, like:
- *
- *   struct TALER_Buffer my_buffer = { 0 };
- */
-struct TALER_Buffer
-{
-  /**
-   * Capacity of the buffer.
-   */
-  size_t capacity;
-
-  /**
-   * Current write position.
-   */
-  size_t position;
-
-  /**
-   * Backing memory.
-   */
-  char *mem;
-
-  /**
-   * Log a warning if the buffer is grown over its initially allocated capacity.
-   */
-  int warn_grow;
-};
-
-
-/**
- * Initialize a buffer with the given capacity.
- *
- * When a buffer is allocated with this function, a warning is logged
- * when the buffer exceeds the initial capacity.
- *
- * @param buf the buffer to initialize
- * @param capacity the capacity (in bytes) to allocate for @a buf
- */
-void
-TALER_buffer_prealloc (struct TALER_Buffer *buf, size_t capacity);
-
-
-/**
- * Make sure that at least @a n bytes remaining in the buffer.
- *
- * @param buf buffer to potentially grow
- * @param n number of bytes that should be available to write
- */
-void
-TALER_buffer_ensure_remaining (struct TALER_Buffer *buf, size_t n);
-
-
-/**
- * Write bytes to the buffer.
- *
- * Grows the buffer if necessary.
- *
- * @param buf buffer to write to
- * @param data data to read from
- * @param len number of bytes to copy from @a data to @a buf
- *
- */
-void
-TALER_buffer_write (struct TALER_Buffer *buf, const char *data, size_t len);
-
-
-/**
- * Write a 0-terminated string to a buffer, excluding the 0-terminator.
- *
- * Grows the buffer if necessary.
- *
- * @param buf the buffer to write to
- * @param str the string to write to @a buf
- */
-void
-TALER_buffer_write_str (struct TALER_Buffer *buf, const char *str);
-
-
-/**
- * Write a path component to a buffer, ensuring that
- * there is exactly one slash between the previous contents
- * of the buffer and the new string.
- *
- * @param buf buffer to write to
- * @param str string containing the new path component
- */
-void
-TALER_buffer_write_path (struct TALER_Buffer *buf, const char *str);
-
-
-/**
- * Write a 0-terminated formatted string to a buffer, excluding the
- * 0-terminator.
- *
- * Grows the buffer if necessary.
- *
- * @param buf the buffer to write to
- * @param fmt format string
- * @param ... format arguments
- */
-void
-TALER_buffer_write_fstr (struct TALER_Buffer *buf, const char *fmt, ...);
-
-
-/**
- * Write a 0-terminated formatted string to a buffer, excluding the
- * 0-terminator.
- *
- * Grows the buffer if necessary.
- *
- * @param buf the buffer to write to
- * @param fmt format string
- * @param args format argument list
- */
-void
-TALER_buffer_write_vfstr (struct TALER_Buffer *buf, const char *fmt, va_list
-                          args);
-
-
-/**
- * Clear the buffer and return the string it contained.
- * The caller is responsible to eventually #GNUNET_free
- * the returned string.
- *
- * The returned string is always 0-terminated.
- *
- * @param buf the buffer to reap the string from
- * @returns the buffer contained in the string
- */
-char *
-TALER_buffer_reap_str (struct TALER_Buffer *buf);
-
-
-/**
- * Free the backing memory of the given buffer.
- * Does not free the memory of the buffer control structure,
- * which is typically stack-allocated.
- */
-void
-TALER_buffer_clear (struct TALER_Buffer *buf);
-
-
 /**
  * Initialize Gcrypt library.
  */
@@ -261,10 +131,23 @@ TALER_b2s (const void *buf,
  * @return #GNUNET_OK on success, #GNUNET_SYSERR on error
  */
 int
-TALER_config_get_denom (const struct GNUNET_CONFIGURATION_Handle *cfg,
-                        const char *section,
-                        const char *option,
-                        struct TALER_Amount *denom);
+TALER_config_get_amount (const struct GNUNET_CONFIGURATION_Handle *cfg,
+                         const char *section,
+                         const char *option,
+                         struct TALER_Amount *denom);
+
+
+/**
+ * Load our currency from the @a cfg (in section [taler]
+ * the option "CURRENCY").
+ *
+ * @param cfg configuration to use
+ * @param[out] currency where to write the result
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR on failure
+ */
+int
+TALER_config_get_currency (const struct GNUNET_CONFIGURATION_Handle *cfg,
+                           char **currency);
 
 
 /**
@@ -295,7 +178,7 @@ TALER_project_data_default (void);
  * URL-encode a string according to rfc3986.
  *
  * @param s string to encode
- * @returns the urlencoded string, the caller must free it with GNUNET_free
+ * @returns the urlencoded string, the caller must free it with GNUNET_free()
  */
 char *
 TALER_urlencode (const char *s);
@@ -306,9 +189,9 @@ TALER_urlencode (const char *s);
  * directly or via proxy).
  *
  * @param connection MHD connection
- * @returns GNUNET_YES if the MHD connection is using https,
- *          GNUNET_NO if the MHD connection is using http,
- *          GNUNET_SYSERR if the connection type couldn't be determined
+ * @returns #GNUNET_YES if the MHD connection is using https,
+ *          #GNUNET_NO if the MHD connection is using http,
+ *          #GNUNET_SYSERR if the connection type couldn't be determined
  */
 int
 TALER_mhd_is_https (struct MHD_Connection *connection);
@@ -317,6 +200,10 @@ TALER_mhd_is_https (struct MHD_Connection *connection);
 /**
  * Make an absolute URL with query parameters.
  *
+ * If a 'value' is given as NULL, both the key and the value are skipped. Note
+ * that a NULL value does not terminate the list, only a NULL key signals the
+ * end of the list of arguments.
+ *
  * @param base_url absolute base URL to use
  * @param path path of the url
  * @param ... NULL-terminated key-value pairs (char *) for query parameters,
@@ -332,6 +219,10 @@ TALER_url_join (const char *base_url,
 /**
  * Make an absolute URL for the given parameters.
  *
+ * If a 'value' is given as NULL, both the key and the value are skipped. Note
+ * that a NULL value does not terminate the list, only a NULL key signals the
+ * end of the list of arguments.
+ *
  * @param proto protocol for the URL (typically https)
  * @param host hostname for the URL
  * @param prefix prefix for the URL
@@ -351,6 +242,10 @@ TALER_url_absolute_raw (const char *proto,
 /**
  * Make an absolute URL for the given parameters.
  *
+ * If a 'value' is given as NULL, both the key and the value are skipped. Note
+ * that a NULL value does not terminate the list, only a NULL key signals the
+ * end of the list of arguments.
+ *
  * @param proto protocol for the URL (typically https)
  * @param host hostname for the URL
  * @param prefix prefix for the URL
@@ -370,6 +265,7 @@ TALER_url_absolute_raw_va (const char *proto,
 /**
  * Make an absolute URL for a given MHD connection.
  *
+ * @param connection the connection to get the URL for
  * @param path path of the url
  * @param ... NULL-terminated key-value pairs (char *) for query parameters,
  *        the value will be url-encoded
@@ -381,4 +277,25 @@ TALER_url_absolute_mhd (struct MHD_Connection *connection,
                         ...);
 
 
+/**
+ * Obtain the payment method from a @a payto_uri
+ *
+ * @param payto_uri the URL to parse
+ * @return NULL on error (malformed @a payto_uri)
+ */
+char *
+TALER_payto_get_method (const char *payto_uri);
+
+
+/**
+ * Obtain the account name from a payto URL.
+ *
+ * @param payto an x-taler-bank payto URL
+ * @return only the account name from the @a payto URL, NULL if not an x-taler-bank
+ *   payto URL
+ */
+char *
+TALER_xtalerbank_account_from_payto (const char *payto);
+
+
 #endif
