@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2015, 2016 GNUnet e.V. and Inria
+  (C) 2015, 2016 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -30,7 +30,7 @@
  * @return 0 on success
  */
 static int
-test_amount ()
+test_amount (void)
 {
   json_t *j;
   struct TALER_Amount a1;
@@ -60,6 +60,8 @@ int
 main (int argc,
       const char *const argv[])
 {
+  (void) argc;
+  (void) argv;
   GNUNET_log_setup ("test-json",
                     "WARNING",
                     NULL);
