@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014, 2015, 2018, 2019 GNUnet e.V.
+  Copyright (C) 2014-2020 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -15,7 +15,7 @@
   <http://www.gnu.org/licenses/>
 */
 /**
- * @file exchange-lib/exchange_api_deposit.c
+ * @file lib/exchange_api_deposit.c
  * @brief Implementation of the /deposit request of the exchange's HTTP API
  * @author Sree Harsha Totakura <sreeharsha@totakura.in>
  * @author Christian Grothoff
@@ -38,10 +38,10 @@
  * 1:#AUDITOR_CHANCE is the probability that we report deposits
  * to the auditor.
  *
- * 2==50% of going to auditor. This is way too high, but set
+ * 20==5% of going to auditor. This is possibly still too high, but set
  * deliberately this high for testing
  */
-#define AUDITOR_CHANCE 2
+#define AUDITOR_CHANCE 20
 
 /**
  * @brief A Deposit Handle
@@ -63,7 +63,7 @@ struct TALER_EXCHANGE_DepositHandle
    * Context for #TEH_curl_easy_post(). Keeps the data that must
    * persist for Curl to make the upload.
    */
-  struct TEAH_PostContext ctx;
+  struct TALER_CURL_PostContext ctx;
 
   /**
    * Handle for the request.
@@ -101,9 +101,10 @@ struct TALER_EXCHANGE_DepositHandle
   struct TALER_Amount amount_with_fee;
 
   /**
-   * Total value of the coin being transacted with.
+   * @brief Public information about the coin's denomination key.
+   * Note that the "key" field itself has been zero'ed out.
    */
-  struct TALER_Amount coin_value;
+  struct TALER_EXCHANGE_DenomPublicKey dki;
 
   /**
    * Chance that we will inform the auditor about the deposit
@@ -145,9 +146,13 @@ auditor_cb (void *cls,
               "Will provide deposit confirmation to auditor `%s'\n",
               TALER_B2S (auditor_pub));
   key_state = TALER_EXCHANGE_get_keys (dh->exchange);
-  spk = TALER_EXCHANGE_get_signing_key_details (key_state,
-                                                &dh->exchange_pub);
-  GNUNET_assert (NULL != spk);
+  spk = TALER_EXCHANGE_get_signing_key_info (key_state,
+                                             &dh->exchange_pub);
+  if (NULL == spk)
+  {
+    GNUNET_break_op (0);
+    return NULL;
+  }
   TALER_amount_ntoh (&amount_without_fee,
                      &dh->depconf.amount_without_fee);
   aie = GNUNET_new (struct TEAH_AuditorInteractionEntry);
@@ -180,8 +185,8 @@ auditor_cb (void *cls,
  *
  * @param dh deposit handle
  * @param json json reply with the signature
- * @param exchange_sig[out] set to the exchange's signature
- * @param exchange_pub[out] set to the exchange's public key
+ * @param[out] exchange_sig set to the exchange's signature
+ * @param[out] exchange_pub set to the exchange's public key
  * @return #GNUNET_OK if the signature is valid, #GNUNET_SYSERR if not
  */
 static int
@@ -240,9 +245,9 @@ verify_deposit_signature_ok (struct TALER_EXCHANGE_DepositHandle *dh,
  * @return #GNUNET_OK if the signature(s) is valid, #GNUNET_SYSERR if not
  */
 static int
-verify_deposit_signature_forbidden (const struct
-                                    TALER_EXCHANGE_DepositHandle *dh,
-                                    const json_t *json)
+verify_deposit_signature_forbidden (
+  const struct TALER_EXCHANGE_DepositHandle *dh,
+  const json_t *json)
 {
   json_t *history;
   struct TALER_Amount total;
@@ -250,7 +255,8 @@ verify_deposit_signature_forbidden (const struct
   history = json_object_get (json,
                              "history");
   if (GNUNET_OK !=
-      TALER_EXCHANGE_verify_coin_history (dh->coin_value.currency,
+      TALER_EXCHANGE_verify_coin_history (&dh->dki,
+                                          dh->dki.value.currency,
                                           &dh->depconf.coin_pub,
                                           history,
                                           &total))
@@ -269,7 +275,7 @@ verify_deposit_signature_forbidden (const struct
   }
 
   if (0 >= TALER_amount_cmp (&total,
-                             &dh->coin_value))
+                             &dh->dki.value))
   {
     /* transaction should have still fit */
     GNUNET_break (0);
@@ -299,11 +305,13 @@ handle_deposit_finished (void *cls,
   struct TALER_ExchangeSignatureP *es = NULL;
   struct TALER_ExchangePublicKeyP *ep = NULL;
   const json_t *j = response;
+  enum TALER_ErrorCode ec;
 
   dh->job = NULL;
   switch (response_code)
   {
   case 0:
+    ec = TALER_EC_INVALID_RESPONSE;
     break;
   case MHD_HTTP_OK:
     if (GNUNET_OK !=
@@ -314,52 +322,62 @@ handle_deposit_finished (void *cls,
     {
       GNUNET_break_op (0);
       response_code = 0;
+      ec = TALER_EC_DEPOSIT_INVALID_SIGNATURE_BY_EXCHANGE;
     }
     else
     {
       es = &exchange_sig;
       ep = &exchange_pub;
+      ec = TALER_EC_NONE;
     }
     break;
   case MHD_HTTP_BAD_REQUEST:
     /* This should never happen, either us or the exchange is buggy
        (or API version conflict); just pass JSON reply to the application */
+    ec = TALER_JSON_get_error_code (j);
     break;
   case MHD_HTTP_CONFLICT:
     /* Double spending; check signatures on transaction history */
+    ec = TALER_JSON_get_error_code (j);
     if (GNUNET_OK !=
         verify_deposit_signature_forbidden (dh,
                                             j))
     {
       GNUNET_break_op (0);
       response_code = 0;
+      ec = TALER_EC_DEPOSIT_INVALID_SIGNATURE_BY_EXCHANGE;
     }
     break;
   case MHD_HTTP_FORBIDDEN:
+    ec = TALER_JSON_get_error_code (j);
     /* Nothing really to verify, exchange says one of the signatures is
        invalid; as we checked them, this should never happen, we
        should pass the JSON reply to the application */
     break;
   case MHD_HTTP_NOT_FOUND:
+    ec = TALER_JSON_get_error_code (j);
     /* Nothing really to verify, this should never
        happen, we should pass the JSON reply to the application */
     break;
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
+    ec = TALER_JSON_get_error_code (j);
     /* Server had an internal issue; we should retry, but this API
        leaves this to the application */
     break;
   default:
     /* unexpected response code */
+    ec = TALER_JSON_get_error_code (j);
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Unexpected response code %u\n",
-                (unsigned int) response_code);
+                "Unexpected response code %u/%d\n",
+                (unsigned int) response_code,
+                ec);
     GNUNET_break (0);
     response_code = 0;
     break;
   }
   dh->cb (dh->cb_cls,
           response_code,
-          TALER_JSON_get_error_code (j),
+          ec,
           es,
           ep,
           j);
@@ -511,10 +529,30 @@ TALER_EXCHANGE_deposit (struct TALER_EXCHANGE_Handle *exchange,
   struct GNUNET_HashCode h_wire;
   struct GNUNET_HashCode denom_pub_hash;
   struct TALER_Amount amount_without_fee;
+  char arg_str[sizeof (struct TALER_CoinSpendPublicKeyP) * 2 + 32];
 
+  {
+    char pub_str[sizeof (struct TALER_CoinSpendPublicKeyP) * 2];
+    char *end;
+
+    end = GNUNET_STRINGS_data_to_string (coin_pub,
+                                         sizeof (struct
+                                                 TALER_CoinSpendPublicKeyP),
+                                         pub_str,
+                                         sizeof (pub_str));
+    *end = '\0';
+    GNUNET_snprintf (arg_str,
+                     sizeof (arg_str),
+                     "/coins/%s/deposit",
+                     pub_str);
+  }
   (void) GNUNET_TIME_round_abs (&wire_deadline);
   (void) GNUNET_TIME_round_abs (&refund_deadline);
-  GNUNET_assert (refund_deadline.abs_value_us <= wire_deadline.abs_value_us);
+  if (refund_deadline.abs_value_us > wire_deadline.abs_value_us)
+  {
+    GNUNET_break (0);
+    return NULL;
+  }
   GNUNET_assert (GNUNET_YES ==
                  TEAH_handle_is_ready (exchange));
   /* initialize h_wire */
@@ -528,11 +566,19 @@ TALER_EXCHANGE_deposit (struct TALER_EXCHANGE_Handle *exchange,
   key_state = TALER_EXCHANGE_get_keys (exchange);
   dki = TALER_EXCHANGE_get_denomination_key (key_state,
                                              denom_pub);
-  GNUNET_assert (NULL != dki);
-  GNUNET_assert (GNUNET_SYSERR !=
-                 TALER_amount_subtract (&amount_without_fee,
-                                        amount,
-                                        &dki->fee_deposit));
+  if (NULL == dki)
+  {
+    GNUNET_break (0);
+    return NULL;
+  }
+  if (GNUNET_SYSERR ==
+      TALER_amount_subtract (&amount_without_fee,
+                             amount,
+                             &dki->fee_deposit))
+  {
+    GNUNET_break_op (0);
+    return NULL;
+  }
   GNUNET_CRYPTO_rsa_public_key_hash (denom_pub->rsa_public_key,
                                      &denom_pub_hash);
   if (GNUNET_OK !=
@@ -554,18 +600,17 @@ TALER_EXCHANGE_deposit (struct TALER_EXCHANGE_Handle *exchange,
   }
 
   deposit_obj = json_pack ("{s:o, s:O," /* f/wire */
-                           " s:o, s:o," /* H_wire, h_contract_terms */
-                           " s:o, s:o," /* coin_pub, denom_pub */
+                           " s:o, s:o," /* h_wire, h_contract_terms */
+                           " s:o," /* denom_pub */
                            " s:o, s:o," /* ub_sig, timestamp */
                            " s:o," /* merchant_pub */
                            " s:o, s:o," /* refund_deadline, wire_deadline */
                            " s:o}",     /* coin_sig */
                            "contribution", TALER_JSON_from_amount (amount),
                            "wire", wire_details,
-                           "H_wire", GNUNET_JSON_from_data_auto (&h_wire),
+                           "h_wire", GNUNET_JSON_from_data_auto (&h_wire),
                            "h_contract_terms", GNUNET_JSON_from_data_auto (
                              h_contract_terms),
-                           "coin_pub", GNUNET_JSON_from_data_auto (coin_pub),
                            "denom_pub_hash", GNUNET_JSON_from_data_auto (
                              &denom_pub_hash),
                            "ub_sig", GNUNET_JSON_from_rsa_signature (
@@ -590,7 +635,8 @@ TALER_EXCHANGE_deposit (struct TALER_EXCHANGE_Handle *exchange,
   dh->exchange = exchange;
   dh->cb = cb;
   dh->cb_cls = cb_cls;
-  dh->url = TEAH_path_to_url (exchange, "/deposit");
+  dh->url = TEAH_path_to_url (exchange,
+                              arg_str);
   dh->depconf.purpose.size = htonl (sizeof (struct
                                             TALER_DepositConfirmationPS));
   dh->depconf.purpose.purpose = htonl (
@@ -604,16 +650,20 @@ TALER_EXCHANGE_deposit (struct TALER_EXCHANGE_Handle *exchange,
   dh->depconf.coin_pub = *coin_pub;
   dh->depconf.merchant = *merchant_pub;
   dh->amount_with_fee = *amount;
-  dh->coin_value = dki->value;
+  dh->dki = *dki;
+  dh->dki.key.rsa_public_key = NULL; /* lifetime not warranted, so better
+                                        not copy the pointer */
 
-  eh = TEL_curl_easy_get (dh->url);
-  if (GNUNET_OK !=
-      TALER_curl_easy_post (&dh->ctx,
-                            eh,
-                            deposit_obj))
+  eh = TALER_EXCHANGE_curl_easy_get_ (dh->url);
+  if ( (NULL == eh) ||
+       (GNUNET_OK !=
+        TALER_curl_easy_post (&dh->ctx,
+                              eh,
+                              deposit_obj)) )
   {
     GNUNET_break (0);
-    curl_easy_cleanup (eh);
+    if (NULL != eh)
+      curl_easy_cleanup (eh);
     json_decref (deposit_obj);
     GNUNET_free (dh->url);
     GNUNET_free (dh);
