@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2018 GNUnet e.V.
+  Copyright (C) 2014-2018 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published
@@ -18,7 +18,7 @@
 */
 
 /**
- * @file exchange-lib/exchange_api_handle.c
+ * @file lib/exchange_api_handle.c
  * @brief Implementation of the "handle" component of the exchange's HTTP API
  * @author Sree Harsha Totakura <sreeharsha@totakura.in>
  * @author Christian Grothoff
@@ -36,13 +36,13 @@
 #include "taler_curl_lib.h"
 
 /**
- * Which revision of the Taler protocol is implemented
+ * Which version of the Taler protocol is implemented
  * by this library?  Used to determine compatibility.
  */
-#define TALER_PROTOCOL_CURRENT 6
+#define TALER_PROTOCOL_CURRENT 7
 
 /**
- * How many revisions back are we compatible to?
+ * How many versions are we backwards compatible with?
  */
 #define TALER_PROTOCOL_AGE 0
 
@@ -216,7 +216,7 @@ struct TALER_EXCHANGE_Handle
   enum ExchangeHandleState state;
 
   /**
-   * If GNUNET_YES, use fake now given by the user, in
+   * If #GNUNET_YES, use fake now given by the user, in
    * request of "/keys".
    */
   int with_now;
@@ -278,11 +278,11 @@ TEAH_acc_confirmation_cb (void *cls,
   struct TEAH_AuditorInteractionEntry *aie = cls;
   struct TEAH_AuditorListEntry *ale = aie->ale;
 
+  (void) json;
   if (MHD_HTTP_OK != http_status)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                _ (
-                  "Failed to submit deposit confirmation to auditor `%s' with HTTP status %d (EC: %d). This is acceptable if it does not happen often.\n"),
+                "Failed to submit deposit confirmation to auditor `%s' with HTTP status %d (EC: %d). This is acceptable if it does not happen often.\n",
                 ale->auditor_url,
                 http_status,
                 (int) ec);
@@ -296,7 +296,7 @@ TEAH_acc_confirmation_cb (void *cls,
 
 /**
  * Iterate over all available auditors for @a h, calling
- * @param ah and giving it a chance to start a deposit
+ * @a ac and giving it a chance to start a deposit
  * confirmation interaction.
  *
  * @param h exchange to go over auditors for
@@ -311,8 +311,7 @@ TEAH_get_auditors_for_dc (struct TALER_EXCHANGE_Handle *h,
   if (NULL == h->auditors_head)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                _ (
-                  "No auditor available for exchange `%s'. Not submitting deposit confirmations.\n"),
+                "No auditor available for exchange `%s'. Not submitting deposit confirmations.\n",
                 h->url);
     return;
   }
@@ -529,7 +528,7 @@ EXITIF_exit:
  * Parse a exchange's auditor information encoded in JSON.
  *
  * @param[out] auditor where to return the result
- * @param check_sig should we check signatures
+ * @param check_sigs should we check signatures
  * @param[in] auditor_obj json to parse
  * @param key_data information about denomination keys
  * @return #GNUNET_OK if all is fine, #GNUNET_SYSERR if the signature is
@@ -577,7 +576,6 @@ parse_json_auditor (struct TALER_EXCHANGE_AuditorInformation *auditor,
   auditor->denom_keys = GNUNET_new_array (len,
                                           struct
                                           TALER_EXCHANGE_AuditorDenominationInfo);
-  i = 0;
   off = 0;
   json_array_foreach (keys, i, key) {
     struct TALER_AuditorSignatureP auditor_sig;
@@ -665,8 +663,8 @@ parse_json_auditor (struct TALER_EXCHANGE_AuditorInformation *auditor,
  * @param compat protocol compatibility information
  */
 static void
-auditor_version_cb
-  (void *cls,
+auditor_version_cb (
+  void *cls,
   const struct TALER_AUDITOR_VersionInformation *vi,
   enum TALER_AUDITOR_VersionCompatibility compat)
 {
@@ -676,7 +674,7 @@ auditor_version_cb
   {
     /* In this case, we don't mark the auditor as 'up' */
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                _ ("Auditor `%s' gave unexpected version response.\n"),
+                "Auditor `%s' gave unexpected version response.\n",
                 ale->auditor_url);
     return;
   }
@@ -684,19 +682,18 @@ auditor_version_cb
   if (0 != (TALER_AUDITOR_VC_INCOMPATIBLE & compat))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                _ ("Auditor `%s' runs incompatible protocol version!\n"),
+                "Auditor `%s' runs incompatible protocol version!\n",
                 ale->auditor_url);
     if (0 != (TALER_AUDITOR_VC_OLDER & compat))
     {
       GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                  _ ("Auditor `%s' runs outdated protocol version!\n"),
+                  "Auditor `%s' runs outdated protocol version!\n",
                   ale->auditor_url);
     }
     if (0 != (TALER_AUDITOR_VC_NEWER & compat))
     {
       GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                  _ (
-                    "Auditor `%s' runs more recent incompatible version. We should upgrade!\n"),
+                  "Auditor `%s' runs more recent incompatible version. We should upgrade!\n",
                   ale->auditor_url);
     }
     return;
@@ -756,19 +753,22 @@ update_auditors (struct TALER_EXCHANGE_Handle *exchange)
 
 
 /**
- * Compare two denomination keys.
+ * Compare two denomination keys.  Ignores revocation data.
  *
- * @param denoma first denomination key
- * @param denomb second denomination key
+ * @param denom1 first denomination key
+ * @param denom2 second denomination key
  * @return 0 if the two keys are equal (not necessarily
  *  the same object), 1 otherwise.
  */
 static unsigned int
-TALER_denoms_cmp (struct TALER_EXCHANGE_DenomPublicKey *denom1,
-                  struct TALER_EXCHANGE_DenomPublicKey *denom2)
+denoms_cmp (struct TALER_EXCHANGE_DenomPublicKey *denom1,
+            struct TALER_EXCHANGE_DenomPublicKey *denom2)
 {
   struct GNUNET_CRYPTO_RsaPublicKey *tmp1;
   struct GNUNET_CRYPTO_RsaPublicKey *tmp2;
+  int r1;
+  int r2;
+  int ret;
 
   /* First check if pub is the same.  */
   if (0 != GNUNET_CRYPTO_rsa_public_key_cmp
@@ -778,24 +778,19 @@ TALER_denoms_cmp (struct TALER_EXCHANGE_DenomPublicKey *denom1,
 
   tmp1 = denom1->key.rsa_public_key;
   tmp2 = denom2->key.rsa_public_key;
+  r1 = denom1->revoked;
+  r2 = denom2->revoked;
 
   denom1->key.rsa_public_key = NULL;
   denom2->key.rsa_public_key = NULL;
-
   /* Then procede with the rest of the object.  */
-  if (0 != GNUNET_memcmp (denom1,
-                          denom2))
-  {
-    denom1->key.rsa_public_key = tmp1;
-    denom2->key.rsa_public_key = tmp2;
-
-    return 1;
-  }
-
+  ret = GNUNET_memcmp (denom1,
+                       denom2);
+  denom1->revoked = r1;
+  denom2->revoked = r2;
   denom1->key.rsa_public_key = tmp1;
   denom2->key.rsa_public_key = tmp2;
-
-  return 0;
+  return ret;
 }
 
 
@@ -806,7 +801,7 @@ TALER_denoms_cmp (struct TALER_EXCHANGE_DenomPublicKey *denom1,
  * @param[in] resp_obj JSON object to parse
  * @param check_sig #GNUNET_YES if we should check the signature
  * @param[out] key_data where to store the results we decoded
- * @param[out] where to store version compatibility data
+ * @param[out] vc where to store version compatibility data
  * @return #GNUNET_OK on success, #GNUNET_SYSERR on error
  * (malformed JSON)
  */
@@ -910,7 +905,6 @@ decode_keys_json (const json_t *resp_obj,
     key_data->sign_keys
       = GNUNET_new_array (key_data->num_sign_keys,
                           struct TALER_EXCHANGE_SigningPublicKey);
-    index = 0;
     json_array_foreach (sign_keys_array, index, sign_key_obj) {
       EXITIF (GNUNET_SYSERR ==
               parse_json_signkey (&key_data->sign_keys[index],
@@ -932,11 +926,13 @@ decode_keys_json (const json_t *resp_obj,
                                         "denoms")));
     EXITIF (JSON_ARRAY != json_typeof (denom_keys_array));
 
-    index = 0;
     json_array_foreach (denom_keys_array, index, denom_key_obj) {
       struct TALER_EXCHANGE_DenomPublicKey dk;
       int found = GNUNET_NO;
 
+      memset (&dk,
+              0,
+              sizeof (dk));
       EXITIF (GNUNET_SYSERR ==
               parse_json_denomkey (&dk,
                                    check_sig,
@@ -948,8 +944,8 @@ decode_keys_json (const json_t *resp_obj,
            j<key_data->num_denom_keys;
            j++)
       {
-        if (0 == TALER_denoms_cmp (&dk,
-                                   &key_data->denom_keys[j]))
+        if (0 == denoms_cmp (&dk,
+                             &key_data->denom_keys[j]))
         {
           found = GNUNET_YES;
           break;
@@ -958,10 +954,7 @@ decode_keys_json (const json_t *resp_obj,
       if (GNUNET_YES == found)
       {
         /* 0:0:0 did not support /keys cherry picking */
-
-        /* Okay, but why breaking here? It could be that the
-         * user redownloaded all the keys in a forced way. */
-        GNUNET_break_op (0 == current);
+        TALER_LOG_DEBUG ("Skipping denomination key: already know it\n");
         GNUNET_CRYPTO_rsa_public_key_free (dk.key.rsa_public_key);
         continue;
       }
@@ -972,13 +965,14 @@ decode_keys_json (const json_t *resp_obj,
       key_data->denom_keys[key_data->num_denom_keys++] = dk;
 
       /* Update "last_denom_issue_date" */
-      TALER_LOG_DEBUG ("Crawling DK 'valid_from': %s\n",
+      TALER_LOG_DEBUG ("Adding denomination key that is valid_from %s\n",
                        GNUNET_STRINGS_absolute_time_to_string (dk.valid_from));
       key_data->last_denom_issue_date
         = GNUNET_TIME_absolute_max (key_data->last_denom_issue_date,
                                     dk.valid_from);
     };
   }
+
   /* parse the auditor information */
   {
     json_t *auditors_array;
@@ -991,7 +985,6 @@ decode_keys_json (const json_t *resp_obj,
     EXITIF (JSON_ARRAY != json_typeof (auditors_array));
 
     /* Merge with the existing auditor information we have (/keys cherry picking) */
-    index = 0;
     json_array_foreach (auditors_array, index, auditor_info) {
       struct TALER_EXCHANGE_AuditorInformation ai;
       int found = GNUNET_NO;
@@ -1014,6 +1007,11 @@ decode_keys_json (const json_t *resp_obj,
           found = GNUNET_YES;
           /* Merge denomination key signatures of downloaded /keys into existing
              auditor information 'aix'. */
+          TALER_LOG_DEBUG (
+            "Merging %u new audited keys with %u known audited keys\n",
+            aix->num_denom_keys,
+            ai.num_denom_keys);
+
           GNUNET_array_grow (aix->denom_keys,
                              aix->num_denom_keys,
                              aix->num_denom_keys + ai.num_denom_keys);
@@ -1041,6 +1039,45 @@ decode_keys_json (const json_t *resp_obj,
     };
   }
 
+  /* parse the revocation/recoup information */
+  {
+    json_t *recoup_array;
+    json_t *recoup_info;
+    unsigned int index;
+
+    if (NULL != (recoup_array =
+                   json_object_get (resp_obj,
+                                    "recoup")))
+    {
+      EXITIF (JSON_ARRAY != json_typeof (recoup_array));
+
+      json_array_foreach (recoup_array, index, recoup_info) {
+        struct GNUNET_HashCode h_denom_pub;
+        struct GNUNET_JSON_Specification spec[] = {
+          GNUNET_JSON_spec_fixed_auto ("h_denom_pub",
+                                       &h_denom_pub),
+          GNUNET_JSON_spec_end ()
+        };
+
+        EXITIF (GNUNET_OK !=
+                GNUNET_JSON_parse (recoup_info,
+                                   spec,
+                                   NULL, NULL));
+        for (unsigned int j = 0;
+             j<key_data->num_denom_keys;
+             j++)
+        {
+          if (0 == GNUNET_memcmp (&h_denom_pub,
+                                  &key_data->denom_keys[j].h_key))
+          {
+            key_data->denom_keys[j].revoked = GNUNET_YES;
+            break;
+          }
+        }
+      };
+    }
+  }
+
   if (check_sig)
   {
     struct TALER_ExchangeKeySetPS ks;
@@ -1113,16 +1150,6 @@ static void
 request_keys (void *cls);
 
 
-/**
- * Put the handle back to the init state.  Might
- * be useful to force-download all /keys.
- *
- * @param h exchange handle.
- */
-void
-TEAH_handle_reset (struct TALER_EXCHANGE_Handle *h);
-
-
 /**
  * Set the fake now to be used when requesting "/keys".
  *
@@ -1171,7 +1198,7 @@ TALER_EXCHANGE_set_last_denom (struct TALER_EXCHANGE_Handle *exchange,
  *
  * @param exchange exchange to check keys for
  * @param force_download #GNUNET_YES to force download even if /keys is still valid
- * @param pull_all_keys if GNUNET_YES, then the exchange state is reset to 'init',
+ * @param pull_all_keys if #GNUNET_YES, then the exchange state is reset to #MHS_INIT,
  *        and all denoms will be redownloaded.
  * @return until when the response is current, 0 if we are re-downloading
  */
@@ -1185,8 +1212,10 @@ TALER_EXCHANGE_check_keys_current (struct TALER_EXCHANGE_Handle *exchange,
 
   if (GNUNET_YES == pull_all_keys)
   {
-    GNUNET_assert (GNUNET_YES == force_download);
-    TEAH_handle_reset (exchange);
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Forcing re-download of all exchange keys\n");
+    GNUNET_break (GNUNET_YES == force_download);
+    exchange->state = MHS_INIT;
   }
   if ( (GNUNET_NO == force_download) &&
        (0 < GNUNET_TIME_absolute_get_remaining (
@@ -1336,6 +1365,8 @@ keys_completed_cb (void *cls,
     exchange->kr = NULL;
     free_keys_request (kr);
     exchange->state = MHS_FAILED;
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Exchange keys download failed\n");
     if (NULL != exchange->key_data_raw)
     {
       json_decref (exchange->key_data_raw);
@@ -1353,6 +1384,8 @@ keys_completed_cb (void *cls,
   exchange->key_data_expiration = kr->expire;
   free_keys_request (kr);
   exchange->state = MHS_CERT;
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Successfully downloaded exchange's keys\n");
   update_auditors (exchange);
   /* notify application about the key information */
   exchange->cert_cb (exchange->cert_cb_cls,
@@ -1378,19 +1411,6 @@ TEAH_handle_to_context (struct TALER_EXCHANGE_Handle *h)
 }
 
 
-/**
- * Put the handle back to the init state.  Might
- * be useful to force-download all /keys.
- *
- * @param h exchange handle.
- */
-void
-TEAH_handle_reset (struct TALER_EXCHANGE_Handle *h)
-{
-  h->state = MHS_INIT;
-}
-
-
 /**
  * Check if the handle is ready to process requests.
  *
@@ -1415,33 +1435,14 @@ char *
 TEAH_path_to_url (struct TALER_EXCHANGE_Handle *h,
                   const char *path)
 {
-  return TEAH_path_to_url2 (h->url,
-                            path);
-}
+  char *ret;
 
-
-/**
- * Obtain the URL to use for an API request.
- *
- * @param base_url base URL of the exchange (i.e. "http://exchange/")
- * @param path Taler API path (i.e. "/reserve/withdraw")
- * @return the full URL to use with cURL
- */
-char *
-TEAH_path_to_url2 (const char *base_url,
-                   const char *path)
-{
-  char *url;
-
-  if ( ('/' == path[0]) &&
-       (0 < strlen (base_url)) &&
-       ('/' == base_url[strlen (base_url) - 1]) )
-    path++; /* avoid generating URL with "//" from concat */
-  GNUNET_asprintf (&url,
-                   "%s%s",
-                   base_url,
-                   path);
-  return url;
+  GNUNET_assert ('/' == path[0]);
+  ret = TALER_url_join (h->url,
+                        path + 1,
+                        NULL);
+  GNUNET_assert (NULL != ret);
+  return ret;
 }
 
 
@@ -1540,7 +1541,7 @@ header_cb (char *buffer,
  * tolerated (i.e. by re-downloading instead).
  *
  * @param exchange which exchange's key and wire data should be deserialized
- * @return data the data to deserialize
+ * @param data the data to deserialize
  */
 static void
 deserialize_data (struct TALER_EXCHANGE_Handle *exchange,
@@ -1556,7 +1557,7 @@ deserialize_data (struct TALER_EXCHANGE_Handle *exchange,
                              &version),
     GNUNET_JSON_spec_json ("keys",
                            &keys),
-    GNUNET_JSON_spec_string ("url",
+    GNUNET_JSON_spec_string ("exchange_url",
                              &url),
     GNUNET_JSON_spec_absolute_time ("expire",
                                     &expire),
@@ -1605,6 +1606,8 @@ deserialize_data (struct TALER_EXCHANGE_Handle *exchange,
   exchange->key_data = key_data;
   exchange->key_data_expiration = expire;
   exchange->state = MHS_CERT;
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Successfully loaded exchange's keys via deserialization\n");
   update_auditors (exchange);
   /* notify application about the key information */
   exchange->cert_cb (exchange->cert_cb_cls,
@@ -1626,8 +1629,7 @@ deserialize_data (struct TALER_EXCHANGE_Handle *exchange,
  *         otherwise JSON object owned by the caller
  */
 json_t *
-TALER_EXCHANGE_serialize_data
-  (struct TALER_EXCHANGE_Handle *exchange)
+TALER_EXCHANGE_serialize_data (struct TALER_EXCHANGE_Handle *exchange)
 {
   const struct TALER_EXCHANGE_Keys *kd = &exchange->key_data;
   struct GNUNET_TIME_Absolute now;
@@ -1851,7 +1853,7 @@ TALER_EXCHANGE_serialize_data
                     (json_int_t) TALER_SERIALIZATION_FORMAT_VERSION,
                     "expire",
                     GNUNET_JSON_from_time_abs (exchange->key_data_expiration),
-                    "url",
+                    "exchange_url",
                     exchange->url,
                     "keys",
                     keys);
@@ -1877,8 +1879,8 @@ TALER_EXCHANGE_serialize_data
  * @return the exchange handle; NULL upon error
  */
 struct TALER_EXCHANGE_Handle *
-TALER_EXCHANGE_connect
-  (struct GNUNET_CURL_Context *ctx,
+TALER_EXCHANGE_connect (
+  struct GNUNET_CURL_Context *ctx,
   const char *url,
   TALER_EXCHANGE_CertificationCallback cert_cb,
   void *cert_cb_cls,
@@ -1975,15 +1977,23 @@ request_keys (void *cls)
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Requesting keys with URL `%s'.\n",
               kr->url);
-  eh = TEL_curl_easy_get (kr->url);
-  GNUNET_assert (CURLE_OK ==
-                 curl_easy_setopt (eh,
-                                   CURLOPT_VERBOSE,
-                                   0));
-  GNUNET_assert (CURLE_OK ==
-                 curl_easy_setopt (eh,
-                                   CURLOPT_TIMEOUT,
-                                   (long) 300));
+  eh = TALER_EXCHANGE_curl_easy_get_ (kr->url);
+  if (NULL == eh)
+  {
+    exchange->retry_delay = EXCHANGE_LIB_BACKOFF (exchange->retry_delay);
+    exchange->retry_task = GNUNET_SCHEDULER_add_delayed (exchange->retry_delay,
+                                                         &request_keys,
+                                                         exchange);
+    return;
+  }
+  GNUNET_break (CURLE_OK ==
+                curl_easy_setopt (eh,
+                                  CURLOPT_VERBOSE,
+                                  0));
+  GNUNET_break (CURLE_OK ==
+                curl_easy_setopt (eh,
+                                  CURLOPT_TIMEOUT,
+                                  (long) 300));
   GNUNET_assert (CURLE_OK ==
                  curl_easy_setopt (eh,
                                    CURLOPT_HEADERFUNCTION,
@@ -2019,8 +2029,7 @@ TALER_EXCHANGE_disconnect (struct TALER_EXCHANGE_Handle *exchange)
     {
       GNUNET_assert (aie->ale == ale);
       GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                  _ (
-                    "Not sending deposit confirmation to auditor `%s' due to exchange disconnect\n"),
+                  "Not sending deposit confirmation to auditor `%s' due to exchange disconnect\n",
                   ale->auditor_url);
       TALER_AUDITOR_deposit_confirmation_cancel (aie->dch);
       GNUNET_CONTAINER_DLL_remove (ale->ai_head,
@@ -2059,30 +2068,6 @@ TALER_EXCHANGE_disconnect (struct TALER_EXCHANGE_Handle *exchange)
 }
 
 
-/**
- * Lookup the given @a pub in @a keys.
- *
- * @param keys the exchange's key set
- * @param pub claimed current online signing key for the exchange
- * @return NULL if @a pub was not found
- */
-const struct TALER_EXCHANGE_SigningPublicKey *
-TALER_EXCHANGE_get_signing_key_details (const struct TALER_EXCHANGE_Keys *keys,
-                                        const struct
-                                        TALER_ExchangePublicKeyP *pub)
-{
-  for (unsigned int i = 0; i<keys->num_sign_keys; i++)
-  {
-    struct TALER_EXCHANGE_SigningPublicKey *spk = &keys->sign_keys[i];
-
-    if (0 == GNUNET_memcmp (pub,
-                            &spk->key))
-      return spk;
-  }
-  return NULL;
-}
-
-
 /**
  * Test if the given @a pub is a the current signing key from the exchange
  * according to @a keys.
@@ -2133,9 +2118,9 @@ TALER_EXCHANGE_get_base_url (const struct TALER_EXCHANGE_Handle *exchange)
  * not found
  */
 const struct TALER_EXCHANGE_DenomPublicKey *
-TALER_EXCHANGE_get_denomination_key (const struct TALER_EXCHANGE_Keys *keys,
-                                     const struct
-                                     TALER_DenominationPublicKey *pk)
+TALER_EXCHANGE_get_denomination_key (
+  const struct TALER_EXCHANGE_Keys *keys,
+  const struct TALER_DenominationPublicKey *pk)
 {
   for (unsigned int i = 0; i<keys->num_denom_keys; i++)
     if (0 == GNUNET_CRYPTO_rsa_public_key_cmp (pk->rsa_public_key,
@@ -2146,6 +2131,42 @@ TALER_EXCHANGE_get_denomination_key (const struct TALER_EXCHANGE_Keys *keys,
 }
 
 
+/**
+ * Create a copy of a denomination public key.
+ *
+ * @param key key to copy
+ * @returns a copy, must be freed with #TALER_EXCHANGE_destroy_denomination_key
+ */
+struct TALER_EXCHANGE_DenomPublicKey *
+TALER_EXCHANGE_copy_denomination_key (
+  const struct TALER_EXCHANGE_DenomPublicKey *key)
+{
+  struct TALER_EXCHANGE_DenomPublicKey *copy;
+
+  copy = GNUNET_new (struct TALER_EXCHANGE_DenomPublicKey);
+  *copy = *key;
+  copy->key.rsa_public_key = GNUNET_CRYPTO_rsa_public_key_dup (
+    key->key.rsa_public_key);
+
+  return copy;
+}
+
+
+/**
+ * Destroy a denomination public key.
+ * Should only be called with keys created by #TALER_EXCHANGE_copy_denomination_key.
+ *
+ * @param key key to destroy.
+ */
+void
+TALER_EXCHANGE_destroy_denomination_key (
+  struct TALER_EXCHANGE_DenomPublicKey *key)
+{
+  GNUNET_CRYPTO_rsa_public_key_free (key->key.rsa_public_key);;
+  GNUNET_free (key);
+}
+
+
 /**
  * Obtain the denomination key details from the exchange.
  *
@@ -2154,9 +2175,9 @@ TALER_EXCHANGE_get_denomination_key (const struct TALER_EXCHANGE_Keys *keys,
  * @return details about the given denomination key
  */
 const struct TALER_EXCHANGE_DenomPublicKey *
-TALER_EXCHANGE_get_denomination_key_by_hash (const struct
-                                             TALER_EXCHANGE_Keys *keys,
-                                             const struct GNUNET_HashCode *hc)
+TALER_EXCHANGE_get_denomination_key_by_hash (
+  const struct TALER_EXCHANGE_Keys *keys,
+  const struct GNUNET_HashCode *hc)
 {
   for (unsigned int i = 0; i<keys->num_denom_keys; i++)
     if (0 == GNUNET_memcmp (hc,
