@@ -38,7 +38,7 @@
  * realizes an MHD POST processor that will (incrementally) process
  * JSON data uploaded to the HTTP server.  It will store the required
  * state in the @a con_cls, which must be cleaned up using
- * #TALER_MHD_post_cleanup_callback().
+ * #TALER_MHD_parse_post_cleanup_callback().
  *
  * @param connection the MHD connection
  * @param con_cls the closure (points to a `struct Buffer *`)
@@ -74,6 +74,7 @@ TALER_MHD_parse_post_json (struct MHD_Connection *connection,
   switch (pr)
   {
   case GNUNET_JSON_PR_OUT_OF_MEMORY:
+    GNUNET_break (NULL == *json);
     return (MHD_NO ==
             TALER_MHD_reply_with_error
               (connection,
@@ -82,12 +83,15 @@ TALER_MHD_parse_post_json (struct MHD_Connection *connection,
               "out of memory")) ? GNUNET_SYSERR : GNUNET_NO;
 
   case GNUNET_JSON_PR_CONTINUE:
+    GNUNET_break (NULL == *json);
     return GNUNET_YES;
   case GNUNET_JSON_PR_REQUEST_TOO_LARGE:
+    GNUNET_break (NULL == *json);
     return (MHD_NO ==
             TALER_MHD_reply_request_too_large
               (connection)) ? GNUNET_SYSERR : GNUNET_NO;
   case GNUNET_JSON_PR_JSON_INVALID:
+    GNUNET_break (NULL == *json);
     return (MHD_YES ==
             TALER_MHD_reply_with_error (connection,
                                         MHD_HTTP_BAD_REQUEST,
@@ -109,12 +113,11 @@ TALER_MHD_parse_post_json (struct MHD_Connection *connection,
  * to clean up our state.
  *
  * @param con_cls value as it was left by
- *        #TALER_MHD_post_json(), to be cleaned up
+ *        #TALER_MHD_parse_post_json(), to be cleaned up
  */
 void
 TALER_MHD_parse_post_cleanup_callback (void *con_cls)
 {
-  // FIXME: this should probably NOT be done with a 'void *' like this!
   GNUNET_JSON_post_parser_cleanup (con_cls);
 }
 
