@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2015, 2016 GNUnet e.V.
+  (C) 2015, 2016 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -88,7 +88,6 @@ run_queries (struct GNUNET_PQ_Context *conn)
   json = json_object ();
   json_object_set_new (json, "foo", json_integer (42));
   GNUNET_assert (NULL != json);
-  /* FIXME: test GNUNET_PQ_result_spec_variable_size */
   {
     struct GNUNET_PQ_QueryParam params_insert[] = {
       TALER_PQ_query_param_amount (&hamount),
@@ -176,14 +175,16 @@ main (int argc,
                             ")"),
     GNUNET_PQ_EXECUTE_STATEMENT_END
   };
-
   struct GNUNET_PQ_Context *conn;
   int ret;
 
+  (void) argc;
+  (void) argv;
   GNUNET_log_setup ("test-pq",
                     "WARNING",
                     NULL);
   conn = GNUNET_PQ_connect ("postgres:///talercheck",
+                            NULL,
                             es,
                             NULL);
   if (NULL == conn)
