@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014 GNUnet e.V.
+  Copyright (C) 2014 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -22,12 +22,7 @@
  * @author Christian Grothoff
  */
 #include "platform.h"
-#if HAVE_GNUNET_GNUNET_UTIL_LIB_H
 #include "taler_util.h"
-#elif HAVE_GNUNET_GNUNET_UTIL_TALER_WALLET_LIB_H
-#include "taler_util_wallet.h"
-#endif
-#include <gcrypt.h>
 
 /**
  * Maximum legal 'value' for an amount, based on IEEE double (for JavaScript compatibility).
@@ -38,7 +33,7 @@
 /**
  * Set @a a to "invalid".
  *
- * @param a amount to set to invalid
+ * @param[out] a amount to set to invalid
  */
 static void
 invalidate (struct TALER_Amount *a)
@@ -50,24 +45,22 @@ invalidate (struct TALER_Amount *a)
 
 
 /**
- * Parse money amount description, in the format "A:B.C".
+ * Parse monetary amount, in the format "T:V.F".
  *
- * @param str amount description
- * @param denom amount to write the result to
- * @return #GNUNET_OK if the string is a valid amount specification,
+ * @param str amount string
+ * @param[out] amount amount to write the result to
+ * @return #GNUNET_OK if the string is a valid monetary amount specification,
  *         #GNUNET_SYSERR if it is invalid.
  */
 int
 TALER_string_to_amount (const char *str,
-                        struct TALER_Amount *denom)
+                        struct TALER_Amount *amount)
 {
-  size_t i;
   int n;
   uint32_t b;
   const char *colon;
   const char *value;
 
-  invalidate (denom);
   /* skip leading whitespace */
   while (isspace ( (unsigned char) str[0]))
     str++;
@@ -75,8 +68,10 @@ TALER_string_to_amount (const char *str,
   {
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                 "Null before currency\n");
+    invalidate (amount);
     return GNUNET_SYSERR;
   }
+
   /* parse currency */
   colon = strchr (str, (int) ':');
   if ( (NULL == colon) ||
@@ -85,105 +80,118 @@ TALER_string_to_amount (const char *str,
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                 "Invalid currency specified before colon: `%s'\n",
                 str);
-    goto fail;
+    invalidate (amount);
+    return GNUNET_SYSERR;
   }
-  memcpy (denom->currency,
+
+  GNUNET_assert (TALER_CURRENCY_LEN > (colon - str));
+  memcpy (amount->currency,
           str,
           colon - str);
+  /* 0-terminate *and* normalize buffer by setting everything to '\0' */
+  memset (&amount->currency [colon - str],
+          0,
+          TALER_CURRENCY_LEN - (colon - str));
+
   /* skip colon */
   value = colon + 1;
   if ('\0' == value[0])
   {
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "Null before value\n");
-    goto fail;
+                "Actual value missing in amount `%s'\n",
+                str);
+    invalidate (amount);
+    return GNUNET_SYSERR;
   }
 
+  amount->value = 0;
+  amount->fraction = 0;
+
   /* parse value */
-  i = 0;
-  while ('.' != value[i])
+  while ('.' != *value)
   {
-    if ('\0' == value[i])
+    if ('\0' == *value)
     {
+      /* we are done */
       return GNUNET_OK;
     }
-    if ( (value[i] < '0') || (value[i] > '9') )
+    if ( (*value < '0') ||
+         (*value > '9') )
     {
       GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                  "Invalid character `%c'\n",
-                  value[i]);
-      goto fail;
+                  "Invalid character `%c' in amount `%s'\n",
+                  (int) *value,
+                  str);
+      invalidate (amount);
+      return GNUNET_SYSERR;
     }
-    n = value[i] - '0';
-    if (denom->value * 10 + n < denom->value)
+    n = *value - '0';
+    if ( (amount->value * 10 + n < amount->value) ||
+         (amount->value > MAX_AMOUNT_VALUE) )
     {
       GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                  "Value too large\n");
-      goto fail;
+                  "Value specified in amount `%s' is too large\n",
+                  str);
+      invalidate (amount);
+      return GNUNET_SYSERR;
     }
-    denom->value = (denom->value * 10) + n;
-    i++;
+    amount->value = (amount->value * 10) + n;
+    value++;
   }
 
   /* skip the dot */
-  i++;
+  value++;
 
   /* parse fraction */
-  if ('\0' == value[i])
+  if ('\0' == *value)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "Null after dot\n");
-    goto fail;
+                "Amount `%s' ends abruptly after `.'\n",
+                str);
+    invalidate (amount);
+    return GNUNET_SYSERR;
   }
   b = TALER_AMOUNT_FRAC_BASE / 10;
-  while ('\0' != value[i])
+  while ('\0' != *value)
   {
     if (0 == b)
     {
       GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                  "Fractional value too small (only %u digits supported)\n",
-                  (unsigned int) TALER_AMOUNT_FRAC_LEN);
-      goto fail;
+                  "Fractional value too small (only %u digits supported) in amount `%s'\n",
+                  (unsigned int) TALER_AMOUNT_FRAC_LEN,
+                  str);
+      invalidate (amount);
+      return GNUNET_SYSERR;
     }
-    if ( (value[i] < '0') || (value[i] > '9') )
+    if ( (*value < '0') ||
+         (*value > '9') )
     {
       GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                   "Error after dot\n");
-      goto fail;
+      invalidate (amount);
+      return GNUNET_SYSERR;
     }
-    n = value[i] - '0';
-    denom->fraction += n * b;
+    n = *value - '0';
+    amount->fraction += n * b;
     b /= 10;
-    i++;
-  }
-  if (denom->value > MAX_AMOUNT_VALUE)
-  {
-    /* too large to be legal */
-    invalidate (denom);
-    return GNUNET_SYSERR;
+    value++;
   }
   return GNUNET_OK;
-
-fail:
-  /* set currency to 'invalid' to prevent accidental use */
-  memset (denom->currency,
-          0,
-          TALER_CURRENCY_LEN);
-  return GNUNET_SYSERR;
 }
 
 
 /**
- * Parse denomination description, in the format "T:V.F".
+ * Parse monetary amount, in the format "T:V.F".
+ * The result is stored in network byte order (NBO).
  *
- * @param str denomination description
- * @param denom denomination to write the result to, in NBO
- * @return #GNUNET_OK if the string is a valid denomination specification,
+ * @param str amount string
+ * @param[out] amount_nbo amount to write the result to
+ * @return #GNUNET_OK if the string is a valid amount specification,
  *         #GNUNET_SYSERR if it is invalid.
  */
 int
 TALER_string_to_amount_nbo (const char *str,
-                            struct TALER_AmountNBO *denom)
+                            struct TALER_AmountNBO *amount_nbo)
 {
   struct TALER_Amount amount;
 
@@ -191,7 +199,7 @@ TALER_string_to_amount_nbo (const char *str,
       TALER_string_to_amount (str,
                               &amount))
     return GNUNET_SYSERR;
-  TALER_amount_hton (denom,
+  TALER_amount_hton (amount_nbo,
                      &amount);
   return GNUNET_OK;
 }
@@ -201,7 +209,7 @@ TALER_string_to_amount_nbo (const char *str,
  * Convert amount from host to network representation.
  *
  * @param res where to store amount in network representation
- * @param d amount in host representation
+ * @param[out] d amount in host representation
  */
 void
 TALER_amount_hton (struct TALER_AmountNBO *res,
@@ -220,7 +228,7 @@ TALER_amount_hton (struct TALER_AmountNBO *res,
 /**
  * Convert amount from network to host representation.
  *
- * @param res where to store amount in host representation
+ * @param[out] res where to store amount in host representation
  * @param dn amount in network representation
  */
 void
@@ -241,23 +249,23 @@ TALER_amount_ntoh (struct TALER_Amount *res,
  * Get the value of "zero" in a particular currency.
  *
  * @param cur currency description
- * @param denom denomination to write the result to
+ * @param[out] amount amount to write the result to
  * @return #GNUNET_OK if @a cur is a valid currency specification,
  *         #GNUNET_SYSERR if it is invalid.
  */
 int
 TALER_amount_get_zero (const char *cur,
-                       struct TALER_Amount *denom)
+                       struct TALER_Amount *amount)
 {
   size_t slen;
 
   slen = strlen (cur);
   if (slen >= TALER_CURRENCY_LEN)
     return GNUNET_SYSERR;
-  memset (denom,
+  memset (amount,
           0,
           sizeof (struct TALER_Amount));
-  memcpy (denom->currency,
+  memcpy (amount->currency,
           cur,
           slen);
   return GNUNET_OK;
@@ -358,7 +366,8 @@ TALER_amount_cmp (const struct TALER_Amount *a1,
   struct TALER_Amount n2;
 
   GNUNET_assert (GNUNET_YES ==
-                 TALER_amount_cmp_currency (a1, a2));
+                 TALER_amount_cmp_currency (a1,
+                                            a2));
   n1 = *a1;
   n2 = *a2;
   GNUNET_assert (GNUNET_SYSERR !=
@@ -382,7 +391,7 @@ TALER_amount_cmp (const struct TALER_Amount *a1,
 /**
  * Perform saturating subtraction of amounts.
  *
- * @param diff where to store (@a a1 - @a a2), or invalid if @a a2 > @a a1
+ * @param[out] diff where to store (@a a1 - @a a2), or invalid if @a a2 > @a a1
  * @param a1 amount to subtract from
  * @param a2 amount to subtract
  * @return #GNUNET_OK if the subtraction worked,
@@ -399,11 +408,14 @@ TALER_amount_subtract (struct TALER_Amount *diff,
   struct TALER_Amount n2;
 
   if (GNUNET_YES !=
-      TALER_amount_cmp_currency (a1, a2))
+      TALER_amount_cmp_currency (a1,
+                                 a2))
   {
     invalidate (diff);
     return GNUNET_SYSERR;
   }
+  /* make local copies to avoid aliasing problems between
+     diff and a1/a2 */
   n1 = *a1;
   n2 = *a2;
   if ( (GNUNET_SYSERR == TALER_amount_normalize (&n1)) ||
@@ -445,7 +457,7 @@ TALER_amount_subtract (struct TALER_Amount *diff,
 /**
  * Perform addition of amounts.
  *
- * @param sum where to store @a a1 + @a a2, set to "invalid" on overflow
+ * @param[out] sum where to store @a a1 + @a a2, set to "invalid" on overflow
  * @param a1 first amount to add
  * @param a2 second amount to add
  * @return #GNUNET_OK if the addition worked,
@@ -466,6 +478,8 @@ TALER_amount_add (struct TALER_Amount *sum,
     invalidate (sum);
     return GNUNET_SYSERR;
   }
+  /* make local copies to avoid aliasing problems between
+     diff and a1/a2 */
   n1 = *a1;
   n2 = *a2;
   if ( (GNUNET_SYSERR == TALER_amount_normalize (&n1)) ||
@@ -507,7 +521,7 @@ TALER_amount_add (struct TALER_Amount *sum,
 /**
  * Normalize the given amount.
  *
- * @param amount amount to normalize
+ * @param[in,out] amount amount to normalize
  * @return #GNUNET_OK if normalization worked
  *         #GNUNET_NO if value was already normalized
  *         #GNUNET_SYSERR if value was invalid or could not be normalized
@@ -539,11 +553,10 @@ TALER_amount_normalize (struct TALER_Amount *amount)
 
 
 /**
- * Convert the fraction of @a amount to a string
- * in decimals.
+ * Convert the fraction of @a amount to a string in decimals.
  *
  * @param amount value to convert
- * @param tail[out] where to write the reesult
+ * @param[out] tail where to write the reesult
  */
 static void
 amount_to_tail (const struct TALER_Amount *amount,
@@ -611,6 +624,7 @@ TALER_amount_to_string (const struct TALER_Amount *amount)
 const char *
 TALER_amount2s (const struct TALER_Amount *amount)
 {
+  /* 12 is sufficient for a uint32_t value in decimal; 3 is for ":.\0" */
   static char result[TALER_AMOUNT_FRAC_LEN + TALER_CURRENCY_LEN + 3 + 12];
   struct TALER_Amount norm;
 
@@ -645,10 +659,10 @@ TALER_amount2s (const struct TALER_Amount *amount)
 
 
 /**
- * Divide an amount by a float.  Note that this function
+ * Divide an amount by a @a divisor.  Note that this function
  * may introduce a rounding error!
  *
- * @param result where to store @a dividend / @a divisor
+ * @param[out] result where to store @a dividend / @a divisor
  * @param dividend amount to divide
  * @param divisor by what to divide, must be positive
  */
@@ -659,16 +673,78 @@ TALER_amount_divide (struct TALER_Amount *result,
 {
   uint64_t modr;
 
-  GNUNET_assert (0 != divisor);
+  GNUNET_assert (0 != divisor); /* division by zero is discouraged */
   *result = *dividend;
+  /* in case @a dividend was not yet normalized */
+  GNUNET_assert (GNUNET_SYSERR !=
+                 TALER_amount_normalize (result));
   if (1 == divisor)
     return;
   modr = result->value % divisor;
   result->value /= divisor;
-  /* modr is a 32-bit value, so we can safely multiply by (<32-bit) base and add fraction! */
+  /* modr fits into 32 bits, so we can safely multiply by (<32-bit) base and add fraction! */
   modr = (modr * TALER_AMOUNT_FRAC_BASE) + result->fraction;
-  GNUNET_assert (modr < TALER_AMOUNT_FRAC_BASE * divisor);
   result->fraction = (uint32_t) (modr / divisor);
+  /* 'fraction' could now be larger than #TALER_AMOUNT_FRAC_BASE, so we must normalize */
+  GNUNET_assert (GNUNET_SYSERR !=
+                 TALER_amount_normalize (result));
+}
+
+
+/**
+ * Round the amount to something that can be transferred on the wire.
+ * The rounding mode is specified via the smallest transferable unit,
+ * which must only have a fractional part *or* only a value (either
+ * of the two must be zero!).
+ *
+ * If the @a round_unit given is zero, we do nothing and return #GNUNET_NO.
+ *
+ * @param[in,out] amount amount to round down
+ * @param[in] round_unit unit that should be rounded down to, and
+ *            either value part or the faction must be zero
+ * @return #GNUNET_OK on success, #GNUNET_NO if rounding was unnecessary,
+ *         #GNUNET_SYSERR if the amount or currency or @a round_unit was invalid
+ */
+int
+TALER_amount_round_down (struct TALER_Amount *amount,
+                         const struct TALER_Amount *round_unit)
+{
+  if (GNUNET_OK !=
+      TALER_amount_cmp_currency (amount,
+                                 round_unit))
+  {
+    GNUNET_break (0);
+    return GNUNET_SYSERR;
+  }
+  if ( (0 != round_unit->fraction) &&
+       (0 != round_unit->value) )
+  {
+    GNUNET_break (0);
+    return GNUNET_SYSERR;
+  }
+  if ( (0 == round_unit->fraction) &&
+       (0 == round_unit->value) )
+    return GNUNET_NO; /* no rounding requested */
+  if (0 != round_unit->fraction)
+  {
+    uint32_t delta;
+
+    delta = amount->fraction % round_unit->fraction;
+    if (0 == delta)
+      return GNUNET_NO;
+    amount->fraction -= delta;
+  }
+  if (0 != round_unit->value)
+  {
+    uint64_t delta;
+
+    delta = amount->value % round_unit->value;
+    if (0 == delta)
+      return GNUNET_NO;
+    amount->value -= delta;
+    amount->fraction = 0;
+  }
+  return GNUNET_OK;
 }
 
 
