@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2015 GNUnet e.V.
+  (C) 2015-2020 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -45,7 +45,8 @@ int
 main (int argc,
       const char *const argv[])
 {
-
+  (void) argc;
+  (void) argv;
   cf (TALER_urlencode (""), "");
   cf (TALER_urlencode ("abc"), "abc");
   cf (TALER_urlencode ("~~"), "~~");
